@@ -24,18 +24,17 @@
 //! _IRR_LINUX_PLATFORM_ for Linux (it is defined here if no other os is defined)
 //! _IRR_SOLARIS_PLATFORM_ for Solaris
 //! _IRR_OSX_PLATFORM_ for Apple systems running OSX
+//! _IRR_IPHONE_PLATFORM_ for Apple devices running iOS
+//! _IRR_ANDROID_PLATFORM_ for devices running Android
 //! _IRR_POSIX_API_ for Posix compatible systems
 //! Note: PLATFORM defines the OS specific layer, API can group several platforms
 
 //! DEVICE is the windowing system used, several PLATFORMs support more than one DEVICE
 //! Irrlicht can be compiled with more than one device
 //! _IRR_COMPILE_WITH_WINDOWS_DEVICE_ for Windows API based device
-//! _IRR_COMPILE_WITH_WINDOWS_CE_DEVICE_ for Windows CE API based device
 //! _IRR_COMPILE_WITH_OSX_DEVICE_ for Cocoa native windowing on OSX
 //! _IRR_COMPILE_WITH_X11_DEVICE_ for Linux X11 based device
 //! _IRR_COMPILE_WITH_SDL_DEVICE_ for platform independent SDL framework
-//! _IRR_COMPILE_WITH_CONSOLE_DEVICE_ for no windowing system, used as a fallback
-//! _IRR_COMPILE_WITH_FB_DEVICE_ for framebuffer systems
 
 //! Passing defines to the compiler which have NO in front of the _IRR definename is an alternative
 //! way which can be used to disable defines (instead of outcommenting them in this header).
@@ -50,11 +49,6 @@
 #undef _IRR_COMPILE_WITH_SDL_DEVICE_
 #endif
 
-//! Comment this line to compile without the fallback console device.
-#define _IRR_COMPILE_WITH_CONSOLE_DEVICE_
-#ifdef NO_IRR_COMPILE_WITH_CONSOLE_DEVICE_
-#undef _IRR_COMPILE_WITH_CONSOLE_DEVICE_
-#endif
 
 //! WIN32 for Windows32
 //! WIN64 for Windows64
@@ -65,13 +59,6 @@
 #define _IRR_COMPILE_WITH_WINDOWS_DEVICE_
 #endif
 
-//! WINCE is a very restricted environment for mobile devices
-#if defined(_WIN32_WCE)
-#define _IRR_WINDOWS_
-#define _IRR_WINDOWS_API_
-#define _IRR_WINDOWS_CE_PLATFORM_
-#define _IRR_COMPILE_WITH_WINDOWS_CE_DEVICE_
-#endif
 
 #if defined(_MSC_VER) && (_MSC_VER < 1300)
 #  error "Only Microsoft Visual Studio 7.0 and later are supported."
@@ -93,20 +80,48 @@
 #if !defined(MACOSX)
 #define MACOSX // legacy support
 #endif
-#define _IRR_OSX_PLATFORM_
+#define _IRR_OSX_PLATFORM_ // we only support OSX on these systems
+
+#if defined(__ENVIRONMENT_IPHONE_OS_VERSION_MIN_REQUIRED__) || defined(__IPHONE_OS_VERSION_MIN_REQUIRED)
+#define _IRR_IPHONE_PLATFORM_
+#define _IRR_COMPILE_WITH_IPHONE_DEVICE_
+#define _IRR_COMPILE_WITH_OGLES2_
+#else
 #define _IRR_COMPILE_WITH_OSX_DEVICE_
 #endif
-
-#if !defined(_IRR_WINDOWS_API_) && !defined(_IRR_OSX_PLATFORM_)
-#ifndef _IRR_SOLARIS_PLATFORM_
-#if !defined(__linux__) && !defined(__FreeBSD__)
-#define _IRR_LINUX_PLATFORM_
 #endif
+
+#if defined(ANDROID)
+#define _IRR_ANDROID_PLATFORM_
+#define _IRR_POSIX_API_
+#endif
+
+#if defined(_IRR_ANDROID_PLATFORM_)
+#define _IRR_COMPILE_WITH_ANDROID_DEVICE_
+#define _IRR_COMPILE_WITH_OGLES2_
+#define _IRR_COMPILE_ANDROID_ASSET_READER_
+#endif
+
+#if defined(_IRR_COMPILE_WITH_OGLES2_) && !defined(_IRR_COMPILE_WITH_IPHONE_DEVICE_)
+#define _IRR_COMPILE_WITH_EGL_
+#endif
+
+#if !defined(_IRR_WINDOWS_API_) && !defined(_IRR_OSX_PLATFORM_) && !defined(_IRR_ANDROID_PLATFORM_)
+#ifndef _IRR_SOLARIS_PLATFORM_
+#define _IRR_LINUX_PLATFORM_
 #endif
 #define _IRR_POSIX_API_
 #define _IRR_COMPILE_WITH_X11_DEVICE_
+//#define _IRR_COMPILE_WITH_WAYLAND_DEVICE_
 #endif
 
+#ifdef NO_IRR_COMPILE_WITH_WAYLAND_DEVICE_
+#undef _IRR_COMPILE_WITH_WAYLAND_DEVICE_
+#endif
+
+#ifdef _IRR_COMPILE_WITH_WAYLAND_DEVICE_
+#define _IRR_COMPILE_WITH_EGL_
+#endif
 
 //! Define _IRR_COMPILE_WITH_JOYSTICK_SUPPORT_ if you want joystick events.
 #define _IRR_COMPILE_WITH_JOYSTICK_EVENTS_
@@ -138,7 +153,7 @@ headers, e.g. Summer 2004.  This is a Microsoft issue, not an Irrlicht one.
 //! Define _IRR_COMPILE_WITH_DIRECTINPUT_JOYSTICK_ if you want to use DirectInput for joystick handling.
 /** This only applies to Windows devices, currently only supported under Win32 device.
 If not defined, Windows Multimedia library is used, which offers also broad support for joystick devices. */
-#undef _IRR_COMPILE_WITH_DIRECTINPUT_JOYSTICK_
+#define _IRR_COMPILE_WITH_DIRECTINPUT_JOYSTICK_
 #ifdef NO_IRR_COMPILE_WITH_DIRECTINPUT_JOYSTICK_
 #undef _IRR_COMPILE_WITH_DIRECTINPUT_JOYSTICK_
 #endif
@@ -152,11 +167,34 @@ If not defined, Windows Multimedia library is used, which offers also broad supp
 //! Define _IRR_COMPILE_WITH_OPENGL_ to compile the Irrlicht engine with OpenGL.
 /** If you do not wish the engine to be compiled with OpenGL, comment this
 define out. */
+#if !defined(_IRR_IPHONE_PLATFORM_) && !defined(_IRR_ANDROID_PLATFORM_)
 #define _IRR_COMPILE_WITH_OPENGL_
+#endif
 #ifdef NO_IRR_COMPILE_WITH_OPENGL_
 #undef _IRR_COMPILE_WITH_OPENGL_
 #endif
 
+//! Define _IRR_COMPILE_WITH_OGLES2_ to compile the Irrlicht engine with OpenGL-ES 2.x.
+/** If you do not wish the engine to be compiled with OpenGL-ES 2.x, comment
+ this define out.
+ You should only use this define if you really need the OpenGL-ES driver, and
+ it should be usually the only HW accelerated one. OpenGL is currently disabled
+ if using this driver, to avoid problems with the ogl-es emulators.
+ */
+// #define _IRR_COMPILE_WITH_OGLES2_
+#ifdef NO_IRR_COMPILE_WITH_OGLES2_
+#undef _IRR_COMPILE_WITH_OGLES2_
+#endif
+#ifndef IRR_OGLES2_SHADER_PATH
+#ifdef _IRR_COMPILE_WITH_IPHONE_DEVICE_
+#define IRR_OGLES2_SHADER_PATH ""
+#elif defined(_IRR_COMPILE_WITH_ANDROID_DEVICE_)
+#define IRR_OGLES2_SHADER_PATH "media/Shaders/"
+#else
+#define IRR_OGLES2_SHADER_PATH "data/shaders/irrlicht/"
+#endif
+#endif
+
 //! Define _IRR_COMPILE_WITH_X11_ to compile the Irrlicht engine with X11 support.
 /** If you do not wish the engine to be compiled with X11, comment this
 define out. */
@@ -169,16 +207,28 @@ define out. */
 //! Define _IRR_OPENGL_USE_EXTPOINTER_ if the OpenGL renderer should use OpenGL extensions via function pointers.
 /** On some systems there is no support for the dynamic extension of OpenGL
 	via function pointers such that this has to be undef'ed. */
+#ifdef _IRR_COMPILE_WITH_OPENGL_
 #if !defined(_IRR_OSX_PLATFORM_) && !defined(_IRR_SOLARIS_PLATFORM_)
 #define _IRR_OPENGL_USE_EXTPOINTER_
 #endif
+#endif
+
+//! Define _IRR_OGLES2_USE_EXTPOINTER_ if the OpenGL-ES 2.x driver should use extensions via function pointers.
+/** This should usually be enabled, but also depends on the specific
+ architecture. You can simply uncomment the define and recompile.
+ */
+#ifdef _IRR_COMPILE_WITH_OGLES2_
+#if !defined(_IRR_IPHONE_PLATFORM_)
+#define _IRR_OGLES2_USE_EXTPOINTER_
+#endif
+#endif
 
 //! On some Linux systems the XF86 vidmode extension or X11 RandR are missing. Use these flags
 //! to remove the dependencies such that Irrlicht will compile on those systems, too.
 //! If you don't need colored cursors you can also disable the Xcursor extension
 #if defined(_IRR_LINUX_PLATFORM_) && defined(_IRR_COMPILE_WITH_X11_)
-#define _IRR_LINUX_X11_VIDMODE_
-//#define _IRR_LINUX_X11_RANDR_
+//#define _IRR_LINUX_X11_VIDMODE_
+#define _IRR_LINUX_X11_RANDR_
 #ifdef NO_IRR_LINUX_X11_VIDMODE_
 #undef _IRR_LINUX_X11_VIDMODE_
 #endif
@@ -193,6 +243,10 @@ define out. */
 #ifdef NO_IRR_LINUX_XCURSOR_
 #undef _IRR_LINUX_XCURSOR_
 #endif
+#else
+
+#undef _IRR_LINUX_X11_VIDMODE_
+#undef _IRR_LINUX_X11_RANDR_
 
 #endif
 
@@ -252,7 +306,7 @@ tool <http://developer.nvidia.com/object/nvperfhud_home.html>. */
 #undef _IRR_USE_NVIDIA_PERFHUD_
 
 //! Uncomment the following line if you want to ignore the deprecated warnings
-//#define IGNORE_DEPRECATED_WARNING
+#define IGNORE_DEPRECATED_WARNING
 
 //! Define _IRR_COMPILE_WITH_SKINNED_MESH_SUPPORT_ if you want to use bone based
 /** animated meshes. If you compile without this, you will be unable to load
