@@ -274,12 +274,12 @@ bool CGUIEditBox::processKey(const SEvent& event)
 
 		switch(event.KeyInput.Key)
 		{
-		case KEY_KEY_A:
+		case IRR_KEY_A:
 			// select all
 			newMarkBegin = 0;
 			newMarkEnd = Text.size();
 			break;
-		case KEY_KEY_C:
+		case IRR_KEY_C:
 			// copy to clipboard
 			if (!PasswordBox && Operator && MarkBegin != MarkEnd)
 			{
@@ -295,7 +295,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 				Operator->copyToClipboard(s.c_str());
 			}
 			break;
-		case KEY_KEY_X:
+		case IRR_KEY_X:
 			// cut to the clipboard
 			if (!PasswordBox && Operator && MarkBegin != MarkEnd)
 			{
@@ -326,7 +326,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 				}
 			}
 			break;
-		case KEY_KEY_V:
+		case IRR_KEY_V:
 			if ( !isEnabled() )
 				break;
 
@@ -380,7 +380,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 				textChanged = true;
 			}
 			break;
-		case KEY_HOME:
+		case IRR_KEY_HOME:
 			// move/highlight to start of text
 			if (event.KeyInput.Shift)
 			{
@@ -395,7 +395,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 				newMarkEnd = 0;
 			}
 			break;
-		case KEY_END:
+		case IRR_KEY_END:
 			// move/highlight to end of text
 			if (event.KeyInput.Shift)
 			{
@@ -418,7 +418,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 	else
 	switch(event.KeyInput.Key)
 	{
-	case KEY_END:
+	case IRR_KEY_END:
 		{
 			s32 p = Text.size();
 			if (WordWrap || MultiLine)
@@ -445,7 +445,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 			BlinkStartTime = os::Timer::getTime();
 		}
 		break;
-	case KEY_HOME:
+	case IRR_KEY_HOME:
 		{
 
 			s32 p = 0;
@@ -470,7 +470,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 			BlinkStartTime = os::Timer::getTime();
 		}
 		break;
-	case KEY_RETURN:
+	case IRR_KEY_RETURN:
 		if (MultiLine)
 		{
 			inputChar(L'\n');
@@ -481,7 +481,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 			sendGuiEvent( EGET_EDITBOX_ENTER );
 		}
 		return true;
-	case KEY_LEFT:
+	case IRR_KEY_LEFT:
 
 		if (event.KeyInput.Shift)
 		{
@@ -503,7 +503,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 		BlinkStartTime = os::Timer::getTime();
 		break;
 
-	case KEY_RIGHT:
+	case IRR_KEY_RIGHT:
 		if (event.KeyInput.Shift)
 		{
 			if (Text.size() > (u32)CursorPos)
@@ -523,7 +523,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 		if (Text.size() > (u32)CursorPos) CursorPos++;
 		BlinkStartTime = os::Timer::getTime();
 		break;
-	case KEY_UP:
+	case IRR_KEY_UP:
 		if (MultiLine || (WordWrap && BrokenText.size() > 1) )
 		{
 			s32 lineNo = getLineFromPos(CursorPos);
@@ -554,7 +554,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 			return false;
 		}
 		break;
-	case KEY_DOWN:
+	case IRR_KEY_DOWN:
 		if (MultiLine || (WordWrap && BrokenText.size() > 1) )
 		{
 			s32 lineNo = getLineFromPos(CursorPos);
@@ -586,7 +586,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 		}
 		break;
 
-	case KEY_BACK:
+	case IRR_KEY_BACK:
 		if ( !isEnabled() )
 			break;
 
@@ -626,7 +626,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 			textChanged = true;
 		}
 		break;
-	case KEY_DELETE:
+	case IRR_KEY_DELETE:
 		if ( !isEnabled() )
 			break;
 
@@ -664,33 +664,33 @@ bool CGUIEditBox::processKey(const SEvent& event)
 		}
 		break;
 
-	case KEY_ESCAPE:
-	case KEY_TAB:
-	case KEY_SHIFT:
-	case KEY_F1:
-	case KEY_F2:
-	case KEY_F3:
-	case KEY_F4:
-	case KEY_F5:
-	case KEY_F6:
-	case KEY_F7:
-	case KEY_F8:
-	case KEY_F9:
-	case KEY_F10:
-	case KEY_F11:
-	case KEY_F12:
-	case KEY_F13:
-	case KEY_F14:
-	case KEY_F15:
-	case KEY_F16:
-	case KEY_F17:
-	case KEY_F18:
-	case KEY_F19:
-	case KEY_F20:
-	case KEY_F21:
-	case KEY_F22:
-	case KEY_F23:
-	case KEY_F24:
+	case IRR_KEY_ESCAPE:
+	case IRR_KEY_TAB:
+	case IRR_KEY_SHIFT:
+	case IRR_KEY_F1:
+	case IRR_KEY_F2:
+	case IRR_KEY_F3:
+	case IRR_KEY_F4:
+	case IRR_KEY_F5:
+	case IRR_KEY_F6:
+	case IRR_KEY_F7:
+	case IRR_KEY_F8:
+	case IRR_KEY_F9:
+	case IRR_KEY_F10:
+	case IRR_KEY_F11:
+	case IRR_KEY_F12:
+	case IRR_KEY_F13:
+	case IRR_KEY_F14:
+	case IRR_KEY_F15:
+	case IRR_KEY_F16:
+	case IRR_KEY_F17:
+	case IRR_KEY_F18:
+	case IRR_KEY_F19:
+	case IRR_KEY_F20:
+	case IRR_KEY_F21:
+	case IRR_KEY_F22:
+	case IRR_KEY_F23:
+	case IRR_KEY_F24:
 		// ignore these keys
 		return false;
 
