@@ -40,7 +40,6 @@
 #include "IWriteFile.h"
 #include "IXMLWriter.h"
 
-#include "BuiltInFont.h"
 #include "os.h"
 
 namespace irr
@@ -78,8 +77,6 @@ CGUIEnvironment::CGUIEnvironment(io::IFileSystem* fs, video::IVideoDriver* drive
 	registerGUIElementFactory(factory);
 	factory->drop();
 
-	loadBuiltInFont();
-
 	IGUISkin* skin = createSkin( gui::EGST_WINDOWS_METALLIC );
 	setSkin(skin);
 	skin->drop();
@@ -165,29 +162,6 @@ CGUIEnvironment::~CGUIEnvironment()
 	}
 }
 
-
-void CGUIEnvironment::loadBuiltInFont()
-{
-	io::IReadFile* file = io::createMemoryReadFile(BuiltInFontData, BuiltInFontDataSize, DefaultFontName, false);
-
-	CGUIFont* font = new CGUIFont(this, DefaultFontName );
-	if (!font->load(file))
-	{
-		os::Printer::log("Error: Could not load built-in Font. Did you compile without the BMP loader?", ELL_ERROR);
-		font->drop();
-		file->drop();
-		return;
-	}
-
-	SFont f;
-	f.NamedPath.setPath(DefaultFontName);
-	f.Font = font;
-	Fonts.push_back(f);
-
-	file->drop();
-}
-
-
 //! draws all gui elements
 void CGUIEnvironment::drawAll()
 {
@@ -587,7 +561,7 @@ bool CGUIEnvironment::postEventFromUser(const SEvent& event)
 			// Send focus changing event
 			if (event.EventType == EET_KEY_INPUT_EVENT &&
 				event.KeyInput.PressedDown &&
-				event.KeyInput.Key == KEY_TAB)
+				event.KeyInput.Key == IRR_KEY_TAB)
 			{
 				IGUIElement *next = getNextElement(event.KeyInput.Shift, event.KeyInput.Control);
 				if (next && next != Focus)
