@@ -13,6 +13,7 @@ namespace irr
 {
 	class CIrrDeviceWin32;
 	class CIrrDeviceLinux;
+	class CIrrDeviceWayland;
 	class CIrrDeviceSDL;
 	class CIrrDeviceMacOSX;
 }
@@ -44,6 +45,13 @@ namespace video
 		bool changeRenderContext(const SExposedVideoData& videoData, CIrrDeviceWin32* device);
 		#endif
 
+		#ifdef _IRR_COMPILE_WITH_WAYLAND_DEVICE_
+		COpenGLDriver(const SIrrlichtCreationParameters& params, io::IFileSystem* io, CIrrDeviceWayland* device);
+		//! inits the EGL specific parts of the open gl driver
+		bool initDriver(CIrrDeviceWayland* device);
+		bool changeRenderContext(const SExposedVideoData& videoData, CIrrDeviceWayland* device);
+		#endif
+
 		#ifdef _IRR_COMPILE_WITH_X11_DEVICE_
 		COpenGLDriver(const SIrrlichtCreationParameters& params, io::IFileSystem* io, CIrrDeviceLinux* device);
 		//! inits the GLX specific parts of the open gl driver
@@ -587,6 +595,9 @@ namespace video
 			Display* X11Display;
 			CIrrDeviceLinux *X11Device;
 		#endif
+		#ifdef _IRR_COMPILE_WITH_WAYLAND_DEVICE_
+			CIrrDeviceWayland *wl_device;
+		#endif
 		#ifdef _IRR_COMPILE_WITH_OSX_DEVICE_
 			CIrrDeviceMacOSX *OSXDevice;
 		#endif
