@@ -22,8 +22,8 @@
 #include "MacOSX/CIrrDeviceMacOSX.h"
 #endif
 
-#ifdef _IRR_COMPILE_WITH_WINDOWS_CE_DEVICE_
-#include "CIrrDeviceWinCE.h"
+#ifdef _IRR_COMPILE_WITH_WAYLAND_DEVICE_
+#include "CIrrDeviceWayland.h"
 #endif
 
 #ifdef _IRR_COMPILE_WITH_X11_DEVICE_
@@ -34,13 +34,12 @@
 #include "CIrrDeviceSDL.h"
 #endif
 
-#ifdef _IRR_COMPILE_WITH_FB_DEVICE_
-#include "CIrrDeviceFB.h"
+#ifdef _IRR_COMPILE_WITH_ANDROID_DEVICE_
+#include "CIrrDeviceAndroid.h"
+#include <android/log.h>
 #endif
 
-#ifdef _IRR_COMPILE_WITH_CONSOLE_DEVICE_
-#include "CIrrDeviceConsole.h"
-#endif
+#include <stdio.h>
 
 namespace irr
 {
@@ -63,48 +62,96 @@ namespace irr
 
 		return createDeviceEx(p);
 	}
+	
+	static void overrideDeviceType(E_DEVICE_TYPE& device_type)
+	{
+		const char* irr_device_type = getenv("IRR_DEVICE_TYPE");
+		
+		if (irr_device_type == NULL)
+			return;
+
+#ifdef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
+		if (strcmp(irr_device_type, "win32") == 0)
+		{
+			device_type = EIDT_WIN32;
+		}
+#endif
+#ifdef _IRR_COMPILE_WITH_OSX_DEVICE_
+		if (strcmp(irr_device_type, "osx") == 0)
+		{
+			device_type = EIDT_OSX;
+		}
+#endif
+#ifdef _IRR_COMPILE_WITH_WAYLAND_DEVICE_
+		if (strcmp(irr_device_type, "wayland") == 0)
+		{
+			device_type = EIDT_WAYLAND;
+		}
+#endif
+#ifdef _IRR_COMPILE_WITH_X11_DEVICE_
+		if (strcmp(irr_device_type, "x11") == 0)
+		{
+			device_type = EIDT_X11;
+		}
+#endif
+#ifdef _IRR_COMPILE_WITH_SDL_DEVICE_
+		if (strcmp(irr_device_type, "sdl") == 0)
+		{
+			device_type = EIDT_SDL;
+		}
+#endif
+#ifdef _IRR_COMPILE_WITH_ANDROID_DEVICE_
+		if (strcmp(irr_device_type, "android") == 0)
+		{
+			device_type = EIDT_ANDROID;
+		}
+#endif
+	}
 
 	extern "C" IRRLICHT_API IrrlichtDevice* IRRCALLCONV createDeviceEx(const SIrrlichtCreationParameters& params)
 	{
 
 		IrrlichtDevice* dev = 0;
+		
+		SIrrlichtCreationParameters creation_params = params;
+		overrideDeviceType(creation_params.DeviceType);
 
 #ifdef _IRR_COMPILE_WITH_WINDOWS_DEVICE_
-		if (params.DeviceType == EIDT_WIN32 || (!dev && params.DeviceType == EIDT_BEST))
-			dev = new CIrrDeviceWin32(params);
+		if (creation_params.DeviceType == EIDT_WIN32 || (!dev && creation_params.DeviceType == EIDT_BEST))
+			dev = new CIrrDeviceWin32(creation_params);
 #endif
 
 #ifdef _IRR_COMPILE_WITH_OSX_DEVICE_
-		if (params.DeviceType == EIDT_OSX || (!dev && params.DeviceType == EIDT_BEST))
-			dev = new CIrrDeviceMacOSX(params);
+		if (creation_params.DeviceType == EIDT_OSX || (!dev && creation_params.DeviceType == EIDT_BEST))
+			dev = new CIrrDeviceMacOSX(creation_params);
 #endif
 
-#ifdef _IRR_COMPILE_WITH_WINDOWS_CE_DEVICE_
-		if (params.DeviceType == EIDT_WINCE || (!dev && params.DeviceType == EIDT_BEST))
-			dev = new CIrrDeviceWinCE(params);
+#ifdef _IRR_COMPILE_WITH_WAYLAND_DEVICE_
+		if (creation_params.DeviceType == EIDT_WAYLAND || (!dev && creation_params.DeviceType == EIDT_BEST))
+		{
+			if (CIrrDeviceWayland::isWaylandDeviceWorking())
+			{
+				dev = new CIrrDeviceWayland(creation_params);
+			}
+		}
 #endif
 
 #ifdef _IRR_COMPILE_WITH_X11_DEVICE_
-		if (params.DeviceType == EIDT_X11 || (!dev && params.DeviceType == EIDT_BEST))
-			dev = new CIrrDeviceLinux(params);
+		if (creation_params.DeviceType == EIDT_X11 || (!dev && creation_params.DeviceType == EIDT_BEST))
+			dev = new CIrrDeviceLinux(creation_params);
 #endif
 
 #ifdef _IRR_COMPILE_WITH_SDL_DEVICE_
-		if (params.DeviceType == EIDT_SDL || (!dev && params.DeviceType == EIDT_BEST))
-			dev = new CIrrDeviceSDL(params);
+		if (creation_params.DeviceType == EIDT_SDL || (!dev && creation_params.DeviceType == EIDT_BEST))
+			dev = new CIrrDeviceSDL(creation_params);
 #endif
 
-#ifdef _IRR_COMPILE_WITH_FB_DEVICE_
-		if (params.DeviceType == EIDT_FRAMEBUFFER || (!dev && params.DeviceType == EIDT_BEST))
-			dev = new CIrrDeviceFB(params);
+#ifdef _IRR_COMPILE_WITH_ANDROID_DEVICE_
+		if (creation_params.DeviceType == EIDT_ANDROID || (!dev && creation_params.DeviceType == EIDT_BEST))
+			dev = new CIrrDeviceAndroid(creation_params);
 #endif
 
-#ifdef _IRR_COMPILE_WITH_CONSOLE_DEVICE_
-		if (params.DeviceType == EIDT_CONSOLE || (!dev && params.DeviceType == EIDT_BEST))
-			dev = new CIrrDeviceConsole(params);
-#endif
-
-		if (dev && !dev->getVideoDriver() && params.DriverType != video::EDT_NULL)
+		if (dev && !dev->getVideoDriver() && creation_params.DriverType != video::EDT_NULL)
 		{
 			dev->closeDevice(); // destroy window
 			dev->run(); // consume quit message
