@@ -40,6 +40,12 @@ NewsManager::NewsManager() : m_news(std::vector<NewsMessage>())
     m_current_news_message = -1;
     m_error_message.setAtomic("");
     m_force_refresh = false;
+
+    // Clean .part file which may be left behind
+    std::string news_part = file_manager->getAddonsFile("news.xml.part");
+    if (file_manager->fileExists(news_part))
+        file_manager->removeFile(news_part);
+
     init(false);
 }   // NewsManage
 
@@ -69,9 +75,6 @@ void NewsManager::init(bool force_refresh)
         pthread_attr_t  attr;
         pthread_attr_init(&attr);
         pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-        // Should be the default, but just in case:
-        pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
-        //pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
         pthread_t thread_id;
         int error = pthread_create(&thread_id, &attr,
             &NewsManager::downloadNews, this);
@@ -99,6 +102,8 @@ void* NewsManager::downloadNews(void *obj)
     me->clearErrorMessage();
 
     std::string xml_file = file_manager->getAddonsFile("news.xml");
+    // Prevent downloading when .part file created, which is already downloaded
+    std::string xml_file_part = file_manager->getAddonsFile("news.xml.part");
     bool news_exists = file_manager->fileExists(xml_file);
 
     // The news message must be updated if either it has never been updated,
@@ -111,8 +116,8 @@ void* NewsManager::downloadNews(void *obj)
                         < StkTime::getTimeSinceEpoch()          ||
                       me->m_force_refresh                       ||
                       !news_exists                                    )
-         && UserConfigParams::m_internet_status==RequestManager::IPERM_ALLOWED;
-
+         && UserConfigParams::m_internet_status==RequestManager::IPERM_ALLOWED
+         && !file_manager->fileExists(xml_file_part);
     const XMLNode *xml = NULL;
 
     if(!download && news_exists)
@@ -314,7 +319,7 @@ void NewsManager::updateNews(const XMLNode *xml, const std::string &filename)
         // a new read on the next start, instead of waiting
         // for some time).
         file_manager->removeFile(filename);
-        NewsMessage n(_("Can't access stkaddons server..."), -1);
+        NewsMessage n(_("Failed to connect to the SuperTuxKart add-ons server."), -1);
         m_news.lock();
         m_news.getData().push_back(n);
 
