@@ -64,9 +64,9 @@ public:
     virtual void       setLoop(bool status)                 = 0;
     virtual void       reallySetLoop(bool status)           = 0;
     virtual void       play()                               = 0;
-    virtual void       reallyPlayNow()                      = 0;
-    virtual void       play(const Vec3 &xyz)                = 0;
-    virtual void       reallyPlayNow(const Vec3 &xyz)       = 0;
+    virtual void       reallyPlayNow(SFXBuffer* buffer = NULL) = 0;
+    virtual void       play(const Vec3 &xyz, SFXBuffer* buffer = NULL) = 0;
+    virtual void       reallyPlayNow(const Vec3 &xyz, SFXBuffer* buffer = NULL) = 0;
     virtual void       stop()                               = 0;
     virtual void       reallyStopNow()                      = 0;
     virtual void       pause()                              = 0;
