@@ -251,23 +251,30 @@ ChallengeData::ChallengeData(const std::string& filename)
             throw std::runtime_error("Unknown unlock entry");
         }
     }
-
-    core::stringw description;
-    if (track_node != NULL && m_minor!=RaceManager::MINOR_MODE_FOLLOW_LEADER)
-    {
-        //I18N: number of laps to race in a challenge
-        description += _("Laps : %i", m_num_laps);
-        description += core::stringw(L"\n");
-    }
-    else if (track_node)
-    {
-        // Follow the leader mode:
-        description = _("Follow the leader");
-    }
-
-    m_challenge_description = description;
 }   // ChallengeData
 
+// ----------------------------------------------------------------------------
+
+const irr::core::stringw ChallengeData::getChallengeDescription() const
+{
+    core::stringw description;
+    if (!m_track_id.empty())
+    {
+        if (m_minor != RaceManager::MINOR_MODE_FOLLOW_LEADER)
+        {
+            //I18N: number of laps to race in a challenge
+            description += _("Laps: %i", m_num_laps);
+            description += core::stringw(L"\n");
+        }
+        else
+        {
+            // Follow the leader mode:
+            description = _("Follow the leader");
+        }
+    }
+    return description;
+}   // getChallengeDescription
+
 // ----------------------------------------------------------------------------
 void ChallengeData::error(const char *id) const
 {
@@ -385,7 +392,11 @@ void ChallengeData::setRace(RaceManager::Difficulty d) const
 
         if (m_time[d] >= 0.0f)
         {
-          race_manager->setTimeTarget(m_time[d]);
+            race_manager->setTimeTarget(m_time[d]);
+        }
+        else
+        {
+            race_manager->setTimeTarget(0.0f);
         }
     }
     else if(m_mode==CM_GRAND_PRIX)
@@ -429,7 +440,7 @@ bool ChallengeData::isChallengeFulfilled() const
     // Single races
     // ------------
     World *world = World::getWorld();
-    std::string track_name = world->getTrack()->getIdent();
+    std::string track_name = Track::getCurrentTrack()->getIdent();
 
     int d = race_manager->getDifficulty();
 
@@ -509,7 +520,6 @@ const irr::core::stringw
             if (track == NULL) return irr::core::stringw( L"????" );
 
             return _("New track '%s' now available", track->getName());
-            break;
         }
         case UNLOCK_MODE:
         {
