@@ -15,28 +15,27 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#include "2dutils.hpp"
+#ifndef SERVER_ONLY
+#include "graphics/2dutils.hpp"
 
 #include "graphics/central_settings.hpp"
+#include "graphics/glwrap.hpp"
+#include "graphics/irr_driver.hpp"
 #include "graphics/shader.hpp"
 #include "graphics/shaders.hpp"
 #include "graphics/shared_gpu_objects.hpp"
 #include "graphics/texture_shader.hpp"
-#include "glwrap.hpp"
 #include "utils/cpp2011.hpp"
 
-#include "../../lib/irrlicht/source/Irrlicht/COpenGLTexture.h"
-
-
 // ============================================================================
-class Primitive2DList : public TextureShader<Primitive2DList, 1>
+class Primitive2DList : public TextureShader<Primitive2DList, 1, float>
 {
 public:
     Primitive2DList()
     {
         loadProgram(OBJECT, GL_VERTEX_SHADER, "primitive2dlist.vert",
                             GL_FRAGMENT_SHADER, "transparent.frag");
-        assignUniforms();
+        assignUniforms("custom_alpha");
         assignSamplerNames(0, "tex", ST_BILINEAR_FILTERED);
     }   // Primitive2DList
 };   //Primitive2DList
@@ -71,7 +70,7 @@ public:
                             GL_FRAGMENT_SHADER, "texturedquad.frag");
         assignUniforms("center", "size", "texcenter", "texsize");
 
-        assignSamplerNames(0, "tex", ST_BILINEAR_FILTERED);
+        assignSamplerNames(0, "tex", ST_BILINEAR_CLAMPED_FILTERED);
     }   // TextureRectShader
 };   // TextureRectShader
 
@@ -94,41 +93,12 @@ class ColoredTextureRectShader : public TextureShader<ColoredTextureRectShader,
                                                core::vector2df, core::vector2df,
                                                core::vector2df, core::vector2df>
 {
-#ifdef XX
-private:
-    GLuint m_quad_buffer;
-
-    void initQuadBuffer()
-    {
-        const float quad_vertex[] = { -1., -1., -1.,  1.,   // UpperLeft
-                                      -1.,  1., -1., -1.,   // LowerLeft
-                                       1., -1.,  1.,  1.,   // UpperRight
-                                       1.,  1.,  1., -1. }; // LowerRight 
-        glGenBuffers(1, &m_quad_buffer);
-        glBindBuffer(GL_ARRAY_BUFFER, m_quad_buffer);
-        glBufferData(GL_ARRAY_BUFFER, 16 * sizeof(float), quad_vertex,
-                     GL_STATIC_DRAW);
-
-        glGenVertexArrays(1, &SharedObject::UIVAO);
-        glBindVertexArray(SharedObject::UIVAO);
-        glEnableVertexAttribArray(0);
-        glEnableVertexAttribArray(3);
-        glBindBuffer(GL_ARRAY_BUFFER, m_quad_buffer);
-        glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 0);
-        glVertexAttribPointer(3, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 
-                              (GLvoid *)(2 * sizeof(float)));
-        glBindVertexArray(0);
-    }   // initQuadBuffer
-#endif
 public:
     GLuint m_color_vbo;
     GLuint m_vao;
 
     ColoredTextureRectShader()
     {
-#ifdef XX
-        initQuadBuffer();
-#endif
         loadProgram(OBJECT, GL_VERTEX_SHADER, "colortexturedquad.vert",
                             GL_FRAGMENT_SHADER, "colortexturedquad.frag");
         assignUniforms("center", "size", "texcenter", "texsize");
@@ -144,16 +114,15 @@ public:
         glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 0);
         glVertexAttribPointer(3, 2, GL_FLOAT, GL_FALSE, 4 * sizeof(float), 
                               (GLvoid *)(2 * sizeof(float)));
-        const unsigned quad_color[] = {   0,   0,   0, 255,
+        glBindVertexArray(0);
+        const uint8_t quad_color[] = {   0,   0,   0, 255,
                                         255,   0,   0, 255,
                                           0, 255,   0, 255,
                                           0,   0, 255, 255 };
         glGenBuffers(1, &m_color_vbo);
         glBindBuffer(GL_ARRAY_BUFFER, m_color_vbo);
-        glBufferData(GL_ARRAY_BUFFER, 16 * sizeof(unsigned), quad_color,
-                     GL_DYNAMIC_DRAW);
-        glVertexAttribIPointer(2, 4, GL_UNSIGNED_INT, 4 * sizeof(unsigned), 0);
-        glBindVertexArray(0);
+        glBufferData(GL_ARRAY_BUFFER, 16, quad_color, GL_DYNAMIC_DRAW);
+
     }   // ColoredTextureRectShader
 };   // ColoredTextureRectShader
 
@@ -165,24 +134,16 @@ static void drawTexColoredQuad(const video::ITexture *texture,
                                float tex_center_pos_y, float tex_width,
                                float tex_height)
 {
-    unsigned colors[] = {
-        col[0].getRed(), col[0].getGreen(), col[0].getBlue(), col[0].getAlpha(),
-        col[1].getRed(), col[1].getGreen(), col[1].getBlue(), col[1].getAlpha(),
-        col[2].getRed(), col[2].getGreen(), col[2].getBlue(), col[2].getAlpha(),
-        col[3].getRed(), col[3].getGreen(), col[3].getBlue(), col[3].getAlpha(),
-    };
-
+    glBindVertexArray(ColoredTextureRectShader::getInstance()->m_vao);
     glBindBuffer(GL_ARRAY_BUFFER,
                  ColoredTextureRectShader::getInstance()->m_color_vbo);
-    glBufferSubData(GL_ARRAY_BUFFER, 0, 16 * sizeof(unsigned), colors);
+    glBufferSubData(GL_ARRAY_BUFFER, 0, 16, col);
+    glVertexAttribPointer(2, 4, GL_UNSIGNED_BYTE, GL_TRUE, 4 , 0);
 
     ColoredTextureRectShader::getInstance()->use();
     glBindVertexArray(ColoredTextureRectShader::getInstance()->m_vao);
-
-    const irr::video::COpenGLTexture *t = 
-                       static_cast<const irr::video::COpenGLTexture*>(texture);
     ColoredTextureRectShader::getInstance()
-        ->setTextureUnits(t->getOpenGLTextureName());
+        ->setTextureUnits(texture->getOpenGLTextureName());
     ColoredTextureRectShader::getInstance()
         ->setUniforms(core::vector2df(center_pos_x, center_pos_y),
                       core::vector2df(width, height),
@@ -190,8 +151,9 @@ static void drawTexColoredQuad(const video::ITexture *texture,
                       core::vector2df(tex_width, tex_height));
 
     glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-    glBindVertexArray(0);
     glBindBuffer(GL_ARRAY_BUFFER, 0);
+    glBindVertexArray(0);
+
 
     glGetError();
 }   // drawTexColoredQuad
@@ -264,7 +226,50 @@ static void getSize(unsigned texture_width, unsigned texture_height,
 }   // getSize
 
 // ----------------------------------------------------------------------------
-void draw2DImage(const video::ITexture* texture, 
+static void getSize(unsigned texture_width, unsigned texture_height,
+                    bool textureisRTT, const core::rect<float>& destRect,
+                    const core::rect<s32>& sourceRect,
+                    float &width, float &height,
+                    float &center_pos_x, float &center_pos_y,
+                    float &tex_width, float &tex_height,
+                    float &tex_center_pos_x, float &tex_center_pos_y   )
+{
+    core::dimension2d<u32> frame_size = irr_driver->getActualScreenSize();
+    const int screen_w = frame_size.Width;
+    const int screen_h =  frame_size.Height;
+    center_pos_x = destRect.UpperLeftCorner.X + destRect.LowerRightCorner.X;
+    center_pos_x /= screen_w;
+    center_pos_x -= 1.;
+    center_pos_y = destRect.UpperLeftCorner.Y + destRect.LowerRightCorner.Y;
+    center_pos_y /= screen_h;
+    center_pos_y = float(1.f - center_pos_y);
+    width = destRect.LowerRightCorner.X - destRect.UpperLeftCorner.X;
+    width /= screen_w;
+    height = destRect.LowerRightCorner.Y - destRect.UpperLeftCorner.Y;
+    height /= screen_h;
+
+    tex_center_pos_x = float(sourceRect.UpperLeftCorner.X + sourceRect.LowerRightCorner.X);
+    tex_center_pos_x /= texture_width * 2.f;
+    tex_center_pos_y = float(sourceRect.UpperLeftCorner.Y + sourceRect.LowerRightCorner.Y);
+    tex_center_pos_y /= texture_height * 2.f;
+    tex_width = float(sourceRect.LowerRightCorner.X - sourceRect.UpperLeftCorner.X);
+    tex_width /= texture_width * 2.f;
+    tex_height = float(sourceRect.LowerRightCorner.Y - sourceRect.UpperLeftCorner.Y);
+    tex_height /= texture_height * 2.f;
+
+    if (textureisRTT)
+        tex_height = -tex_height;
+
+    const f32 invW = 1.f / static_cast<f32>(texture_width);
+    const f32 invH = 1.f / static_cast<f32>(texture_height);
+    const core::rect<f32> tcoords(sourceRect.UpperLeftCorner.X  * invW,
+                                  sourceRect.UpperLeftCorner.Y  * invH,
+                                  sourceRect.LowerRightCorner.X * invW,
+                                  sourceRect.LowerRightCorner.Y * invH);
+}   // getSize
+
+// ----------------------------------------------------------------------------
+void draw2DImage(const video::ITexture* texture,
                  const core::rect<s32>& destRect,
                  const core::rect<s32>& sourceRect,
                  const core::rect<s32>* clip_rect,
@@ -281,7 +286,82 @@ void draw2DImage(const video::ITexture* texture,
     float width, height, center_pos_x, center_pos_y;
     float tex_width, tex_height, tex_center_pos_x, tex_center_pos_y;
 
-    getSize(texture->getSize().Width, texture->getSize().Height, 
+    getSize(texture->getSize().Width, texture->getSize().Height,
+            texture->isRenderTarget(), destRect, sourceRect, width, height,
+            center_pos_x, center_pos_y, tex_width, tex_height,
+            tex_center_pos_x, tex_center_pos_y);
+
+    if (use_alpha_channel_of_texture)
+    {
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    }
+    else
+    {
+        glDisable(GL_BLEND);
+    }
+
+    if (clip_rect)
+    {
+        if (!clip_rect->isValid())
+            return;
+
+        glEnable(GL_SCISSOR_TEST);
+        const core::dimension2d<u32>& render_target_size =
+                           irr_driver->getActualScreenSize();
+        glScissor(clip_rect->UpperLeftCorner.X,
+                  render_target_size.Height - clip_rect->LowerRightCorner.Y,
+                  clip_rect->getWidth(), clip_rect->getHeight());
+    }
+
+    UniformColoredTextureRectShader::getInstance()->use();
+    glBindVertexArray(SharedGPUObjects::getUI_VAO());
+    UniformColoredTextureRectShader::getInstance()
+        ->setTextureUnits(texture->getOpenGLTextureName());
+
+    UniformColoredTextureRectShader::getInstance()
+        ->setUniforms(core::vector2df(center_pos_x, center_pos_y),
+                      core::vector2df(width, height),
+                      core::vector2df(tex_center_pos_x, tex_center_pos_y),
+                      core::vector2df(tex_width, tex_height),
+                      colors);
+
+    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+    glBindVertexArray(0);
+    glBindBuffer(GL_ARRAY_BUFFER, 0);
+    if (clip_rect)
+        glDisable(GL_SCISSOR_TEST);
+    glUseProgram(0);
+
+    glGetError();
+}   // draw2DImage
+
+// ----------------------------------------------------------------------------
+void draw2DImage(const video::ITexture* texture,
+                 const core::rect<float>& destRect,
+                 const core::rect<s32>& sourceRect,
+                 const core::rect<s32>* clip_rect,
+                 const video::SColor &colors,
+                 bool use_alpha_channel_of_texture)
+{
+    if (!CVS->isGLSL())
+    {
+        core::rect<irr::s32> dest_rect
+            (irr::s32(destRect.UpperLeftCorner.X),
+            irr::s32(destRect.UpperLeftCorner.Y),
+            irr::s32(destRect.LowerRightCorner.X),
+            irr::s32(destRect.LowerRightCorner.Y));
+
+        video::SColor duplicatedArray[4] = { colors, colors, colors, colors };
+        draw2DImage(texture, dest_rect, sourceRect, clip_rect, duplicatedArray,
+                    use_alpha_channel_of_texture);
+        return;
+    }
+
+    float width, height, center_pos_x, center_pos_y;
+    float tex_width, tex_height, tex_center_pos_x, tex_center_pos_y;
+
+    getSize(texture->getSize().Width, texture->getSize().Height,
             texture->isRenderTarget(), destRect, sourceRect, width, height,
             center_pos_x, center_pos_y, tex_width, tex_height,
             tex_center_pos_x, tex_center_pos_y);
@@ -304,18 +384,15 @@ void draw2DImage(const video::ITexture* texture,
         glEnable(GL_SCISSOR_TEST);
         const core::dimension2d<u32>& render_target_size = 
                            irr_driver->getActualScreenSize();
-        glScissor(clip_rect->UpperLeftCorner.X, 
+        glScissor(clip_rect->UpperLeftCorner.X,
                   render_target_size.Height - clip_rect->LowerRightCorner.Y,
                   clip_rect->getWidth(), clip_rect->getHeight());
     }
 
     UniformColoredTextureRectShader::getInstance()->use();
     glBindVertexArray(SharedGPUObjects::getUI_VAO());
-
-    const video::COpenGLTexture *c_texture = 
-        static_cast<const video::COpenGLTexture*>(texture);
     UniformColoredTextureRectShader::getInstance()
-        ->setTextureUnits(c_texture->getOpenGLTextureName());
+        ->setTextureUnits(texture->getOpenGLTextureName());
 
     UniformColoredTextureRectShader::getInstance()
         ->setUniforms(core::vector2df(center_pos_x, center_pos_y),
@@ -372,7 +449,7 @@ void draw2DImageFromRTT(GLuint texture, size_t texture_w, size_t texture_h,
 }   // draw2DImageFromRTT
 
 // ----------------------------------------------------------------------------
-void draw2DImage(const video::ITexture* texture, 
+void draw2DImage(const video::ITexture* texture,
                  const core::rect<s32>& destRect,
                  const core::rect<s32>& sourceRect,
                  const core::rect<s32>* clip_rect,
@@ -430,9 +507,83 @@ void draw2DImage(const video::ITexture* texture,
     }
     else
     {
-        const video::COpenGLTexture *c_texture = 
-                            static_cast<const video::COpenGLTexture*>(texture);
-        drawTexQuad(c_texture->getOpenGLTextureName(), width, height,
+        drawTexQuad(texture->getOpenGLTextureName(), width, height,
+                    center_pos_x, center_pos_y, tex_center_pos_x,
+                    tex_center_pos_y, tex_width, tex_height);
+    }
+    if (clip_rect)
+        glDisable(GL_SCISSOR_TEST);
+    glUseProgram(0);
+
+    glGetError();
+}   // draw2DImage
+
+// ----------------------------------------------------------------------------
+void draw2DImage(const video::ITexture* texture,
+                 const core::rect<float>& destRect,
+                 const core::rect<s32>& sourceRect,
+                 const core::rect<s32>* clip_rect,
+                 const video::SColor* const colors,
+                 bool use_alpha_channel_of_texture,
+                 bool draw_translucently)
+{
+    if (!CVS->isGLSL())
+    {
+        core::rect<irr::s32> dest_rect
+            (irr::s32(destRect.UpperLeftCorner.X),
+            irr::s32(destRect.UpperLeftCorner.Y),
+            irr::s32(destRect.LowerRightCorner.X),
+            irr::s32(destRect.LowerRightCorner.Y));
+
+        irr_driver->getVideoDriver()->draw2DImage(texture, dest_rect, sourceRect,
+                                                  clip_rect, colors,
+                                                  use_alpha_channel_of_texture);
+        return;
+    }
+
+    float width, height, center_pos_x, center_pos_y, tex_width, tex_height;
+    float tex_center_pos_x, tex_center_pos_y;
+
+    getSize(texture->getSize().Width, texture->getSize().Height,
+            texture->isRenderTarget(), destRect, sourceRect, width, height,
+            center_pos_x, center_pos_y, tex_width, tex_height,
+            tex_center_pos_x, tex_center_pos_y);
+
+    if (draw_translucently)
+    {
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_SRC_ALPHA, GL_ONE);
+    }
+    else if (use_alpha_channel_of_texture)
+    {
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    }
+    else
+    {
+        glDisable(GL_BLEND);
+    }
+    if (clip_rect)
+    {
+        if (!clip_rect->isValid())
+            return;
+
+        glEnable(GL_SCISSOR_TEST);
+        const core::dimension2d<u32>& render_target_size =
+                            irr_driver->getActualScreenSize();
+        glScissor(clip_rect->UpperLeftCorner.X,
+                  render_target_size.Height - clip_rect->LowerRightCorner.Y,
+                  clip_rect->getWidth(), clip_rect->getHeight());
+    }
+    if (colors)
+    {
+        drawTexColoredQuad(texture, colors, width, height, center_pos_x,
+                           center_pos_y, tex_center_pos_x, tex_center_pos_y,
+                           tex_width, tex_height);
+    }
+    else
+    {
+        drawTexQuad(texture->getOpenGLTextureName(), width, height,
                     center_pos_x, center_pos_y, tex_center_pos_x,
                     tex_center_pos_y, tex_width, tex_height);
     }
@@ -474,9 +625,8 @@ void draw2DVertexPrimitiveList(video::ITexture *tex, const void* vertices,
     VertexUtils::bindVertexArrayAttrib(vType);
 
     Primitive2DList::getInstance()->use();
-    Primitive2DList::getInstance()->setUniforms();
-    compressTexture(tex, false);
-    Primitive2DList::getInstance()->setTextureUnits(getTextureGLuint(tex));
+    Primitive2DList::getInstance()->setUniforms(1.0f);
+    Primitive2DList::getInstance()->setTextureUnits(tex->getOpenGLTextureName());
     glDrawElements(GL_TRIANGLE_FAN, primitiveCount, GL_UNSIGNED_SHORT, 0);
 
     glDeleteVertexArrays(1, &tmpvao);
@@ -548,3 +698,6 @@ void GL32_draw2DRectangle(video::SColor color, const core::rect<s32>& position,
 
     glGetError();
 }   // GL32_draw2DRectangle
+
+#endif   // !SERVER_ONLY
+
