@@ -28,7 +28,7 @@ private:
     bool hasVSLayer;
     bool hasBaseInstance;
     bool hasDrawIndirect;
-    bool hasBuffserStorage;
+    bool hasBufferStorage;
     bool hasComputeShaders;
     bool hasArraysOfArrays;
     bool hasTextureStorage;
@@ -42,10 +42,20 @@ private:
     bool hasSSBO;
     bool hasImageLoadStore;
     bool hasMultiDrawIndirect;
+    bool hasTextureFilterAnisotropic;
+    bool hasTextureSwizzle;
+    bool hasPixelBufferObject;
+    bool hasSRGBFramebuffer;
+
+#if defined(USE_GLES2)
+    bool hasBGRA;
+    bool hasColorBufferFloat;
+#endif
 
     bool m_need_rh_workaround;
     bool m_need_srgb_workaround;
     bool m_need_srgb_visual_workaround;
+    bool m_need_vertex_id_workaround;
     bool m_GI_has_artifact;
 public:
     void init();
@@ -56,6 +66,7 @@ public:
     bool needRHWorkaround() const;
     bool needsRGBBindlessWorkaround() const;
     bool needsSRGBCapableVisualWorkaround() const;
+    bool needsVertexIdWorkaround() const;
 
     // Extension is available and safe to use
     bool isARBUniformBufferObjectUsable() const;
@@ -75,6 +86,15 @@ public:
     bool isARBImageLoadStoreUsable() const;
     bool isARBMultiDrawIndirectUsable() const;
     bool isARBExplicitAttribLocationUsable() const;
+    bool isEXTTextureFilterAnisotropicUsable() const;
+    bool isARBTextureSwizzleUsable() const;
+    bool isARBPixelBufferObjectUsable() const;
+    bool isARBSRGBFramebufferUsable() const;
+
+#if defined(USE_GLES2)
+    bool isEXTTextureFormatBGRA8888Usable() const;
+    bool isEXTColorBufferFloatUsable() const;
+#endif
 
 
     // Are all required extensions available for feature support
@@ -83,6 +103,9 @@ public:
     bool supportsIndirectInstancingRendering() const;
     bool supportsComputeShadersFiltering() const;
     bool supportsAsyncInstanceUpload() const;
+    bool supportsHardwareSkinning() const;
+    bool supportsThreadedTextureLoading() const;
+    bool supportsTextureCompression() const;
 
     // "Macro" around feature support and user config
     bool isShadowEnabled() const;
