@@ -18,10 +18,14 @@
 #ifndef GL_HEADER_HPP
 #define GL_HEADER_HPP
 
+#ifndef SERVER_ONLY
+
 #define GLEW_STATIC
 
 extern "C" {
-#include <GL/glew.h>
+#if !defined(USE_GLES2)
+#   include <GL/glew.h>
+#endif
 }
 #include <cinttypes>
 
@@ -38,8 +42,12 @@ extern "C" {
 #    ifndef GL_TEXTURE_SWIZZLE_RGBA
 #        define GL_TEXTURE_SWIZZLE_RGBA 0x8E46
 #    endif
-#elif defined(ANDROID)
-#    include <GLES/gl.h>
+#elif defined(USE_GLES2)
+#    define __gl2_h_
+#    include <GLES3/gl3.h>
+#    include <GLES3/gl3ext.h>
+#    include <GLES2/gl2ext.h>
+#    define glVertexAttribDivisorARB glVertexAttribDivisor
 #elif defined(WIN32)
 #    define WIN32_LEAN_AND_MEAN
 #    include <windows.h>
@@ -50,6 +58,22 @@ extern "C" {
 #    include <GL/glext.h>
 #endif
 
+#if defined(USE_GLES2)
+#define GL_BGRA 0x80E1
+#define GL_BGR 0x80E0
+#define GL_FRAMEBUFFER_COMPLETE_EXT GL_FRAMEBUFFER_COMPLETE
+
+// The glDrawElementsBaseVertex is available only in OpenGL ES 3.2. At this
+// stage the 'basevertex' argument is always equal to 0 because features that
+// use it are disabled in OpenGL ES renderer. We can simply use glDrawElements
+// instead.
+inline void glDrawElementsBaseVertex(GLenum mode, GLsizei count, GLenum type,
+                                     GLvoid *indices, GLint basevertex)
+{
+    glDrawElements(mode, count, type, indices);
+}
+#endif
+
 struct DrawElementsIndirectCommand{
     GLuint count;
     GLuint instanceCount;
@@ -57,5 +81,12 @@ struct DrawElementsIndirectCommand{
     GLuint baseVertex;
     GLuint baseInstance;
 };
+#else
+  typedef unsigned int GLuint;
+  typedef unsigned int GLsync;
+  typedef unsigned int GLenum;
+
+#endif   // server only
 
 #endif
+
