@@ -15,18 +15,19 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+#ifndef SERVER_ONLY
+
 #ifndef GLWRAP_HEADER_H
 #define GLWRAP_HEADER_H
 
 #include "graphics/gl_headers.hpp"
 
-#include "graphics/irr_driver.hpp"
-#include "graphics/texture_manager.hpp"
-#include "graphics/vao_manager.hpp"
 #include "utils/log.hpp"
+#include "utils/leak_check.hpp"
 #include "utils/no_copy.hpp"
 #include "utils/vec3.hpp"
 
+#include <irrlicht.h>
 #include <vector>
 
 namespace HardwareStats
@@ -35,7 +36,6 @@ namespace HardwareStats
 }
 
 void initGL();
-video::ITexture* getUnicolorTexture(const video::SColor &c);
 
 class GPUTimer;
 
@@ -66,19 +66,19 @@ private:
     GLuint fbo, fbolayer;
     std::vector<GLuint> RenderTargets;
     GLuint DepthTexture;
-    size_t width, height;
+    unsigned int width, height;
 public:
     FrameBuffer();
-    FrameBuffer(const std::vector <GLuint> &RTTs, size_t w, size_t h, bool layered = false);
-    FrameBuffer(const std::vector <GLuint> &RTTs, GLuint DS, size_t w, size_t h, bool layered = false);
+    FrameBuffer(const std::vector <GLuint> &RTTs, unsigned int w, unsigned int h, bool layered = false);
+    FrameBuffer(const std::vector <GLuint> &RTTs, GLuint DS, unsigned int w, unsigned int h, bool layered = false);
     ~FrameBuffer();
     void bind() const;
-    void bindLayer(unsigned);
+    void bindLayer(unsigned) const;
     const std::vector<GLuint> &getRTT() const { return RenderTargets; }
-    GLuint &getDepthTexture() { assert(DepthTexture); return DepthTexture; }
-    size_t getWidth() const { return width; }
-    size_t getHeight() const { return height; }
-    static void Blit(const FrameBuffer &Src, FrameBuffer &Dst, GLbitfield mask = GL_COLOR_BUFFER_BIT, GLenum filter = GL_NEAREST);
+    GLuint getDepthTexture() const { assert(DepthTexture); return DepthTexture; }
+    unsigned int getWidth() const { return width; }
+    unsigned int getHeight() const { return height; }
+    static void Blit(const FrameBuffer &Src, const FrameBuffer &Dst, GLbitfield mask = GL_COLOR_BUFFER_BIT, GLenum filter = GL_NEAREST);
     void BlitToDefault(size_t, size_t, size_t, size_t);
 
     LEAK_CHECK();
@@ -142,6 +142,22 @@ public:
             glEnableVertexAttribArray(6);
             glVertexAttribPointer(6, 3, GL_FLOAT, GL_FALSE, getVertexPitchFromType(tp), (GLvoid*)48);
             break;
+        case video::EVT_SKINNED_MESH:
+            glEnableVertexAttribArray(0);
+            glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, getVertexPitchFromType(tp), 0);
+            glEnableVertexAttribArray(1);
+            glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, getVertexPitchFromType(tp), (GLvoid*)12);
+            glEnableVertexAttribArray(2);
+            glVertexAttribPointer(2, 4, GL_UNSIGNED_BYTE, GL_TRUE, getVertexPitchFromType(tp), (GLvoid*)24);
+            glEnableVertexAttribArray(3);
+            glVertexAttribPointer(3, 4, GL_FLOAT, GL_FALSE, getVertexPitchFromType(tp), (GLvoid*)28);
+            glEnableVertexAttribArray(4);
+            glVertexAttribPointer(4, 4, GL_FLOAT, GL_FALSE, getVertexPitchFromType(tp), (GLvoid*)44);
+            glEnableVertexAttribArray(5);
+            glVertexAttribIPointer(5, 4, GL_SHORT, getVertexPitchFromType(tp), (GLvoid*)60);
+            glEnableVertexAttribArray(6);
+            glVertexAttribPointer(6, 4, GL_HALF_FLOAT, GL_FALSE, getVertexPitchFromType(tp), (GLvoid*)68);
+            break;
         }
     }
 };
@@ -152,5 +168,9 @@ void draw3DLine(const core::vector3df& start,
 bool hasGLExtension(const char* extension);
 const std::string getGLExtensions();
 void getGLLimits(HardwareStats::Json *json);
+bool checkGLError();
 
 #endif
+
+#endif   // supertuxkart
+
