@@ -25,7 +25,12 @@
 #include "utils/string_utils.hpp"
 #include "utils/types.hpp"
 
+#ifdef ANDROID
+#include "main_android.hpp"
+#endif
+
 #include <algorithm>
+#include <array>
 
 namespace GraphicsRestrictions
 {
@@ -39,30 +44,41 @@ namespace GraphicsRestrictions
         /** The list of names used in the XML file for the graphics
          *  restriction types. They must be in the same order as the types. */
 
-        const char *m_names_of_restrictions[] = {
-            "UniformBufferObject",
-            "GeometryShader",
-            "DrawIndirect",
-            "TextureView",
-            "TextureStorage",
-            "ImageLoadStore",
-            "BaseInstance",
-            "ComputeShader",
-            "ArraysOfArrays",
-            "ShaderStorageBufferObject",
-            "MultiDrawIndirect",
-            "ShaderAtomicCounters",
-            "BufferStorage",
-            "BindlessTexture",
-            "TextureCompressionS3TC",
-            "AMDVertexShaderLayer",
-            "ExplicitAttribLocation",
-            "DriverRecentEnough",
-            "HighDefinitionTextures",
-            "AdvancedPipeline",
-            "FramebufferSRGBWorking",
-            "FramebufferSRGBCapable",
-            "GI",
+        std::array<std::string, 31> m_names_of_restrictions =
+        {
+            {
+                "UniformBufferObject",
+                "GeometryShader",
+                "DrawIndirect",
+                "TextureView",
+                "TextureStorage",
+                "ImageLoadStore",
+                "BaseInstance",
+                "ComputeShader",
+                "ArraysOfArrays",
+                "ShaderStorageBufferObject",
+                "MultiDrawIndirect",
+                "ShaderAtomicCounters",
+                "BufferStorage",
+                "BindlessTexture",
+                "TextureCompressionS3TC",
+                "AMDVertexShaderLayer",
+                "ExplicitAttribLocation",
+                "TextureFilterAnisotropic",
+                "TextureFormatBGRA8888",
+                "ColorBufferFloat",
+                "DriverRecentEnough",
+                "HighDefinitionTextures",
+                "HighDefinitionTextures256",
+                "AdvancedPipeline",
+                "FramebufferSRGB",
+                "FramebufferSRGBWorkaround1",
+                "FramebufferSRGBWorkaround2",
+                "GI",
+                "ForceLegacyDevice",
+                "VertexIdWorking",
+                "HardwareSkinning"
+            }
         };
     }   // namespace Private
     using namespace Private;
@@ -71,12 +87,10 @@ namespace GraphicsRestrictions
      *  GR_COUNT if the name is not found. */
 GraphicsRestrictionsType getTypeForName(const std::string &name)
 {
-    unsigned int i = 0;
-    while (m_names_of_restrictions[i] != NULL)
+    for (unsigned int i = 0; i < m_names_of_restrictions.size(); i++)
     {
         if (name == m_names_of_restrictions[i])
             return (GraphicsRestrictionsType)i;
-        i++;
     }
     return GR_COUNT;
 }   // getTypeForName
@@ -127,6 +141,18 @@ public:
     Version(const std::string &driver_version, const std::string &card_name)
     {
         m_version.clear();
+        
+#ifdef ANDROID
+        // Android version should be enough to disable certain features on this
+        // platform
+        int version = AConfiguration_getSdkVersion(global_android_app->config);
+        
+        if (version > 0)
+        {
+            m_version.push_back(version);
+            return;
+        }
+#endif
 
         // Mesa needs to be tested first, otherwise (if testing for card name
         // further down) it would be detected as a non-mesa driver.
@@ -140,12 +166,16 @@ public:
             std::vector<std::string> l = StringUtils::split(driver, ' ');
             if (l.size() > 2)
             {
-                // driver can be: "1.4 (3.0 Mesa 10.1.0)" --> l[3] must be used
-                if (l[2] != "Mesa")
-                    convertVersionString(l[2]);
-                else
-                    convertVersionString(l[3]);
-                return;
+                // driver can be: "1.4 (3.0 Mesa 10.1.0)" -->
+                // we use value next to "Mesa" word.
+                for (unsigned int i = 0; i < l.size(); i++)
+                {
+                    if (l[i] == "Mesa" && i < l.size() - 1)
+                    {
+                        convertVersionString(l[i+1]);
+                        return;
+                    }
+                }
             }
         }
 
@@ -188,7 +218,8 @@ public:
 
         // AMD: driver_version = "4.3.13283 Core Profile/Debug Context 14.501.1003.0"
         // ----------------------------------------------
-        if (card_name.find("AMD") != std::string::npos)
+        if (card_name.find("AMD") != std::string::npos
+            || card_name.find("Radeon") != std::string::npos)
         {
             std::vector<std::string> s = StringUtils::split(driver_version, ' ');
             if (s.size() == 5)
@@ -235,7 +266,7 @@ public:
     /** If *this < other. */
     bool operator< (const Version &other) const
     {
-        unsigned int min_n = std::min(m_version.size(), other.m_version.size());
+        unsigned int min_n = (unsigned int)std::min(m_version.size(), other.m_version.size());
         for (unsigned int i = 0; i<min_n; i++)
         {
             if (m_version[i] > other.m_version[i]) return false;
@@ -250,7 +281,7 @@ public:
     /** If *this <= other. */
     bool operator<= (const Version &other) const
     {
-        unsigned int min_n = std::min(m_version.size(), other.m_version.size());
+        unsigned int min_n = (unsigned int)std::min(m_version.size(), other.m_version.size());
         for (unsigned int i = 0; i<min_n; i++)
         {
             if (m_version[i] > other.m_version[i]) return false;
@@ -268,13 +299,13 @@ class Rule : public NoCopy
 {
 private:
     /** Operators to test for a card. */
-    enum {CARD_IS, CARD_CONTAINS} m_card_test;
+    enum {CARD_IGNORE, CARD_IS, CARD_CONTAINS} m_card_test;
 
     /** Name of the card for which this rule applies. */
     std::string m_card_name;
 
     /** Operators to test version numbers with. */
-    enum {VERSION_IGNORE, VERSION_EQUAL, VERSION_LESS, 
+    enum {VERSION_IGNORE, VERSION_EQUAL, VERSION_LESS,
           VERSION_LESS_EQUAL}  m_version_test;
 
     /** Driver version for which this rule applies. */
@@ -283,12 +314,17 @@ private:
     /** For which OS this rule applies. */
     std::string m_os;
 
+    /** For which vendor this rule applies. */
+    std::string m_vendor;
+
     /** Which options to disable. */
     std::vector<std::string> m_disable_options;
 public:
     Rule(const XMLNode *rule)
     {
         m_version_test = VERSION_IGNORE;
+        m_card_test = CARD_IGNORE;
+
         if(rule->get("is", &m_card_name))
         {
             m_card_test = CARD_IS;
@@ -299,6 +335,7 @@ public:
         }
 
         rule->get("os", &m_os);
+        rule->get("vendor", &m_vendor);
 
         std::string s;
         if(rule->get("version", &s) && s.size()>1)
@@ -330,34 +367,45 @@ public:
             m_disable_options = StringUtils::split(s, ' ');
     }   // Rule
     // ------------------------------------------------------------------------
-    bool applies(const std::string &card, const Version &version) const
+    bool applies(const std::string &card, const Version &version,
+                 const std::string &vendor) const
     {
         // Test for OS
         // -----------
         if(m_os.size()>0)
         {
-#ifdef __linux__
+#if defined(__linux__) && !defined(ANDROID)
             if(m_os!="linux") return false;
-#endif
-#ifdef WIN32
+#elif defined(WIN32)
             if(m_os!="windows") return false;
-#endif
-#ifdef __APPLE__
+#elif defined(__APPLE__)
             if(m_os!="osx") return false;
-#endif
-#ifdef BSD
+#elif defined(BSD)
             if(m_os!="bsd") return false;
+#elif defined(ANDROID)
+            if(m_os!="android") return false;
+#else
+            return false;
 #endif
         }   // m_os.size()>0
+        
+        // Test for vendor
+        // ---------------
+        if (m_vendor.size() > 0)
+        {
+            if (m_vendor != vendor)
+                return false;
+        }
 
         // Test for card
         // -------------
         switch(m_card_test)
         {
+        case CARD_IGNORE: break;   // always true
         case CARD_IS:
             if(card!=m_card_name) return false;
             break;
-        case CARD_CONTAINS: 
+        case CARD_CONTAINS:
             if(card.find(m_card_name)==std::string::npos)
                 return false;
             break;
@@ -413,7 +461,7 @@ void unitTesting()
     assert(Version("10.3") <=  Version("10.3.2"));
     assert(!(Version("10.3.2") <  Version("10.3")));
     assert(!(Version("10.3.2") <= Version("10.3")));
-    assert(Version("3.3 NVIDIA-10.0.19 310.90.10.05b1", 
+    assert(Version("3.3 NVIDIA-10.0.19 310.90.10.05b1",
                    "NVIDIA GeForce GTX 680MX OpenGL Engine")
            == Version("310.90.10.5")                                    );
 
@@ -424,7 +472,7 @@ void unitTesting()
     assert(Version("3.1 (Core Profile) Mesa 10.3.0",
                   "Mesa DRI Mobile Intel\u00ae GM45 Express Chipset")
            == Version("10.3.0")                                         );
-    assert(Version("3.3 (Core Profile) Mesa 10.5.0-devel", 
+    assert(Version("3.3 (Core Profile) Mesa 10.5.0-devel",
                    "Gallium 0.4 on NVC1")
            == Version("10.5.0")                                         );
     assert(Version("3.3 (Core Profile) Mesa 10.5.0-devel",
@@ -442,7 +490,7 @@ void unitTesting()
     assert(Version("4.0.10188 Core Profile Context",
                    "ATI Radeon HD 5400 Series")
         == Version("4.0.10188"));
-    assert(Version("4.1 ATI-1.24.38", "AMD Radeon HD 6970M OpenGL Engine") 
+    assert(Version("4.1 ATI-1.24.38", "AMD Radeon HD 6970M OpenGL Engine")
         == Version("1.24.38"));
 
 }   // unitTesting
@@ -452,9 +500,11 @@ void unitTesting()
  *  \param driver_version The GL_VERSION string (i.e. opengl and version
  *         number).
  *  \param card_name The GL_RENDERER string (i.e. graphics card).
+ *  \param vendor The GL_VENDOR string
  */
 void init(const std::string &driver_version,
-          const std::string &card_name)
+          const std::string &card_name,
+          const std::string &vendor)
 {
     for (unsigned int i = 0; i < GR_COUNT; i++)
         m_all_graphics_restriction.push_back(false);
@@ -488,7 +538,7 @@ void init(const std::string &driver_version,
             continue;
         }
         Rule rule(xml_rule);
-        if (rule.applies(card_name, version))
+        if (rule.applies(card_name, version, vendor))
         {
             std::vector<std::string> restrictions = rule.getRestrictions();
             std::vector<std::string>::iterator p;
