@@ -24,6 +24,7 @@
 
 #include "config/user_config.hpp"
 #include "graphics/material.hpp"
+#include "graphics/particle_kind_manager.hpp"
 #include "graphics/shaders.hpp"
 #include "io/file_manager.hpp"
 #include "io/xml_node.hpp"
@@ -78,15 +79,10 @@ Material* MaterialManager::getMaterialFor(video::ITexture* t,
 }
 
 //-----------------------------------------------------------------------------
-
-Material* MaterialManager::getMaterialFor(video::ITexture* t,
-    video::E_MATERIAL_TYPE material_type)
+Material* MaterialManager::getMaterialFor(video::ITexture* t)
 {
-    if (t == NULL)
-        return getDefaultMaterial(material_type);
+    const io::path& img_path = t->getName().getInternalName();
 
-    core::stringc img_path = core::stringc(t->getName());
-    const std::string image = StringUtils::getBasename(img_path.c_str());
     if (!img_path.empty() && (img_path.findFirst('/') != -1 || img_path.findFirst('\\') != -1))
     {
         // Search backward so that temporary (track) textures are found first
@@ -100,14 +96,30 @@ Material* MaterialManager::getMaterialFor(video::ITexture* t,
     }
     else
     {
+        core::stringc image(StringUtils::getBasename(img_path.c_str()).c_str());
+        image.make_lower();
+
         for (int i = (int)m_materials.size() - 1; i >= 0; i--)
         {
-            if (m_materials[i]->getTexFname() == image)
+            if (m_materials[i]->getTexFname() == image.c_str())
             {
                 return m_materials[i];
             }
         }   // for i
     }
+    return NULL;
+}
+
+//-----------------------------------------------------------------------------
+Material* MaterialManager::getMaterialFor(video::ITexture* t,
+    video::E_MATERIAL_TYPE material_type)
+{
+    if (t == NULL)
+        return getDefaultMaterial(material_type);
+
+    Material* m = getMaterialFor(t);
+    if (m != NULL)
+        return m;
 
     return getDefaultMaterial(material_type);
 }
@@ -139,12 +151,13 @@ Material* MaterialManager::getDefaultMaterial(video::E_MATERIAL_TYPE shader_type
     auto it = m_default_materials.find(shader_type);
     if (it == m_default_materials.end())
     {
-        Material* default_material = new Material("", false, false, false);
+        Material* default_material = new Material("Default", false, false, false);
 
         // TODO: workaround, should not hardcode these material types here?
         // Try to find a cleaner way
         // If graphics are disabled, shaders should not be accessed (getShader
         // asserts that shaders are initialised).
+#ifndef SERVER_ONLY
         if(!ProfileWorld::isNoGraphics() && CVS->isGLSL() &&
             shader_type == Shaders::getShader(ShaderType::ES_OBJECT_UNLIT))
             default_material->setShaderType(Material::SHADERTYPE_SOLID_UNLIT);
@@ -156,7 +169,7 @@ Material* MaterialManager::getDefaultMaterial(video::E_MATERIAL_TYPE shader_type
         //    default_material->setShaderType(Material::SHADERTYPE_ALPHA_BLEND);
         else
             default_material->setShaderType(Material::SHADERTYPE_SOLID);
-
+#endif
         m_default_materials[shader_type] = default_material;
         return default_material;
     }
@@ -318,7 +331,7 @@ Material *MaterialManager::getMaterial(const std::string& fname,
                                        bool is_full_path,
                                        bool make_permanent,
                                        bool complain_if_not_found,
-                                       bool strip_path)
+                                       bool strip_path, bool install)
 {
     if(fname=="")
     {
@@ -338,14 +351,18 @@ Material *MaterialManager::getMaterial(const std::string& fname,
     else
         basename = fname;
         
+    core::stringc basename_lower(basename.c_str());
+    basename_lower.make_lower();
+
     // Search backward so that temporary (track) textures are found first
-    for(int i = (int)m_materials.size()-1; i>=0; i-- )
+    for (int i = (int)m_materials.size()-1; i>=0; i-- )
     {
-        if(m_materials[i]->getTexFname()==basename) return m_materials[i];
+        if (m_materials[i]->getTexFname() == basename_lower.c_str())
+            return m_materials[i];
     }
 
     // Add the new material
-    Material* m = new Material(fname, is_full_path, complain_if_not_found);
+    Material* m = new Material(fname, is_full_path, complain_if_not_found, install);
     m_materials.push_back(m);
     if(make_permanent)
     {
@@ -364,6 +381,30 @@ void MaterialManager::makeMaterialsPermanent()
     m_shared_material_index = (int) m_materials.size();
 }   // makeMaterialsPermanent
 
+// ----------------------------------------------------------------------------
+void MaterialManager::unloadAllTextures()
+{
+    std::string texture_folder =
+        file_manager->getAssetDirectory(FileManager::TEXTURE);
+    texture_folder = file_manager->getFileSystem()->getAbsolutePath
+        (texture_folder.c_str()).c_str();
+    core::stringc texfname(texture_folder.c_str());
+    texfname.make_lower();
+    texture_folder = texfname.c_str();
+    for (int i = 0; i < m_shared_material_index; i++)
+    {
+        // Global particle textures will stay until exit
+        // STK, which avoid hangs when lazy-loading the texture when being
+        // triggered.
+        Material* m = m_materials[i];
+        if (!ParticleKindManager::get()->isGlobalParticleMaterial(m)
+            && m->getTexFullPath().find(texture_folder) != std::string::npos)
+        {
+            m->unloadTexture();
+        }
+    }
+}
+
 // ----------------------------------------------------------------------------
 bool MaterialManager::hasMaterial(const std::string& fname)
 {
