@@ -61,6 +61,7 @@ public:
                              scene::IMeshBuffer *mb);
     Material* getMaterialFor(video::ITexture* t,
                              video::E_MATERIAL_TYPE material_type);
+    Material* getMaterialFor(video::ITexture* t);
     void      setAllMaterialFlags(video::ITexture* t,
                                   scene::IMeshBuffer *mb);
     void      adjustForFog(video::ITexture* t,
@@ -75,7 +76,7 @@ public:
                                 bool is_full_path=false,
                                 bool make_permanent=false,
                                 bool complain_if_not_found=true,
-                                bool strip_path=true);
+                                bool strip_path=true, bool install=true);
     void      addSharedMaterial(const std::string& filename, bool deprecated = false);
     bool      pushTempMaterial (const std::string& filename, bool deprecated = false);
     bool      pushTempMaterial (const XMLNode *root, const std::string& filename, bool deprecated = false);
@@ -83,6 +84,8 @@ public:
     void      makeMaterialsPermanent();
     bool      hasMaterial(const std::string& fname);
 
+    void      unloadAllTextures();
+
     Material* getLatestMaterial() { return m_materials[m_materials.size()-1]; }
 };   // MaterialManager
 
