@@ -16,146 +16,33 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#include "graphics/shader.hpp"
+#ifndef SERVER_ONLY
 
-#include "graphics/central_settings.hpp"
-#include "graphics/gl_headers.hpp"
+#include "graphics/shader.hpp"
 #include "graphics/irr_driver.hpp"
-#include "graphics/shared_gpu_objects.hpp"
-#include "io/file_manager.hpp"
+#include "graphics/spherical_harmonics.hpp"
 #include "utils/log.hpp"
 
 #include <fstream>
 #include <sstream>
 #include <stdio.h>
 
-std::string             ShaderBase::m_shader_header = "";
 std::vector<void(*)()>  ShaderBase::m_all_kill_functions;
 
-// ----------------------------------------------------------------------------
-/** Returns a string with the content of header.txt (which contains basic
- *  shader defines).
- */
-const std::string& ShaderBase::getHeader()
-{
-    // Only read file first time
-    if (m_shader_header.empty())
-    {
-        std::ifstream stream(file_manager->getShader("header.txt"), std::ios::in);
-        if (stream.is_open())
-        {
-            std::string line = "";
-            while (getline(stream, line))
-                m_shader_header += "\n" + line;
-            stream.close();
-        }
-    }   // if m_shader_header.empty()
-
-    return m_shader_header;
-}   // getHeader
-
-// ----------------------------------------------------------------------------
-/** Loads a single shader.
- *  \param file Filename of the shader to load.
- *  \param type Type of the shader.
- */
-GLuint ShaderBase::loadShader(const std::string &file, unsigned type)
-{
-    GLuint id = glCreateShader(type);
-    
-    std::ostringstream code;
-    code << "#version " << CVS->getGLSLVersion()<<"\n";
-
-    // Some drivers report that the compute shaders extension is available,
-    // but they report only OpenGL 3.x version, and thus these extensions
-    // must be enabled manually. Otherwise the shaders compilation will fail
-    // because STK tries to use extensions which are available, but disabled
-    // by default.
-    if (type == GL_COMPUTE_SHADER)
-    {
-        if (CVS->isARBComputeShaderUsable())
-            code << "#extension GL_ARB_compute_shader : enable\n";
-        if (CVS->isARBImageLoadStoreUsable())
-            code << "#extension GL_ARB_shader_image_load_store : enable\n";
-        if (CVS->isARBArraysOfArraysUsable())
-            code << "#extension GL_ARB_arrays_of_arrays : enable\n";
-    }
-    
-    if (CVS->isAMDVertexShaderLayerUsable())
-        code << "#extension GL_AMD_vertex_shader_layer : enable\n";
-        
-    if (CVS->isARBExplicitAttribLocationUsable())
-        code << "#extension GL_ARB_explicit_attrib_location : enable\n";
-    
-    if (CVS->isAZDOEnabled())
-    {
-        code << "#extension GL_ARB_bindless_texture : enable\n";
-        code << "#define Use_Bindless_Texture\n";
-    }
-    code << "//" << file << "\n";
-    if (!CVS->isARBUniformBufferObjectUsable())
-        code << "#define UBO_DISABLED\n";
-    if (CVS->isAMDVertexShaderLayerUsable())
-        code << "#define VSLayer\n";
-    if (CVS->needsRGBBindlessWorkaround())
-        code << "#define SRGBBindlessFix\n";
-
-    //shader compilation fails with some drivers if there is no precision qualifier
-    if (type == GL_FRAGMENT_SHADER)
-        code << "precision mediump float;\n";
-    
-    code << getHeader();
-
-    std::ifstream stream(file_manager->getShader(file), std::ios::in);
-    if (stream.is_open())
-    {
-        std::string Line = "";
-        while (getline(stream, Line))
-            code << "\n" << Line;
-        stream.close();
-    }
-    else
-        Log::error("shader", "Can not open '%s'.", file.c_str());
-
-    Log::info("shader", "Compiling shader : %s", file.c_str());
-    const std::string &source  = code.str();
-    char const *source_pointer = source.c_str();
-    int len                    = source.size();
-    glShaderSource(id, 1, &source_pointer, &len);
-    glCompileShader(id);
-
-    GLint result = GL_FALSE;
-    glGetShaderiv(id, GL_COMPILE_STATUS, &result);
-    if (result == GL_FALSE)
-    {
-        int info_length;
-        Log::error("GLWrap", "Error in shader %s", file.c_str());
-        glGetShaderiv(id, GL_INFO_LOG_LENGTH, &info_length);
-        if (info_length<0)
-            info_length = 1024;
-        char *error_message = new char[info_length];
-        error_message[0] = 0;
-        glGetShaderInfoLog(id, info_length, NULL, error_message);
-        Log::error("GLWrap", error_message);
-        delete[] error_message;
-    }
-
-    glGetError();
-
-    return id;
-}   // loadShader
-
 // ----------------------------------------------------------------------------
 /** Loads a transform feedback buffer shader with a given number of varying
  *  parameters.
 */
-int ShaderBase::loadTFBProgram(const std::string &shader_name, 
+int ShaderBase::loadTFBProgram(const std::string &shader_name,
                                const char **varyings,
                                unsigned varying_count)
 {
     m_program = glCreateProgram();
     loadAndAttachShader(GL_VERTEX_SHADER, shader_name);
-    if (CVS->getGLSLVersion() < 330)
+#ifdef USE_GLES2
+    loadAndAttachShader(GL_FRAGMENT_SHADER, "tfb_dummy.frag");
+#endif
+    if (!CVS->isARBExplicitAttribLocationUsable())
         setAttribute(PARTICLES_SIM);
 
     glTransformFeedbackVaryings(m_program, varying_count, varyings,
@@ -170,7 +57,7 @@ int ShaderBase::loadTFBProgram(const std::string &shader_name,
         glGetProgramiv(m_program, GL_INFO_LOG_LENGTH, &info_log_length);
         char *error_message = new char[info_log_length];
         glGetProgramInfoLog(m_program, info_log_length, NULL, error_message);
-        Log::error("GLWrap", error_message);
+        Log::error("ShaderBase", error_message);
         delete[] error_message;
     }
 
@@ -188,6 +75,9 @@ void ShaderBase::bypassUBO() const
     GLint PM = glGetUniformLocation(m_program, "ProjectionMatrix");
     glUniformMatrix4fv(PM, 1, GL_FALSE, irr_driver->getProjMatrix().pointer());
 
+    GLint PVM = glGetUniformLocation(m_program, "ProjectionViewMatrix");
+    glUniformMatrix4fv(PVM, 1, GL_FALSE, irr_driver->getProjViewMatrix().pointer());
+
     GLint IVM = glGetUniformLocation(m_program, "InverseViewMatrix");
     glUniformMatrix4fv(IVM, 1, GL_FALSE, irr_driver->getInvViewMatrix().pointer());
 
@@ -195,20 +85,19 @@ void ShaderBase::bypassUBO() const
     glUniformMatrix4fv(IPM, 1, GL_FALSE, irr_driver->getInvProjMatrix().pointer());
 
     GLint Screen = glGetUniformLocation(m_program, "screen");
-    glUniform2f(Screen, irr_driver->getCurrentScreenSize().X, 
+    glUniform2f(Screen, irr_driver->getCurrentScreenSize().X,
                         irr_driver->getCurrentScreenSize().Y);
 
+    const SHCoefficients* sh_coeff = irr_driver->getSHCoefficients();
+
     GLint bLmn = glGetUniformLocation(m_program, "blueLmn[0]");
-    const float*  blue_SH_coeff = irr_driver->getSphericalHarmonics()->getBlueSHCoeff();
-    glUniform1fv(bLmn, 9, blue_SH_coeff);
+    glUniform1fv(bLmn, 9, sh_coeff->blue_SH_coeff);
 
     GLint gLmn = glGetUniformLocation(m_program, "greenLmn[0]");
-    const float*  green_SH_coeff = irr_driver->getSphericalHarmonics()->getGreenSHCoeff();
-    glUniform1fv(gLmn, 9, green_SH_coeff);
+    glUniform1fv(gLmn, 9, sh_coeff->green_SH_coeff);
 
     GLint rLmn = glGetUniformLocation(m_program, "redLmn[0]");
-    const float*  red_SH_coeff = irr_driver->getSphericalHarmonics()->getRedSHCoeff();
-    glUniform1fv(rLmn, 9, red_SH_coeff);
+    glUniform1fv(rLmn, 9, sh_coeff->red_SH_coeff);
 
     GLint sun_dir = glGetUniformLocation(m_program, "sun_direction");
     const core::vector3df &sd = irr_driver->getSunDirection();
@@ -256,6 +145,7 @@ void ShaderBase::setAttribute(AttributeType type)
         glBindAttribLocation(m_program, 7, "Origin");
         glBindAttribLocation(m_program, 8, "Orientation");
         glBindAttribLocation(m_program, 9, "Scale");
+        glBindAttribLocation(m_program, 10, "misc_data");
         break;
     case PARTICLES_SIM:
         glBindAttribLocation(m_program, 0, "particle_position");
@@ -266,14 +156,34 @@ void ShaderBase::setAttribute(AttributeType type)
         glBindAttribLocation(m_program, 5, "lifetime_initial");
         glBindAttribLocation(m_program, 6, "particle_velocity_initial");
         glBindAttribLocation(m_program, 7, "size_initial");
+        if (CVS->needsVertexIdWorkaround())
+        {
+            glBindAttribLocation(m_program, 8, "vertex_id");
+        }
         break;
     case PARTICLES_RENDERING:
-        glBindAttribLocation(m_program, 1, "lifetime");
+        glBindAttribLocation(m_program, 0, "Position");
+        glBindAttribLocation(m_program, 1, "color_lifetime");
         glBindAttribLocation(m_program, 2, "size");
+        glBindAttribLocation(m_program, 3, "Texcoord");
         glBindAttribLocation(m_program, 4, "quadcorner");
         glBindAttribLocation(m_program, 5, "rotationvec");
         glBindAttribLocation(m_program, 6, "anglespeed");
         break;
+    case SKINNED_MESH:
+        glBindAttribLocation(m_program, 0, "Position");
+        glBindAttribLocation(m_program, 1, "Normal");
+        glBindAttribLocation(m_program, 2, "Color");
+        glBindAttribLocation(m_program, 3, "Data1");
+        glBindAttribLocation(m_program, 4, "Data2");
+        glBindAttribLocation(m_program, 5, "Joint");
+        glBindAttribLocation(m_program, 6, "Weight");
+        glBindAttribLocation(m_program, 7, "Origin");
+        glBindAttribLocation(m_program, 8, "Orientation");
+        glBindAttribLocation(m_program, 9, "Scale");
+        glBindAttribLocation(m_program, 10, "misc_data");
+        glBindAttribLocation(m_program, 15, "skinning_offset");
+        break;
     }
 }   // setAttribute
 
@@ -297,3 +207,5 @@ GLuint ShaderBase::createVAO()
 }   // createVAO
 
 // ============================================================================
+
+#endif   // !SERVER_ONLY
