@@ -16,10 +16,12 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+#ifndef SERVER_ONLY
+
 #include "graphics/show_curve.hpp"
 
-#include "graphics/glwrap.hpp"
 #include "graphics/irr_driver.hpp"
+#include "graphics/stk_tex_manager.hpp"
 #include "utils/vec3.hpp"
 
 #include <IMeshSceneNode.h>
@@ -65,8 +67,9 @@ void ShowCurve::addEmptyMesh()
     m_mesh            = irr_driver->createQuadMesh(&m,
                                                    /*create_one_quad*/ false);
     m_buffer          = m_mesh->getMeshBuffer(0);
-    m_buffer->getMaterial().setTexture(0, getUnicolorTexture(video::SColor(128, 255, 105, 180)));
-    m_buffer->getMaterial().setTexture(1, getUnicolorTexture(video::SColor(0, 0, 0, 0)));
+    m_buffer->getMaterial().setTexture(0, STKTexManager::getInstance()->getUnicolorTexture(video::SColor(128, 255, 105, 180)));
+    m_buffer->getMaterial().setTexture(1, STKTexManager::getInstance()->getUnicolorTexture(video::SColor(0, 0, 0, 0)));
+    m_buffer->getMaterial().setTexture(2, STKTexManager::getInstance()->getUnicolorTexture(video::SColor(0, 0, 0, 0)));
 
     assert(m_buffer->getVertexType()==video::EVT_STANDARD);
 }   // addEmptyMesh
@@ -199,3 +202,6 @@ void ShowCurve::setPosition(const Vec3 &xyz)
     m_scene_node->setPosition(xyz.toIrrVector());
 }   // setPosition
 // ----------------------------------------------------------------------------
+
+#endif   // !SERVER_ONLY
+
