@@ -25,6 +25,7 @@
 #include "karts/controller/controller.hpp"
 #include "karts/abstract_kart.hpp"
 #include "karts/skidding.hpp"
+#include "modes/world.hpp"
 #include "physics/btKart.hpp"
 
 #include <IMeshSceneNode.h>
@@ -131,12 +132,14 @@ void SkidMarks::update(float dt, bool force_skid_marks,
         if (!is_skidding)   // end skid marking
         {
             m_skid_marking = false;
+#ifndef SERVER_ONLY
             // The vertices and indices will not change anymore
             // (till these skid mark quads are deleted)
             m_left[m_current]->setHardwareMappingHint(scene::EHM_STATIC);
             m_right[m_current]->setHardwareMappingHint(scene::EHM_STATIC);
             if (STKMeshSceneNode* stkm = dynamic_cast<STKMeshSceneNode*>(m_nodes[m_current]))
                 stkm->setReloadEachFrame(false);
+#endif
             return;
         }
 
@@ -153,9 +156,9 @@ void SkidMarks::update(float dt, bool force_skid_marks,
         float distance = (newPoint - start).length();
 
         m_left [m_current]->add(raycast_left-delta, raycast_left+delta,
-                                distance);
+                                m_kart.getNormal(), distance);
         m_right[m_current]->add(raycast_right-delta, raycast_right+delta,
-                                distance);
+                                m_kart.getNormal(), distance);
         // Adjust the boundary box of the mesh to include the
         // adjusted aabb of its buffers.
         core::aabbox3df aabb=m_nodes[m_current]->getMesh()
@@ -180,21 +183,25 @@ void SkidMarks::update(float dt, bool force_skid_marks,
     delta *= m_width*0.5f;
 
     SkidMarkQuads *smq_left =
-        new SkidMarkQuads(raycast_left-delta, raycast_left+delta ,
-                          m_material, m_avoid_z_fighting, custom_color);
+        new SkidMarkQuads(raycast_left-delta, raycast_left+delta,
+                          m_kart.getNormal(), m_material, m_avoid_z_fighting,
+                          custom_color);
     scene::SMesh *new_mesh = new scene::SMesh();
     new_mesh->addMeshBuffer(smq_left);
 
     SkidMarkQuads *smq_right =
         new SkidMarkQuads(raycast_right-delta, raycast_right+delta,
-                          m_material, m_avoid_z_fighting, custom_color);
+                          m_kart.getNormal(), m_material, m_avoid_z_fighting,
+                          custom_color);
     new_mesh->addMeshBuffer(smq_right);
     scene::IMeshSceneNode *new_node = irr_driver->addMesh(new_mesh, "skidmark");
+#ifndef SERVER_ONLY
     if (STKMeshSceneNode* stkm = dynamic_cast<STKMeshSceneNode*>(new_node))
         stkm->setReloadEachFrame(true);
 #ifdef DEBUG
     std::string debug_name = m_kart.getIdent()+" (skid-mark)";
     new_node->setName(debug_name.c_str());
+#endif
 #endif
 
     // We don't keep a reference to the mesh here, so we have to decrement
@@ -202,7 +209,9 @@ void SkidMarks::update(float dt, bool force_skid_marks,
     // The scene node will keep the mesh alive.
     new_mesh->drop();
     m_current++;
-    if(m_current>=stk_config->m_max_skidmarks)
+    int cleaning_threshold =
+        core::clamp(int(World::getWorld()->getNumKarts()), 5, 15);
+    if(m_current>=int(stk_config->m_max_skidmarks/cleaning_threshold))
         m_current = 0;
     if(m_current>=(int)m_left.size())
     {
@@ -233,6 +242,7 @@ void SkidMarks::update(float dt, bool force_skid_marks,
 //=============================================================================
 SkidMarks::SkidMarkQuads::SkidMarkQuads(const Vec3 &left,
                                         const Vec3 &right,
+                                        const Vec3 &normal,
                                         video::SMaterial *material,
                                         float z_offset,
                                         video::SColor* custom_color)
@@ -250,7 +260,7 @@ SkidMarks::SkidMarkQuads::SkidMarkQuads(const Vec3 &left,
 
     Material   = *material;
     m_aabb     = core::aabbox3df(left.toIrrVector());
-    add(left, right, 0.0f);
+    add(left, right, normal, 0.0f);
 
 
 }   // SkidMarkQuads
@@ -261,6 +271,7 @@ SkidMarks::SkidMarkQuads::SkidMarkQuads(const Vec3 &left,
  */
 void SkidMarks::SkidMarkQuads::add(const Vec3 &left,
                                    const Vec3 &right,
+                                   const Vec3 &normal,
                                    float distance)
 {
     // The skid marks must be raised slightly higher, otherwise it blends
@@ -280,13 +291,11 @@ void SkidMarks::SkidMarkQuads::add(const Vec3 &left,
         Vertices[n - 2].Color.setAlpha(m_start_alpha);
     }
 
-    v.Pos = left.toIrrVector();
-    v.Pos.Y += m_z_offset;
-    v.Normal = core::vector3df(0, 1, 0);
+    v.Pos = Vec3(left + normal * m_z_offset).toIrrVector();
+    v.Normal = normal.toIrrVector();
     v.TCoords = core::vector2df(0.0f, distance*0.5f);
     Vertices.push_back(v);
-    v.Pos = right.toIrrVector();
-    v.Pos.Y += m_z_offset;
+    v.Pos = Vec3(right + normal * m_z_offset).toIrrVector();
     v.TCoords = core::vector2df(1.0f, distance*0.5f);
     Vertices.push_back(v);
     // Out of the box Irrlicht only supports triangle meshes and not
