@@ -15,22 +15,20 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#include "graphics/glwrap.hpp"
+#ifndef SERVER_ONLY
 
-#include "config/user_config.hpp"
-#include "central_settings.hpp"
-#include "graphics/camera.hpp"
-#include "graphics/irr_driver.hpp"
 #include "graphics/stk_animated_mesh.hpp"
+
+#include "graphics/central_settings.hpp"
 #include "graphics/material_manager.hpp"
-#include "modes/world.hpp"
-#include "tracks/track.hpp"
-#include "utils/profiler.hpp"
-#include "utils/cpp2011.hpp"
+#include "graphics/render_info.hpp"
+#include "graphics/shared_gpu_objects.hpp"
+#include "graphics/stk_mesh.hpp"
+#include "graphics/vao_manager.hpp"
 
 #include <IMaterialRenderer.h>
 #include <ISceneManager.h>
-#include <ISkinnedMesh.h>
+#include "../../lib/irrlicht/source/Irrlicht/CSkinnedMesh.h"
 
 using namespace irr;
 
@@ -38,14 +36,19 @@ STKAnimatedMesh::STKAnimatedMesh(irr::scene::IAnimatedMesh* mesh, irr::scene::IS
 irr::scene::ISceneManager* mgr, s32 id, const std::string& debug_name,
 const core::vector3df& position,
 const core::vector3df& rotation,
-const core::vector3df& scale) :
-    CAnimatedMeshSceneNode(mesh, parent, mgr, id, position, rotation, scale)
+const core::vector3df& scale, RenderInfo* render_info, bool all_parts_colorized) :
+    CAnimatedMeshSceneNode(mesh, parent, mgr, id, position, rotation, scale),
+    m_skinned_mesh(NULL), m_skinning_offset(-1)
 {
     isGLInitialized = false;
     isMaterialInitialized = false;
+    m_got_animated_matrix = false;
+    m_mesh_render_info = render_info;
+    m_all_parts_colorized = all_parts_colorized;
 #ifdef DEBUG
     m_debug_name = debug_name;
 #endif
+    resetSkinningState(mesh);
 }
 
 STKAnimatedMesh::~STKAnimatedMesh()
@@ -80,6 +83,7 @@ void STKAnimatedMesh::setMesh(scene::IAnimatedMesh* mesh)
     isMaterialInitialized = false;
     cleanGLMeshes();
     CAnimatedMeshSceneNode::setMesh(mesh);
+    resetSkinningState(mesh);
 }
 
 void STKAnimatedMesh::updateNoGL()
@@ -97,10 +101,57 @@ void STKAnimatedMesh::updateNoGL()
     if (!isMaterialInitialized)
     {
         video::IVideoDriver* driver = SceneManager->getVideoDriver();
-        for (u32 i = 0; i < m->getMeshBufferCount(); ++i)
+        const u32 mb_count = m->getMeshBufferCount();
+        for (u32 i = 0; i < mb_count; ++i)
         {
             scene::IMeshBuffer* mb = Mesh->getMeshBuffer(i);
-            GLmeshes.push_back(allocateMeshBuffer(mb, m_debug_name));
+            if (!mb)
+                continue;
+
+            scene::SSkinMeshBuffer* ssmb = NULL;
+            video::E_VERTEX_TYPE prev_type = mb->getVertexType();
+            if (m_skinned_mesh)
+            {
+                ssmb = dynamic_cast<scene::SSkinMeshBuffer*>(mb);
+                ssmb->VertexType = video::EVT_SKINNED_MESH;
+            }
+
+            bool affected = false;
+            RenderInfo* cur_ri = m_mesh_render_info;
+            if (!m_all_parts_colorized && mb && cur_ri)
+            {
+                if (m_mesh_render_info && !m_mesh_render_info->isStatic())
+                {
+                    // Convert to static render info for each mesh buffer
+                    assert(m_mesh_render_info->getNumberOfHue() == mb_count);
+                    const float hue = m_mesh_render_info->getDynamicHue(i);
+                    if (hue > 0.0f)
+                    {
+                        cur_ri = new RenderInfo(hue);
+                        m_static_render_info.push_back(cur_ri);
+                        affected = true;
+                    }
+                    else
+                    {
+                        cur_ri = NULL;
+                    }
+                }
+                else
+                {
+                    // Test if material is affected by static hue change
+                    Material* m = material_manager->getMaterialFor(mb
+                        ->getMaterial().getTexture(0), mb);
+                    if (m->isColorizable())
+                        affected = true;
+                }
+            }
+
+            assert(cur_ri ? cur_ri->isStatic() : true);
+            GLmeshes.push_back(allocateMeshBuffer(mb, m_debug_name,
+                affected || m_all_parts_colorized || (cur_ri
+                && cur_ri->isTransparent()) ? cur_ri : NULL));
+
+            if (m_skinned_mesh) ssmb->VertexType = prev_type;
         }
 
         for (u32 i = 0; i < m->getMeshBufferCount(); ++i)
@@ -108,6 +159,15 @@ void STKAnimatedMesh::updateNoGL()
             scene::IMeshBuffer* mb = Mesh->getMeshBuffer(i);
             if (!mb)
                 continue;
+
+            scene::SSkinMeshBuffer* ssmb = NULL;
+            video::E_VERTEX_TYPE prev_type = mb->getVertexType();
+            if (m_skinned_mesh)
+            {
+                ssmb = dynamic_cast<scene::SSkinMeshBuffer*>(mb);
+                ssmb->VertexType = video::EVT_SKINNED_MESH;
+            }
+
             video::E_MATERIAL_TYPE type = mb->getMaterial().MaterialType;
             f32 MaterialTypeParam = mb->getMaterial().MaterialTypeParam;
             video::IMaterialRenderer* rnd = driver->getMaterialRenderer(type);
@@ -119,22 +179,29 @@ void STKAnimatedMesh::updateNoGL()
                 continue;
             }
             GLMesh &mesh = GLmeshes[i];
+            video::E_VERTEX_TYPE vt = mb->getVertexType();
             Material* material = material_manager->getMaterialFor(mb->getMaterial().getTexture(0), mb);
 
             if (rnd->isTransparent())
             {
-                TransparentMaterial TranspMat = getTransparentMaterialFromType(type, MaterialTypeParam, material);
+                TransparentMaterial TranspMat = getTransparentMaterialFromType(type, vt, MaterialTypeParam, material);
                 TransparentMesh[TranspMat].push_back(&mesh);
             }
-            else if (m->getRenderType() == video::ERT_TRANSPARENT)
+            else if (mesh.m_render_info != NULL && mesh.m_render_info->isTransparent())
             {
-                TransparentMesh[TM_ADDITIVE].push_back(&mesh);
+                if (mesh.VAOType == video::EVT_SKINNED_MESH)
+                    TransparentMesh[TM_TRANSLUCENT_SKN].push_back(&mesh);
+                else if (mesh.VAOType == video::EVT_TANGENTS)
+                    TransparentMesh[TM_TRANSLUCENT_TAN].push_back(&mesh);
+                else
+                    TransparentMesh[TM_TRANSLUCENT_STD].push_back(&mesh);
             }
             else
             {
-                Material::ShaderType MatType = material->getShaderType();// getMeshMaterialFromType(type, mb->getVertexType(), material);
+                Material::ShaderType MatType = getMeshMaterialFromType(type, vt, material, NULL);
                 MeshSolidMaterial[MatType].push_back(&mesh);
             }
+            if (m_skinned_mesh) ssmb->VertexType = prev_type;
         }
         isMaterialInitialized = true;
     }
@@ -142,25 +209,39 @@ void STKAnimatedMesh::updateNoGL()
     for (u32 i = 0; i < m->getMeshBufferCount(); ++i)
     {
         scene::IMeshBuffer* mb = Mesh->getMeshBuffer(i);
-        if (!mb)
-            continue;
-        if (mb)
-            GLmeshes[i].TextureMatrix = getMaterial(i).getTextureMatrix(0);
+        if (mb != NULL)
+        {
+            // Test if texture matrix needs to be updated every frame
+            const core::matrix4& mat = getMaterial(i).getTextureMatrix(0);
+            if (mat.isIdentity() && !m_got_animated_matrix)
+                continue;
+            else
+            {
+                m_got_animated_matrix = true;
+                GLmeshes[i].texture_trans.X = mat[8];
+                GLmeshes[i].texture_trans.Y = mat[9];
+            }
+        }
     }
 }
 
 void STKAnimatedMesh::updateGL()
 {
-
-    scene::IMesh* m = getMeshForCurrentFrame();
-
     if (!isGLInitialized)
     {
-        for (u32 i = 0; i < m->getMeshBufferCount(); ++i)
+        for (u32 i = 0; i < Mesh->getMeshBufferCount(); ++i)
         {
             scene::IMeshBuffer* mb = Mesh->getMeshBuffer(i);
             if (!mb)
                 continue;
+            scene::SSkinMeshBuffer* ssmb = NULL;
+            video::E_VERTEX_TYPE prev_type = mb->getVertexType();
+            if (m_skinned_mesh)
+            {
+                ssmb = dynamic_cast<scene::SSkinMeshBuffer*>(mb);
+                ssmb->VertexType = video::EVT_SKINNED_MESH;
+            }
+
             video::IVideoDriver* driver = SceneManager->getVideoDriver();
             video::E_MATERIAL_TYPE type = mb->getMaterial().MaterialType;
             video::IMaterialRenderer* rnd = driver->getMaterialRenderer(type);
@@ -191,10 +272,20 @@ void STKAnimatedMesh::updateGL()
                 mesh.vao = createVAO(mesh.vertex_buffer, mesh.index_buffer, mb->getVertexType());
                 glBindVertexArray(0);
             }
+            if (m_skinned_mesh) ssmb->VertexType = prev_type;
         }
         isGLInitialized = true;
     }
 
+    if (useHardwareSkinning() && m_skinned_mesh->getTotalJoints() == 0) return;
+
+    scene::IMesh* m = getMeshForCurrentFrame();
+    if (useHardwareSkinning())
+    {
+        m_skinning_offset = -1;
+        return;
+    }
+
     for (u32 i = 0; i<m->getMeshBufferCount(); ++i)
     {
         scene::IMeshBuffer* mb = m->getMeshBuffer(i);
@@ -237,3 +328,63 @@ void STKAnimatedMesh::render()
     updateNoGL();
     updateGL();
 }
+
+int STKAnimatedMesh::getTotalJoints() const
+{
+    return m_skinned_mesh->getTotalJoints();
+}
+
+void STKAnimatedMesh::resetSkinningState(scene::IAnimatedMesh* mesh)
+{
+    if (!CVS->supportsHardwareSkinning()) return;
+    m_skinning_offset = -1;
+    m_skinned_mesh = NULL;
+    if (mesh == NULL) return;
+    for (u32 i = 0; i < mesh->getMeshBufferCount(); ++i)
+    {
+        scene::IMeshBuffer* mb = Mesh->getMeshBuffer(i);
+        if (!mb)
+            continue;
+        if (mb->getVertexType() == video::EVT_2TCOORDS)
+            return;
+    }
+    setHardwareSkinning(true);
+    if (m_skinned_mesh)
+        m_skinned_mesh->convertForSkinning();
+}
+
+scene::IMesh* STKAnimatedMesh::getMeshForCurrentFrame(SkinningCallback sc,
+                                                      int offset)
+{
+    if (!useHardwareSkinning())
+        return scene::CAnimatedMeshSceneNode::getMeshForCurrentFrame();
+    if (m_skinning_offset == -1)
+        return Mesh;
+
+    return scene::CAnimatedMeshSceneNode::getMeshForCurrentFrame
+        (uploadJoints, m_skinning_offset);
+}
+
+void STKAnimatedMesh::setHardwareSkinning(bool val)
+{
+    if (!CVS->supportsHardwareSkinning()) return;
+    if (val)
+        m_skinned_mesh = dynamic_cast<scene::CSkinnedMesh*>(Mesh);
+    else
+        m_skinned_mesh = NULL;
+}
+
+void STKAnimatedMesh::uploadJoints(const irr::core::matrix4& m,
+                                   int joint, int offset)
+{
+    assert(offset != -1);
+#ifdef USE_GLES2
+    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, (offset >> 6) + joint, 16, 1, GL_RGBA,
+        GL_FLOAT, m.pointer());
+#else
+    glBufferSubData(GL_TEXTURE_BUFFER, offset + joint * 16 * sizeof(float),
+        16 * sizeof(float), m.pointer());
+#endif
+}
+
+#endif   // !SERVER_ONLY
