@@ -15,18 +15,16 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+#ifndef SERVER_ONLY
+
 #include "graphics/stk_mesh.hpp"
 
-#include "config/user_config.hpp"
-#include "central_settings.hpp"
 #include "graphics/callbacks.hpp"
-#include "graphics/camera.hpp"
-#include "graphics/glwrap.hpp"
+#include "graphics/central_settings.hpp"
 #include "graphics/irr_driver.hpp"
+#include "graphics/material_manager.hpp"
 #include "graphics/shaders.hpp"
-#include "modes/world.hpp"
-#include "tracks/track.hpp"
-#include "utils/helpers.hpp"
+#include "graphics/stk_tex_manager.hpp"
 
 #include <ISceneManager.h>
 #include <IMaterialRenderer.h>
@@ -42,6 +40,24 @@ Material::ShaderType getMeshMaterialFromType(video::E_MATERIAL_TYPE material_typ
         layer2_material->getShaderType() == Material::SHADERTYPE_SPLATTING)
         return Material::SHADERTYPE_SPLATTING;
 
+    if (tp == video::EVT_SKINNED_MESH)
+    {
+        switch (material->getShaderType())
+        {
+        case Material::SHADERTYPE_SOLID:
+            if (material_type == Shaders::getShader(ES_NORMAL_MAP))
+                return Material::SHADERTYPE_NORMAL_MAP_SKINNED_MESH;
+            else
+                return Material::SHADERTYPE_SOLID_SKINNED_MESH;
+        case Material::SHADERTYPE_ALPHA_TEST:
+            return Material::SHADERTYPE_ALPHA_TEST_SKINNED_MESH;
+        case Material::SHADERTYPE_SOLID_UNLIT:
+            return Material::SHADERTYPE_SOLID_UNLIT_SKINNED_MESH;
+        default:
+            return Material::SHADERTYPE_SOLID_SKINNED_MESH;
+        }
+    }
+
     switch (material->getShaderType())
     {
     default:
@@ -57,11 +73,22 @@ Material::ShaderType getMeshMaterialFromType(video::E_MATERIAL_TYPE material_typ
 
 // ----------------------------------------------------------------------------
 TransparentMaterial getTransparentMaterialFromType(video::E_MATERIAL_TYPE type,
+                                                   video::E_VERTEX_TYPE tp,
                                                    f32 MaterialTypeParam,
                                                    Material* material)
 {
+    if (tp == video::EVT_SKINNED_MESH)
+        return TM_TRANSLUCENT_SKN;
+
     if (type == Shaders::getShader(ES_DISPLACE))
-        return TM_DISPLACEMENT;
+    {
+        if (CVS->isDefferedEnabled())
+            return TM_DISPLACEMENT;
+        else if (tp == video::EVT_2TCOORDS)
+            return TM_TRANSLUCENT_2TC;
+        else
+            return TM_TRANSLUCENT_STD;
+    }
     if (material->getShaderType() == Material::SHADERTYPE_ADDITIVE)
         return TM_ADDITIVE;
     return TM_DEFAULT;
@@ -154,6 +181,22 @@ GLuint createVAO(GLuint vbo, GLuint idx, video::E_VERTEX_TYPE type)
         glVertexAttribPointer(6, 3, GL_FLOAT, GL_FALSE,
                               getVertexPitchFromType(type), (GLvoid*)48);
         break;
+    case video::EVT_SKINNED_MESH:
+        glEnableVertexAttribArray(0);
+        glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, getVertexPitchFromType(type), 0);
+        glEnableVertexAttribArray(1);
+        glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, getVertexPitchFromType(type), (GLvoid*)12);
+        glEnableVertexAttribArray(2);
+        glVertexAttribPointer(2, 4, GL_UNSIGNED_BYTE, GL_TRUE, getVertexPitchFromType(type), (GLvoid*)24);
+        glEnableVertexAttribArray(3);
+        glVertexAttribPointer(3, 4, GL_FLOAT, GL_FALSE, getVertexPitchFromType(type), (GLvoid*)28);
+        glEnableVertexAttribArray(4);
+        glVertexAttribPointer(4, 4, GL_FLOAT, GL_FALSE, getVertexPitchFromType(type), (GLvoid*)44);
+        glEnableVertexAttribArray(5);
+        glVertexAttribIPointer(5, 4, GL_SHORT, getVertexPitchFromType(type), (GLvoid*)60);
+        glEnableVertexAttribArray(6);
+        glVertexAttribPointer(6, 4, GL_HALF_FLOAT, GL_FALSE, getVertexPitchFromType(type), (GLvoid*)68);
+        break;
     default:
         assert(0 && "Wrong vertex type");
     }
@@ -163,13 +206,21 @@ GLuint createVAO(GLuint vbo, GLuint idx, video::E_VERTEX_TYPE type)
 }   // createVAO
 
 // ----------------------------------------------------------------------------
-GLMesh allocateMeshBuffer(scene::IMeshBuffer* mb, const std::string& debug_name)
+GLMesh allocateMeshBuffer(scene::IMeshBuffer* mb, const std::string& debug_name,
+                          RenderInfo* render_info)
 {
     GLMesh result = {};
+    result.m_material = NULL;
+    result.m_render_info = NULL;
     if (!mb)
         return result;
     result.mb = mb;
-
+    if (render_info != NULL)
+    {
+        result.m_render_info = render_info;
+        result.m_material = material_manager->getMaterialFor(mb
+            ->getMaterial().getTexture(0), mb);
+    }
 #ifdef DEBUG
     result.debug_name = debug_name;
 #endif
@@ -194,8 +245,6 @@ GLMesh allocateMeshBuffer(scene::IMeshBuffer* mb, const std::string& debug_name)
     }
     result.VAOType = mb->getVertexType();
     result.Stride = getVertexPitchFromType(result.VAOType);
-
-
     result.IndexCount = mb->getIndexCount();
     switch (mb->getPrimitiveType())
     {
@@ -224,8 +273,8 @@ GLMesh allocateMeshBuffer(scene::IMeshBuffer* mb, const std::string& debug_name)
     }
     for (unsigned i = 0; i < 8; i++)
         result.textures[i] = mb->getMaterial().getTexture(i);
-    result.TextureMatrix = 0;
-    result.VAOType = mb->getVertexType();
+    result.texture_trans = core::vector2df(0.0f, 0.0f);
+
     return result;
 }   // allocateMeshBuffer
 
@@ -290,7 +339,7 @@ core::vector3df getWindDir()
     const float time = irr_driver->getDevice()->getTimer()->getTime() / 1000.0f;
     GrassShaderProvider *gsp = 
         (GrassShaderProvider *)Shaders::getCallback(ES_GRASS);
-    return (gsp->getSpeed() * cos(time)) * vector3df(1., 0., 0.);
+    return (gsp->getSpeed() * time * vector3df(1., 0., 0.));
 }   // getWindDir
 
 // ----------------------------------------------------------------------------
@@ -341,20 +390,17 @@ static void setTexture(GLMesh &mesh, unsigned i, bool is_srgb,
                    mat_name.c_str());
         // use unicolor texture to replace missing texture
         mesh.textures[i] = 
-                      getUnicolorTexture(video::SColor(255, 127, 127, 127));
+                      STKTexManager::getInstance()->getUnicolorTexture(video::SColor(255, 127, 127, 127));
     }
-    compressTexture(mesh.textures[i], is_srgb);
+#if !defined(USE_GLES2)
     if (CVS->isAZDOEnabled())
     {
         if (!mesh.TextureHandles[i])
         {
-            mesh.TextureHandles[i] = glGetTextureSamplerHandleARB(
-                getTextureGLuint(mesh.textures[i]),
-                Shaders::ObjectPass1Shader::getInstance()->m_sampler_ids[0]);
+            mesh.TextureHandles[i] = mesh.textures[i]->getHandle();
         }
-        if (!glIsTextureHandleResidentARB(mesh.TextureHandles[i]))
-            glMakeTextureHandleResidentARB(mesh.TextureHandles[i]);
     }
+#endif
 }   // setTexture
 
 // ----------------------------------------------------------------------------
@@ -390,18 +436,24 @@ void initTextures(GLMesh &mesh, Material::ShaderType mat)
     default:
     case Material::SHADERTYPE_SOLID:
     case Material::SHADERTYPE_ALPHA_TEST:
-    case Material::SHADERTYPE_VEGETATION:
     case Material::SHADERTYPE_SPHERE_MAP:
     case Material::SHADERTYPE_SOLID_UNLIT:
-        setTexture(mesh, 0, true, getShaderTypeName(mat));
-        setTexture(mesh, 1, false, getShaderTypeName(mat));
-        break;
-    case Material::SHADERTYPE_DETAIL_MAP:
-    case Material::SHADERTYPE_NORMAL_MAP:
+    case Material::SHADERTYPE_VEGETATION:
+    case Material::SHADERTYPE_SOLID_SKINNED_MESH:
+    case Material::SHADERTYPE_ALPHA_TEST_SKINNED_MESH:
+    case Material::SHADERTYPE_SOLID_UNLIT_SKINNED_MESH:
         setTexture(mesh, 0, true, getShaderTypeName(mat));
         setTexture(mesh, 1, false, getShaderTypeName(mat));
         setTexture(mesh, 2, false, getShaderTypeName(mat));
         break;
+    case Material::SHADERTYPE_DETAIL_MAP:
+    case Material::SHADERTYPE_NORMAL_MAP:
+    case Material::SHADERTYPE_NORMAL_MAP_SKINNED_MESH:
+        setTexture(mesh, 0, true, getShaderTypeName(mat));
+        setTexture(mesh, 1, false, getShaderTypeName(mat));
+        setTexture(mesh, 2, false, getShaderTypeName(mat));
+        setTexture(mesh, 3, false, getShaderTypeName(mat));
+        break;
     case Material::SHADERTYPE_SPLATTING:
         setTexture(mesh, 0, true, getShaderTypeName(mat));
         setTexture(mesh, 1, false, getShaderTypeName(mat));
@@ -410,6 +462,7 @@ void initTextures(GLMesh &mesh, Material::ShaderType mat)
         setTexture(mesh, 4, true, getShaderTypeName(mat));
         setTexture(mesh, 5, true, getShaderTypeName(mat));
         setTexture(mesh, 6, false, getShaderTypeName(mat));
+        setTexture(mesh, 7, false, getShaderTypeName(mat));
         break;
     }
 }   // initTextures
@@ -419,18 +472,18 @@ void initTexturesTransparent(GLMesh &mesh)
 {
     if (!mesh.textures[0])
     {
-        mesh.textures[0] = getUnicolorTexture(video::SColor(255, 255, 255, 255));
+        mesh.textures[0] = STKTexManager::getInstance()->getUnicolorTexture(video::SColor(255, 255, 255, 255));
     }
-    compressTexture(mesh.textures[0], true);
+#if !defined(USE_GLES2)
     if (CVS->isAZDOEnabled())
     {
         if (!mesh.TextureHandles[0])
         {
-            mesh.TextureHandles[0] = glGetTextureSamplerHandleARB(
-                getTextureGLuint(mesh.textures[0]),
-                Shaders::ObjectPass1Shader::getInstance()->m_sampler_ids[0]);
+            mesh.TextureHandles[0] = mesh.textures[0]->getHandle();
         }
-        if (!glIsTextureHandleResidentARB(mesh.TextureHandles[0]))
-            glMakeTextureHandleResidentARB(mesh.TextureHandles[0]);
     }
+#endif
 }   // initTexturesTransparent
+
+#endif   // !SERVER_ONLY
+
