@@ -15,25 +15,23 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+#ifndef SERVER_ONLY
+
 #include "graphics/stk_mesh_scene_node.hpp"
 
-#include "config/user_config.hpp"
-#include "graphics/callbacks.hpp"
 #include "graphics/central_settings.hpp"
-#include "graphics/camera.hpp"
-#include "graphics/glwrap.hpp"
 #include "graphics/irr_driver.hpp"
 #include "graphics/material_manager.hpp"
-#include "graphics/stk_mesh.hpp"
+#include "graphics/materials.hpp"
+#include "graphics/render_info.hpp"
+#include "graphics/rtts.hpp"
+#include "graphics/shaders.hpp"
+#include "graphics/vao_manager.hpp"
 #include "tracks/track.hpp"
 #include "modes/world.hpp"
-#include "utils/cpp2011.hpp"
-#include "utils/helpers.hpp"
-#include "utils/tuple.hpp"
 
-#include <ISceneManager.h>
 #include <IMaterialRenderer.h>
-
+#include <ISceneManager.h>
 
 // ============================================================================
 class ColorizeShader : public Shader<ColorizeShader, core::matrix4, 
@@ -54,18 +52,19 @@ STKMeshSceneNode::STKMeshSceneNode(irr::scene::IMesh* mesh, ISceneNode* parent,
     irr::s32 id, const std::string& debug_name,
     const irr::core::vector3df& position,
     const irr::core::vector3df& rotation,
-    const irr::core::vector3df& scale, bool createGLMeshes) :
+    const irr::core::vector3df& scale, bool createGLMeshes, RenderInfo* render_info, bool all_parts_colorized) :
     CMeshSceneNode(mesh, parent, mgr, id, position, rotation, scale)
 {
     isDisplacement = false;
     immediate_draw = false;
     update_each_frame = false;
     isGlow = false;
+    m_got_animated_matrix = false;
 
     m_debug_name = debug_name;
 
     if (createGLMeshes)
-        this->createGLMeshes();
+        this->createGLMeshes(render_info, all_parts_colorized);
 }
 
 void STKMeshSceneNode::setReloadEachFrame(bool val)
@@ -75,12 +74,46 @@ void STKMeshSceneNode::setReloadEachFrame(bool val)
         immediate_draw = true;
 }
 
-void STKMeshSceneNode::createGLMeshes()
+void STKMeshSceneNode::createGLMeshes(RenderInfo* render_info, bool all_parts_colorized)
 {
-    for (u32 i = 0; i<Mesh->getMeshBufferCount(); ++i)
+    const u32 mb_count = Mesh->getMeshBufferCount();
+    for (u32 i = 0; i < mb_count; ++i)
     {
         scene::IMeshBuffer* mb = Mesh->getMeshBuffer(i);
-        GLmeshes.push_back(allocateMeshBuffer(mb, m_debug_name));
+        bool affected = false;
+        RenderInfo* cur_ri = render_info;
+        if (!all_parts_colorized && mb && render_info)
+        {
+            if (render_info && !render_info->isStatic())
+            {
+                // Convert to static render info for each mesh buffer
+                assert(render_info->getNumberOfHue() == mb_count);
+                const float hue = render_info->getDynamicHue(i);
+                if (hue > 0.0f)
+                {
+                    cur_ri = new RenderInfo(hue);
+                    m_static_render_info.push_back(cur_ri);
+                    affected = true;
+                }
+                else
+                {
+                    cur_ri = NULL;
+                }
+            }
+            else
+            {
+                // Test if material is affected by static hue change
+                Material* m = material_manager->getMaterialFor(mb
+                    ->getMaterial().getTexture(0), mb);
+                if (m->isColorizable())
+                    affected = true;
+            }
+        }
+
+        assert(cur_ri ? cur_ri->isStatic() : true);
+        GLmeshes.push_back(allocateMeshBuffer(mb, m_debug_name,
+            affected || all_parts_colorized || (cur_ri &&
+            cur_ri->isTransparent()) ? cur_ri : NULL));
     }
     isMaterialInitialized = false;
     isGLInitialized = false;
@@ -125,7 +158,7 @@ void STKMeshSceneNode::drawGlow(const GLMesh &mesh)
 
     GLenum ptype = mesh.PrimitiveType;
     GLenum itype = mesh.IndexType;
-    size_t count = mesh.IndexCount;
+    unsigned int count = mesh.IndexCount;
     ColorizeShader::getInstance()->setUniforms(AbsoluteTransformation, video::SColorf(glowcolor.getRed() / 255.f, glowcolor.getGreen() / 255.f, glowcolor.getBlue() / 255.f));
     glDrawElementsBaseVertex(ptype, count, itype, (GLvoid *)mesh.vaoOffset, mesh.vaoBaseVertex);
 }
@@ -172,17 +205,19 @@ void STKMeshSceneNode::updateNoGL()
             }
 
             GLMesh &mesh = GLmeshes[i];
+            video::E_VERTEX_TYPE vt = mb->getVertexType();
             Material* material = material_manager->getMaterialFor(mb->getMaterial().getTexture(0), mb);
-            if (Mesh->getRenderType() == video::ERT_TRANSPARENT)
+            if (mesh.m_render_info != NULL && mesh.m_render_info->isTransparent() && !rnd->isTransparent())
             {
-                if (!immediate_draw)
-                    TransparentMesh[TM_ADDITIVE].push_back(&mesh);
+                assert(!immediate_draw);
+                if (mesh.VAOType == video::EVT_TANGENTS)
+                    TransparentMesh[TM_TRANSLUCENT_TAN].push_back(&mesh);
                 else
-                    additive = true;
+                    TransparentMesh[TM_TRANSLUCENT_STD].push_back(&mesh);
             }
             else if (rnd->isTransparent())
             {
-                TransparentMaterial TranspMat = getTransparentMaterialFromType(type, MaterialTypeParam, material);
+                TransparentMaterial TranspMat = getTransparentMaterialFromType(type, vt, MaterialTypeParam, material);
                 if (!immediate_draw)
                     TransparentMesh[TranspMat].push_back(&mesh);
                 else
@@ -194,7 +229,7 @@ void STKMeshSceneNode::updateNoGL()
                 Material* material2 = NULL;
                 if (mb->getMaterial().getTexture(1) != NULL)
                     material2 = material_manager->getMaterialFor(mb->getMaterial().getTexture(1), mb);
-                Material::ShaderType MatType = getMeshMaterialFromType(type, mb->getVertexType(), material, material2);
+                Material::ShaderType MatType = getMeshMaterialFromType(type, vt, material, material2);
                 if (!immediate_draw)
                     MeshSolidMaterial[MatType].push_back(&mesh);
             }
@@ -205,9 +240,19 @@ void STKMeshSceneNode::updateNoGL()
     for (u32 i = 0; i < Mesh->getMeshBufferCount(); ++i)
     {
         scene::IMeshBuffer* mb = Mesh->getMeshBuffer(i);
-        if (!mb)
-            continue;
-        GLmeshes[i].TextureMatrix = getMaterial(i).getTextureMatrix(0);
+        if (mb != NULL)
+        {
+            // Test if texture matrix needs to be updated every frame
+            const core::matrix4& mat = getMaterial(i).getTextureMatrix(0);
+            if (mat.isIdentity() && !m_got_animated_matrix)
+                continue;
+            else
+            {
+                m_got_animated_matrix = true;
+                GLmeshes[i].texture_trans.X = mat[8];
+                GLmeshes[i].texture_trans.Y = mat[9];
+            }
+        }
     }
 }
 
@@ -299,33 +344,29 @@ void STKMeshSceneNode::render()
         glDisable(GL_CULL_FACE);
         if (update_each_frame)
             updatevbo();
-        Shaders::ObjectPass1Shader::getInstance()->use();
+        ObjectPass1Shader::getInstance()->use();
         // Only untextured
         for (unsigned i = 0; i < GLmeshes.size(); i++)
         {
-            irr_driver->IncreaseObjectCount();
+            irr_driver->increaseObjectCount();
             GLMesh &mesh = GLmeshes[i];
             GLenum ptype = mesh.PrimitiveType;
             GLenum itype = mesh.IndexType;
-            size_t count = mesh.IndexCount;
+            unsigned int count = mesh.IndexCount;
 
-            compressTexture(mesh.textures[0], true);
+#if !defined(USE_GLES2)
             if (CVS->isAZDOEnabled())
             {
                 if (!mesh.TextureHandles[0])
                 {
-                    mesh.TextureHandles[0] = 
-                        glGetTextureSamplerHandleARB(getTextureGLuint(mesh.textures[0]), 
-                                                     Shaders::ObjectPass1Shader
-                                                     ::getInstance()->m_sampler_ids[0]);
+                    mesh.TextureHandles[0] = mesh.textures[0]->getHandle();
                 }
-                if (!glIsTextureHandleResidentARB(mesh.TextureHandles[0]))
-                    glMakeTextureHandleResidentARB(mesh.TextureHandles[0]);
-                Shaders::ObjectPass1Shader::getInstance()->setTextureHandles(mesh.TextureHandles[0]);
+                ObjectPass1Shader::getInstance()->setTextureHandles(mesh.TextureHandles[0]);
             }
             else
-                Shaders::ObjectPass1Shader::getInstance()->setTextureUnits(getTextureGLuint(mesh.textures[0]));
-            Shaders::ObjectPass1Shader::getInstance()->setUniforms(AbsoluteTransformation, invmodel);
+#endif
+                ObjectPass1Shader::getInstance()->setTextureUnits(mesh.textures[0]->getOpenGLTextureName());
+            ObjectPass1Shader::getInstance()->setUniforms(AbsoluteTransformation, invmodel);
             assert(mesh.vao);
             glBindVertexArray(mesh.vao);
             glDrawElements(ptype, count, itype, 0);
@@ -343,62 +384,58 @@ void STKMeshSceneNode::render()
         glDisable(GL_CULL_FACE);
         if (update_each_frame && !CVS->isDefferedEnabled())
             updatevbo();
-        Shaders::ObjectPass2Shader::getInstance()->use();
+        ObjectPass2Shader::getInstance()->use();
         // Only untextured
         for (unsigned i = 0; i < GLmeshes.size(); i++)
         {
-            irr_driver->IncreaseObjectCount();
+            irr_driver->increaseObjectCount();
             GLMesh &mesh = GLmeshes[i];
             GLenum ptype = mesh.PrimitiveType;
             GLenum itype = mesh.IndexType;
-            size_t count = mesh.IndexCount;
+            unsigned int count = mesh.IndexCount;
 
+#if !defined(USE_GLES2)
             if (CVS->isAZDOEnabled())
             {
                 GLuint64 DiffuseHandle =
                     glGetTextureSamplerHandleARB(irr_driver->getRenderTargetTexture(RTT_DIFFUSE), 
-                                                 Shaders::ObjectPass2Shader
+                                                 ObjectPass2Shader
                                                     ::getInstance()->m_sampler_ids[0]);
-                if (!glIsTextureHandleResidentARB(DiffuseHandle))
-                    glMakeTextureHandleResidentARB(DiffuseHandle);
 
                 GLuint64 SpecularHandle =
                     glGetTextureSamplerHandleARB(irr_driver->getRenderTargetTexture(RTT_SPECULAR),
-                                        Shaders::ObjectPass2Shader::getInstance()->m_sampler_ids[1]);
-                if (!glIsTextureHandleResidentARB(SpecularHandle))
-                    glMakeTextureHandleResidentARB(SpecularHandle);
+                                        ObjectPass2Shader::getInstance()->m_sampler_ids[1]);
 
                 GLuint64 SSAOHandle =
                     glGetTextureSamplerHandleARB(irr_driver->getRenderTargetTexture(RTT_HALF1_R),
-                                    Shaders::ObjectPass2Shader::getInstance()->m_sampler_ids[2]);
-                if (!glIsTextureHandleResidentARB(SSAOHandle))
-                    glMakeTextureHandleResidentARB(SSAOHandle);
+                                    ObjectPass2Shader::getInstance()->m_sampler_ids[2]);
 
                 if (!mesh.TextureHandles[0])
-                    mesh.TextureHandles[0] = 
-                    glGetTextureSamplerHandleARB(getTextureGLuint(mesh.textures[0]), 
-                                Shaders::ObjectPass2Shader::getInstance()->m_sampler_ids[0]);
-                if (!glIsTextureHandleResidentARB(mesh.TextureHandles[0]))
-                    glMakeTextureHandleResidentARB(mesh.TextureHandles[0]);
+                    mesh.TextureHandles[0] = mesh.textures[0]->getHandle();
+
                 if (!mesh.TextureHandles[1])
-                    mesh.TextureHandles[1] =
-                    glGetTextureSamplerHandleARB(getTextureGLuint(mesh.textures[1]),
-                                Shaders::ObjectPass2Shader::getInstance()->m_sampler_ids[0]);
-                if (!glIsTextureHandleResidentARB(mesh.TextureHandles[1]))
-                    glMakeTextureHandleResidentARB(mesh.TextureHandles[1]);
-                Shaders::ObjectPass2Shader::getInstance()
+                    mesh.TextureHandles[1] = mesh.textures[1]->getHandle();
+
+                if (!mesh.TextureHandles[2])
+                    mesh.TextureHandles[2] = mesh.textures[2]->getHandle();
+
+                ObjectPass2Shader::getInstance()
                     ->setTextureHandles(DiffuseHandle, SpecularHandle, SSAOHandle,
-                                        mesh.TextureHandles[0], mesh.TextureHandles[1]);
+                                        mesh.TextureHandles[0], mesh.TextureHandles[1],
+                                        mesh.TextureHandles[2]);
             }
             else
-                Shaders::ObjectPass2Shader::getInstance()->setTextureUnits(
+#endif
+                ObjectPass2Shader::getInstance()->setTextureUnits(
                 irr_driver->getRenderTargetTexture(RTT_DIFFUSE),
                 irr_driver->getRenderTargetTexture(RTT_SPECULAR),
                 irr_driver->getRenderTargetTexture(RTT_HALF1_R),
-                getTextureGLuint(mesh.textures[0]),
-                getTextureGLuint(mesh.textures[1]));
-            Shaders::ObjectPass2Shader::getInstance()->setUniforms(AbsoluteTransformation,
-                                                                   mesh.TextureMatrix);
+                mesh.textures[0]->getOpenGLTextureName(),
+                mesh.textures[1]->getOpenGLTextureName(),
+                mesh.textures[2]->getOpenGLTextureName());
+            ObjectPass2Shader::getInstance()->setUniforms(AbsoluteTransformation,
+                                                                   mesh.texture_trans,
+                                                                   core::vector2df(0.0f, 0.0f));
             assert(mesh.vao);
             glBindVertexArray(mesh.vao);
             glDrawElements(ptype, count, itype, 0);
@@ -437,50 +474,46 @@ void STKMeshSceneNode::render()
             else
                 glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
 
-            if (World::getWorld() && World::getWorld()->isFogEnabled())
+            const Track * const track = Track::getCurrentTrack();
+            if (track && track->isFogEnabled())
             {
                 Shaders::TransparentFogShader::getInstance()->use();
                 for (unsigned i = 0; i < GLmeshes.size(); i++)
                 {
                     GLMesh &mesh = GLmeshes[i];
-                    irr_driver->IncreaseObjectCount();
+                    irr_driver->increaseObjectCount();
                     GLenum ptype = mesh.PrimitiveType;
                     GLenum itype = mesh.IndexType;
-                    size_t count = mesh.IndexCount;
-
-                    const Track * const track = World::getWorld()->getTrack();
+                    unsigned int count = mesh.IndexCount;
 
                     // This function is only called once per frame - thus no need for setters.
                     const float fogmax = track->getFogMax();
                     const float startH = track->getFogStartHeight();
-                    const float endH = track->getFogEndHeight();
-                    const float start = track->getFogStart();
-                    const float end = track->getFogEnd();
+                    const float endH   = track->getFogEndHeight();
+                    const float start  = track->getFogStart();
+                    const float end    = track->getFogEnd();
                     const video::SColor tmpcol = track->getFogColor();
 
                     video::SColorf col(tmpcol.getRed() / 255.0f,
-                        tmpcol.getGreen() / 255.0f,
-                        tmpcol.getBlue() / 255.0f);
+                                       tmpcol.getGreen() / 255.0f,
+                                       tmpcol.getBlue() / 255.0f);
 
-                    compressTexture(mesh.textures[0], true);
+#if !defined(USE_GLES2)
                     if (CVS->isAZDOEnabled())
                     {
                         if (!mesh.TextureHandles[0])
-                            mesh.TextureHandles[0] =
-                            glGetTextureSamplerHandleARB(getTextureGLuint(mesh.textures[0]),
-                                   Shaders::TransparentFogShader::getInstance()->m_sampler_ids[0]);
-                        if (!glIsTextureHandleResidentARB(mesh.TextureHandles[0]))
-                            glMakeTextureHandleResidentARB(mesh.TextureHandles[0]);
+                            mesh.TextureHandles[0] = mesh.textures[0]->getHandle();
                         Shaders::TransparentFogShader::getInstance()
                                     ->setTextureHandles(mesh.TextureHandles[0]);
                     }
                     else
+#endif
                     {
                         Shaders::TransparentFogShader::getInstance()
-                            ->setTextureUnits(getTextureGLuint(mesh.textures[0]));
+                            ->setTextureUnits(mesh.textures[0]->getOpenGLTextureName());
                     }
                     Shaders::TransparentFogShader::getInstance()
-                           ->setUniforms(AbsoluteTransformation, mesh.TextureMatrix,
+                           ->setUniforms(AbsoluteTransformation, mesh.texture_trans,
                                          fogmax, startH, endH, start, end, col);
 
                     assert(mesh.vao);
@@ -494,27 +527,24 @@ void STKMeshSceneNode::render()
                 Shaders::TransparentShader::getInstance()->use();
                 for (unsigned i = 0; i < GLmeshes.size(); i++)
                 {
-                    irr_driver->IncreaseObjectCount();
+                    irr_driver->increaseObjectCount();
                     GLMesh &mesh = GLmeshes[i];
                     GLenum ptype = mesh.PrimitiveType;
                     GLenum itype = mesh.IndexType;
-                    size_t count = mesh.IndexCount;
+                    unsigned int count = mesh.IndexCount;
 
-                    compressTexture(mesh.textures[0], true);
+#if !defined(USE_GLES2)
                     if (CVS->isAZDOEnabled())
                     {
                         if (!mesh.TextureHandles[0])
-                            mesh.TextureHandles[0] =
-                            glGetTextureSamplerHandleARB(getTextureGLuint(mesh.textures[0]),
-                            Shaders::TransparentShader::getInstance()->m_sampler_ids[0]);
-                        if (!glIsTextureHandleResidentARB(mesh.TextureHandles[0]))
-                            glMakeTextureHandleResidentARB(mesh.TextureHandles[0]);
+                            mesh.TextureHandles[0] = mesh.textures[0]->getHandle();
                         Shaders::TransparentShader::getInstance()->setTextureHandles(mesh.TextureHandles[0]);
                     }
                     else
-                        Shaders::TransparentShader::getInstance()->setTextureUnits(getTextureGLuint(mesh.textures[0]));
+#endif
+                        Shaders::TransparentShader::getInstance()->setTextureUnits(mesh.textures[0]->getOpenGLTextureName());
 
-                    Shaders::TransparentShader::getInstance()->setUniforms(AbsoluteTransformation, mesh.TextureMatrix);
+                    Shaders::TransparentShader::getInstance()->setUniforms(AbsoluteTransformation, mesh.texture_trans, 1.0f);
                     assert(mesh.vao);
                     glBindVertexArray(mesh.vao);
                     glDrawElements(ptype, count, itype, 0);
@@ -525,3 +555,20 @@ void STKMeshSceneNode::render()
         }
     }
 }
+
+void STKMeshSceneNode::setIsDisplacement(bool v)
+{
+    isDisplacement = v;
+    for (u32 i = 0; i < Mesh->getMeshBufferCount(); ++i)
+    {
+        scene::IMeshBuffer* mb = Mesh->getMeshBuffer(i);
+        if (!mb)
+            continue;
+
+        if (isDisplacement)
+            mb->getMaterial().MaterialType = Shaders::getShader(ES_DISPLACE);
+    }
+}
+
+#endif   // !SERVER_ONLY
+
