@@ -39,9 +39,6 @@ namespace irr
 #include "utils/constants.hpp"
 #include "utils/ptr_vector.hpp"
 
-#include "guiengine/ft_environment.hpp"
-#include "guiengine/glyph_page_creator.hpp"
-
 /**
  * \ingroup guiengine
  * \brief Contains all GUI engine related classes and functions
@@ -84,8 +81,6 @@ namespace GUIEngine
     {
         extern irr::gui::IGUIEnvironment* g_env;
         extern Skin* g_skin;
-        extern FTEnvironment*  g_ft_env;
-        extern GlyphPageCreator*  g_gp_creator;
         extern irr::gui::ScalableFont* g_small_font;
         extern irr::gui::ScalableFont* g_font;
         extern irr::gui::ScalableFont* g_outline_font;
@@ -177,18 +172,6 @@ namespace GUIEngine
       */
     inline Skin*                      getSkin()          { return Private::g_skin;           }
 
-    /**
-      * \pre GUIEngine::init must have been called first
-      * \return       the freetype and library with face
-      */
-    inline FTEnvironment*             getFreetype()      { return Private::g_ft_env;         }
-
-    /**
-      * \pre GUIEngine::init must have been called first
-      * \return       the glyph page creator, useful to create a glyph page from individual char
-      */
-    inline GlyphPageCreator*          getGlyphPageCreator() { return Private::g_gp_creator;  }
-
     Screen*                           getScreenNamed(const char* name);
 
     /** \return the height of the title font in pixels */
@@ -265,11 +248,6 @@ namespace GUIEngine
       */
     void reloadSkin();
 
-    /**
-      * \brief call when translation in user config was updated for freetype rendering STK
-      */
-    void cleanHollowCopyFont();
-    void reloadHollowCopyFont(irr::gui::ScalableFont*);
 }
 
 #endif
