@@ -17,12 +17,16 @@
 
 #include "guiengine/event_handler.hpp"
 
+#include "audio/music_manager.hpp"
+#include "audio/sfx_manager.hpp"
 #include "config/user_config.hpp"
 #include "graphics/irr_driver.hpp"
+#include "graphics/stk_tex_manager.hpp"
 #include "guiengine/abstract_state_manager.hpp"
 #include "guiengine/engine.hpp"
 #include "guiengine/modaldialog.hpp"
 #include "guiengine/screen.hpp"
+#include "guiengine/screen_keyboard.hpp"
 #include "guiengine/widget.hpp"
 #include "guiengine/widgets/list_widget.hpp"
 #include "guiengine/widgets/ribbon_widget.hpp"
@@ -39,6 +43,10 @@
 
 #include <iostream>
 
+#ifdef ANDROID
+#include <android_native_app_glue.h>
+#endif
+
 using GUIEngine::EventHandler;
 using GUIEngine::EventPropagation;
 
@@ -148,6 +156,40 @@ bool EventHandler::OnEvent (const SEvent &event)
     {
         return onGUIEvent(event) == EVENT_BLOCK;
     }
+#ifdef ANDROID    
+    else if (event.EventType == EET_SYSTEM_EVENT &&
+             event.SystemEvent.EventType == ESET_ANDROID_CMD)
+    {
+        int cmd = event.SystemEvent.AndroidCmd.Cmd;
+        
+        IrrlichtDevice* device = irr_driver->getDevice();
+        assert(device != NULL);
+        
+        if (cmd == APP_CMD_PAUSE || cmd == APP_CMD_LOST_FOCUS)
+        {
+            // Make sure that pause/unpause is executed only once
+            if (device->isWindowMinimized() == device->isWindowFocused())
+            {
+                music_manager->pauseMusic();
+                SFXManager::get()->pauseAll();
+            }
+        }
+        else if (cmd == APP_CMD_RESUME || cmd == APP_CMD_GAINED_FOCUS)
+        {
+            if (device->isWindowActive())
+            {
+                music_manager->resumeMusic();
+                SFXManager::get()->resumeAll();
+            }
+        }
+        else if (cmd == APP_CMD_LOW_MEMORY)
+        {
+            Log::warn("EventHandler", "Low memory event received");
+        }
+
+        return false;
+    }
+#endif
     else if (GUIEngine::getStateManager()->getGameState() != GUIEngine::GAME &&
              event.EventType != EET_KEY_INPUT_EVENT && event.EventType != EET_JOYSTICK_INPUT_EVENT &&
              event.EventType != EET_LOG_TEXT_EVENT)
@@ -155,8 +197,10 @@ bool EventHandler::OnEvent (const SEvent &event)
         return false; // EVENT_LET
     }
     else if (event.EventType == EET_MOUSE_INPUT_EVENT ||
+             event.EventType == EET_TOUCH_INPUT_EVENT ||
              event.EventType == EET_KEY_INPUT_EVENT   ||
-             event.EventType == EET_JOYSTICK_INPUT_EVENT)
+             event.EventType == EET_JOYSTICK_INPUT_EVENT ||
+             event.EventType == EET_ACCELEROMETER_EVENT)
     {
         // Remember the mouse position
         if (event.EventType == EET_MOUSE_INPUT_EVENT &&
@@ -193,7 +237,7 @@ bool EventHandler::OnEvent (const SEvent &event)
         const EventPropagation blockPropagation = input_manager->input(event);
 
         if (event.EventType == EET_KEY_INPUT_EVENT &&
-            event.KeyInput.Key == irr::KEY_TAB)
+            event.KeyInput.Key == irr::IRR_KEY_TAB)
         {
             // block all tab events, if we let them go, irrlicht will try
             // to apply its own focus code
@@ -219,7 +263,7 @@ bool EventHandler::OnEvent (const SEvent &event)
 #else
             return true; // EVENT_BLOCK
 #endif
-            const std::string &error_info = irr_driver->getTextureErrorMessage();
+            const std::string &error_info = STKTexManager::getInstance()->getTextureErrorMessage();
             if (event.LogEvent.Level == irr::ELL_WARNING)
             {
                 if(error_info.size()>0)
@@ -387,7 +431,7 @@ const bool NAVIGATION_DEBUG = false;
  */
 void EventHandler::navigate(const int playerID, Input::InputType type, const bool pressedDown, const bool reverse)
 {
-    IGUIElement *el = NULL, *closest = NULL;
+    IGUIElement *el = NULL;
 
     if (type == Input::IT_STICKBUTTON && !pressedDown)
         return;
@@ -436,9 +480,15 @@ void EventHandler::navigate(const int playerID, Input::InputType type, const boo
     }
 
     // don't allow navigating to any widget when a dialog is shown; only navigate to widgets in the dialog
-    if (ModalDialog::isADialogActive() && !ModalDialog::getCurrent()->isMyIrrChild(el))
+    if (ScreenKeyboard::isActive())
     {
-        el = NULL;
+        if (!ScreenKeyboard::getCurrent()->isMyIrrChild(el))
+            el = NULL;
+    }
+    else if (ModalDialog::isADialogActive())
+    {
+        if (!ModalDialog::getCurrent()->isMyIrrChild(el))
+            el = NULL;
     }
 
     bool found = false;
@@ -450,7 +500,7 @@ void EventHandler::navigate(const int playerID, Input::InputType type, const boo
         // Down: if the current widget is e.g. 5, search for widget 6, 7, 8, 9, ..., 15 (up to 10 IDs may be missing)
         for (int n = 1; n < 10 && !found; n++)
         {
-            closest = GUIEngine::getGUIEnv()->getRootGUIElement()->getElementFromId(el->getTabOrder() + (reverse ? -n : n), true);
+            IGUIElement *closest = GUIEngine::getGUIEnv()->getRootGUIElement()->getElementFromId(el->getTabOrder() + (reverse ? -n : n), true);
 
             if (closest != NULL && Widget::isFocusableId(closest->getID()))
             {
@@ -459,8 +509,16 @@ void EventHandler::navigate(const int playerID, Input::InputType type, const boo
                 if (playerID != PLAYER_ID_GAME_MASTER && !closestWidget->m_supports_multiplayer) return;
 
                 // if a dialog is shown, restrict to items in the dialog
-                if (ModalDialog::isADialogActive() && !ModalDialog::getCurrent()->isMyChild(closestWidget))
-                    continue;
+                if (ScreenKeyboard::isActive())
+                {
+                    if (!ScreenKeyboard::getCurrent()->isMyChild(closestWidget))
+                        continue;
+                }
+                else if (ModalDialog::isADialogActive())
+                {
+                    if (!ModalDialog::getCurrent()->isMyChild(closestWidget))
+                        continue;
+                }
 
                 if (NAVIGATION_DEBUG)
                 {
@@ -495,7 +553,12 @@ void EventHandler::navigate(const int playerID, Input::InputType type, const boo
         // select the last/first widget
         Widget* wrapWidget = NULL;
 
-        if (ModalDialog::isADialogActive())
+        if (ScreenKeyboard::isActive())
+        {
+            wrapWidget = reverse ? ScreenKeyboard::getCurrent()->getLastWidget():
+                ScreenKeyboard::getCurrent()->getFirstWidget();
+        }
+        else if (ModalDialog::isADialogActive())
         {
             wrapWidget = reverse ? ModalDialog::getCurrent()->getLastWidget() :
                 ModalDialog::getCurrent()->getFirstWidget();
@@ -516,6 +579,14 @@ void EventHandler::navigate(const int playerID, Input::InputType type, const boo
 
 void EventHandler::sendEventToUser(GUIEngine::Widget* widget, std::string& name, const int playerID)
 {
+    if (ScreenKeyboard::isActive())
+    {
+        if (ScreenKeyboard::getCurrent()->processEvent(widget->m_properties[PROP_ID]) == EVENT_BLOCK)
+        {
+            return;
+        }
+    }
+    
     if (ModalDialog::isADialogActive())
     {
         if (ModalDialog::getCurrent()->processEvent(widget->m_properties[PROP_ID]) == EVENT_BLOCK)
@@ -532,10 +603,19 @@ void EventHandler::sendEventToUser(GUIEngine::Widget* widget, std::string& name,
 
 EventPropagation EventHandler::onWidgetActivated(GUIEngine::Widget* w, const int playerID)
 {
-    if (!w->isActivated()) return EVENT_BLOCK;
+    if (w->onActivationInput(playerID) == EVENT_BLOCK)
+        return EVENT_BLOCK;
 
     Widget* parent = w->m_event_handler;
     
+    if (ScreenKeyboard::isActive())
+    {
+        if (ScreenKeyboard::getCurrent()->processEvent(w->m_properties[PROP_ID]) == EVENT_BLOCK)
+        {
+            return EVENT_BLOCK;
+        }
+    }
+
     if (ModalDialog::isADialogActive() && (parent == NULL || parent->m_type != GUIEngine::WTYPE_RIBBON))
     {
         if (ModalDialog::getCurrent()->processEvent(w->m_properties[PROP_ID]) == EVENT_BLOCK)
@@ -627,10 +707,23 @@ EventPropagation EventHandler::onGUIEvent(const SEvent& event)
                 if (!w->isFocusable() || !w->isActivated()) return GUIEngine::EVENT_BLOCK;
 
                 // When a modal dialog is shown, don't select widgets out of the dialog
-                if (ModalDialog::isADialogActive() && !ModalDialog::getCurrent()->isMyChild(w))
+                if (ScreenKeyboard::isActive())
                 {
                     // check for parents too before discarding event
-                    if (w->m_event_handler != NULL)
+                    if (!ScreenKeyboard::getCurrent()->isMyChild(w) && 
+                        w->m_event_handler != NULL)
+                    {
+                        if (!ScreenKeyboard::getCurrent()->isMyChild(w->m_event_handler))
+                        {
+                            break;
+                        }
+                    }
+                }
+                else if (ModalDialog::isADialogActive())
+                {
+                    // check for parents too before discarding event
+                    if (!ModalDialog::getCurrent()->isMyChild(w) && 
+                        w->m_event_handler != NULL)
                     {
                         if (!ModalDialog::getCurrent()->isMyChild(w->m_event_handler))
                         {
@@ -728,7 +821,8 @@ EventPropagation EventHandler::onGUIEvent(const SEvent& event)
             {
                 // currently, enter pressed in text ctrl events can only happen in dialogs.
                 // FIXME : find a cleaner way to route the event to its proper location
-                if (ModalDialog::isADialogActive()) ModalDialog::onEnterPressed();
+                if (!ScreenKeyboard::isActive() && ModalDialog::isADialogActive()) 
+                    ModalDialog::onEnterPressed();
                 break;
             }
             default:
