@@ -34,9 +34,17 @@ namespace MessageQueue
      *  different look. This type is used to sort the messages, so it is
      *  important that messages that need to be shown as early as possible
      *  will be listed last (i.e. have highest priority). */
-    enum MessageType { MT_FRIEND, MT_ACHIEVEMENT, MT_ERROR, MT_GENERIC};
+    enum MessageType
+    {
+        MT_FRIEND,
+        MT_ACHIEVEMENT,
+        MT_ERROR,
+        MT_GENERIC,
+        MT_PROGRESS
+    };
 
     void add(MessageType mt, const core::stringw &message);
+    void showProgressBar(int progress, const wchar_t* msg);
     void updatePosition();
     void update(float dt);
 
