@@ -12,11 +12,17 @@
 //#include "os.h"
 #include "Keycodes.h"
 
+#include "config/user_config.hpp"
 #include "graphics/2dutils.hpp"
+#include "graphics/irr_driver.hpp"
+#include "guiengine/screen_keyboard.hpp"
 #include "utils/string_utils.hpp"
 #include "utils/translation.hpp"
 #include "utils/time.hpp"
 
+#include "../../../lib/irrlicht/include/IrrCompileConfig.h"
+#include "../../../lib/irrlicht/source/Irrlicht/CIrrDeviceLinux.h"
+
 /*
     todo:
     optional scrollbars
@@ -96,11 +102,21 @@ CGUIEditBox::CGUIEditBox(const wchar_t* text, bool border,
 //! destructor
 CGUIEditBox::~CGUIEditBox()
 {
+#ifndef SERVER_ONLY
     if (OverrideFont)
         OverrideFont->drop();
 
     if (Operator)
         Operator->drop();
+#ifdef _IRR_COMPILE_WITH_X11_DEVICE_
+    if (irr_driver->getDevice()->getType() == irr::EIDT_X11)
+    {
+        CIrrDeviceLinux* dl = dynamic_cast<CIrrDeviceLinux*>(
+                                                       irr_driver->getDevice());
+        dl->setIMEEnable(false);
+    }
+#endif
+#endif
 }
 
 
@@ -228,9 +244,9 @@ void CGUIEditBox::setTextAlignment(EGUI_ALIGNMENT horizontal, EGUI_ALIGNMENT ver
 //! called if an event happened.
 bool CGUIEditBox::OnEvent(const SEvent& event)
 {
+#ifndef SERVER_ONLY
     if (isEnabled())
     {
-
         switch(event.EventType)
         {
         case EET_GUI_EVENT:
@@ -241,7 +257,27 @@ bool CGUIEditBox::OnEvent(const SEvent& event)
                     MouseMarking = false;
                     setTextMarkers(0,0);
                 }
+#ifdef _IRR_COMPILE_WITH_X11_DEVICE_
+                if (irr_driver->getDevice()->getType() == irr::EIDT_X11)
+                {
+                    CIrrDeviceLinux* dl = dynamic_cast<CIrrDeviceLinux*>(
+                                                       irr_driver->getDevice());
+                    dl->setIMEEnable(false);
+                }
+#endif
             }
+#ifdef _IRR_COMPILE_WITH_X11_DEVICE_
+            else if (event.GUIEvent.EventType == EGET_ELEMENT_FOCUSED)
+            {
+                if (irr_driver->getDevice()->getType() == irr::EIDT_X11)
+                {
+                    CIrrDeviceLinux* dl = dynamic_cast<CIrrDeviceLinux*>(
+                                                       irr_driver->getDevice());
+                    dl->setIMEEnable(true);
+                    dl->setIMELocation(calculateICPos());
+                }
+            }
+#endif
             break;
 #if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
         case EET_IMPUT_METHOD_EVENT:
@@ -261,7 +297,7 @@ bool CGUIEditBox::OnEvent(const SEvent& event)
             break;
         }
     }
-
+#endif
     return IGUIElement::OnEvent(event);
 }
 
@@ -288,12 +324,12 @@ bool CGUIEditBox::processKey(const SEvent& event)
 
         switch(event.KeyInput.Key)
         {
-        case KEY_KEY_A:
+        case IRR_KEY_A:
             // select all
             newMarkBegin = 0;
             newMarkEnd = Text.size();
             break;
-        case KEY_KEY_C:
+        case IRR_KEY_C:
             // copy to clipboard
             if (!PasswordBox && Operator && MarkBegin != MarkEnd)
             {
@@ -309,7 +345,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
 #endif
             }
             break;
-        case KEY_KEY_X:
+        case IRR_KEY_X:
             // cut to the clipboard
             if (!PasswordBox && Operator && MarkBegin != MarkEnd)
             {
@@ -341,7 +377,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
             }
             break;
 
-        case KEY_KEY_V:
+        case IRR_KEY_V:
             if ( !isEnabled() )
                 break;
 
@@ -412,7 +448,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
                 textChanged = true;
             }
             break;
-        case KEY_HOME:
+        case IRR_KEY_HOME:
             if (!m_rtl)
             {
                 // move/highlight to start of text
@@ -430,7 +466,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
                 }
             }
             break;
-        case KEY_END:
+        case IRR_KEY_END:
             if (!m_rtl)
             {
                 // move/highlight to end of text
@@ -457,33 +493,33 @@ bool CGUIEditBox::processKey(const SEvent& event)
     switch(event.KeyInput.Key)
     {
             /*
-        case KEY_KEY_Q:
+        case IRR_KEY_Q:
             inputChar(L'\u05DC');
             textChanged = true;
             return true;
-        case KEY_KEY_W:
+        case IRR_KEY_W:
             inputChar(L'\u05DB');
             textChanged = true;
             return true;
-        case KEY_KEY_E:
+        case IRR_KEY_E:
             inputChar(L'\u05DA');
             textChanged = true;
             return true;
-        case KEY_KEY_R:
+        case IRR_KEY_R:
             inputChar(L'\u05D9');
             textChanged = true;
             return true;
-        case KEY_KEY_T:
+        case IRR_KEY_T:
             inputChar(L'\u05D8');
             textChanged = true;
             return true;
-        case KEY_KEY_Y:
+        case IRR_KEY_Y:
             inputChar(L'\u05D7');
             textChanged = true;
             return true;
             */
 
-    case KEY_END:
+    case IRR_KEY_END:
         if (!m_rtl)
         {
             s32 p = Text.size();
@@ -511,7 +547,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
             BlinkStartTime = getTime();
         }
         break;
-    case KEY_HOME:
+    case IRR_KEY_HOME:
         if (!m_rtl)
         {
 
@@ -537,7 +573,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
             BlinkStartTime = getTime();
         }
         break;
-    case KEY_RETURN:
+    case IRR_KEY_RETURN:
         if (MultiLine)
         {
             inputChar(L'\n');
@@ -548,7 +584,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
             sendGuiEvent( EGET_EDITBOX_ENTER );
         }
         break;
-    case KEY_LEFT:
+    case IRR_KEY_LEFT:
         if (!m_rtl)
         {
             if (event.KeyInput.Shift)
@@ -572,7 +608,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
         }
         break;
 
-    case KEY_RIGHT:
+    case IRR_KEY_RIGHT:
         if (!m_rtl)
         {
             if (event.KeyInput.Shift)
@@ -595,7 +631,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
             BlinkStartTime = getTime();
         }
         break;
-    case KEY_UP:
+    case IRR_KEY_UP:
         if (MultiLine || (WordWrap && BrokenText.size() > 1) )
         {
             s32 lineNo = getLineFromPos(CursorPos);
@@ -626,7 +662,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
             return false;
         }
         break;
-    case KEY_DOWN:
+    case IRR_KEY_DOWN:
         if (MultiLine || (WordWrap && BrokenText.size() > 1) )
         {
             s32 lineNo = getLineFromPos(CursorPos);
@@ -658,7 +694,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
         }
         break;
 
-    case KEY_BACK:
+    case IRR_KEY_BACK:
         if ( !isEnabled() )
             break;
 
@@ -698,7 +734,7 @@ bool CGUIEditBox::processKey(const SEvent& event)
             textChanged = true;
         }
         break;
-    case KEY_DELETE:
+    case IRR_KEY_DELETE:
         if ( !isEnabled() )
             break;
 
@@ -736,33 +772,33 @@ bool CGUIEditBox::processKey(const SEvent& event)
         }
         break;
 
-    case KEY_ESCAPE:
-    case KEY_TAB:
-    case KEY_SHIFT:
-    case KEY_F1:
-    case KEY_F2:
-    case KEY_F3:
-    case KEY_F4:
-    case KEY_F5:
-    case KEY_F6:
-    case KEY_F7:
-    case KEY_F8:
-    case KEY_F9:
-    case KEY_F10:
-    case KEY_F11:
-    case KEY_F12:
-    case KEY_F13:
-    case KEY_F14:
-    case KEY_F15:
-    case KEY_F16:
-    case KEY_F17:
-    case KEY_F18:
-    case KEY_F19:
-    case KEY_F20:
-    case KEY_F21:
-    case KEY_F22:
-    case KEY_F23:
-    case KEY_F24:
+    case IRR_KEY_ESCAPE:
+    case IRR_KEY_TAB:
+    case IRR_KEY_SHIFT:
+    case IRR_KEY_F1:
+    case IRR_KEY_F2:
+    case IRR_KEY_F3:
+    case IRR_KEY_F4:
+    case IRR_KEY_F5:
+    case IRR_KEY_F6:
+    case IRR_KEY_F7:
+    case IRR_KEY_F8:
+    case IRR_KEY_F9:
+    case IRR_KEY_F10:
+    case IRR_KEY_F11:
+    case IRR_KEY_F12:
+    case IRR_KEY_F13:
+    case IRR_KEY_F14:
+    case IRR_KEY_F15:
+    case IRR_KEY_F16:
+    case IRR_KEY_F17:
+    case IRR_KEY_F18:
+    case IRR_KEY_F19:
+    case IRR_KEY_F20:
+    case IRR_KEY_F21:
+    case IRR_KEY_F22:
+    case IRR_KEY_F23:
+    case IRR_KEY_F24:
         // ignore these keys
         return false;
 
@@ -786,29 +822,29 @@ bool CGUIEditBox::processKey(const SEvent& event)
 #if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_)
     switch(event.KeyInput.Key)
     {
-    // If cursor points the surrogate low, send KEY_LEFT event.
-    case KEY_UP:
-    case KEY_DOWN:
+    // If cursor points the surrogate low, send IRR_KEY_LEFT event.
+    case IRR_KEY_UP:
+    case IRR_KEY_DOWN:
         if (MultiLine || (WordWrap && BrokenText.size() > 1) )
         {
             if (UTF16_IS_SURROGATE_LO(Text[CursorPos]))
             {
                 SEvent leftEvent;
                 leftEvent = event;
-                leftEvent.KeyInput.Key = KEY_LEFT;
+                leftEvent.KeyInput.Key = IRR_KEY_LEFT;
                 Environment->postEventFromUser(leftEvent);
             }
         }
         break;
     // If cursor points the surrogate low, send a same event.
-    case KEY_LEFT:
-    case KEY_RIGHT:
-    case KEY_DELETE:
+    case IRR_KEY_LEFT:
+    case IRR_KEY_RIGHT:
+    case IRR_KEY_DELETE:
         if (UTF16_IS_SURROGATE_LO(Text[CursorPos]))
             Environment->postEventFromUser(event);
         break;
     // If cursor points front of the surrogate high, send a same event.
-    case KEY_BACK:
+    case IRR_KEY_BACK:
         if (CursorPos > 0)
         {
             if (UTF16_IS_SURROGATE_HI(Text[CursorPos-1]))
@@ -851,7 +887,9 @@ bool CGUIEditBox::processIMEEvent(const SEvent& event)
 
     return false;
 }
+#endif
 
+#if defined(_IRR_COMPILE_WITH_WINDOWS_DEVICE_) || defined(_IRR_COMPILE_WITH_X11_DEVICE_)
 //! calculate the position of input composition window
 core::position2di CGUIEditBox::calculateICPos()
 {
@@ -882,6 +920,7 @@ core::position2di CGUIEditBox::calculateICPos()
 //! draws the element and its children
 void CGUIEditBox::draw()
 {
+#ifndef SERVER_ONLY
     if (!IsVisible)
         return;
 
@@ -1083,6 +1122,7 @@ void CGUIEditBox::draw()
 
     // draw children
     IGUIElement::draw();
+#endif
 }
 
 
@@ -1210,6 +1250,7 @@ bool CGUIEditBox::processMouse(const SEvent& event)
 #endif
             setTextMarkers(CursorPos, CursorPos );
             calculateScrollPos();
+
             return true;
         }
         else if (!m_rtl)
@@ -1219,6 +1260,19 @@ bool CGUIEditBox::processMouse(const SEvent& event)
             {
                 return false;
             }
+            else if (UserConfigParams::m_screen_keyboard)
+            {
+                CursorPos = Text.size();
+                setTextMarkers(CursorPos, CursorPos);
+                calculateScrollPos();
+
+                if (GUIEngine::ScreenKeyboard::getCurrent() == NULL)
+                {
+                    new GUIEngine::ScreenKeyboard(0.98f, 0.30f, this);
+                }
+
+                return true;
+            }
             else
             {
                 // move cursor
@@ -1238,6 +1292,7 @@ bool CGUIEditBox::processMouse(const SEvent& event)
                 MouseMarking = true;
                 setTextMarkers( newMarkBegin, CursorPos);
                 calculateScrollPos();
+
                 return true;
             }
         }
@@ -1321,11 +1376,10 @@ void CGUIEditBox::breakText()
     s32 size = Text.size();
     s32 length = 0;
     s32 elWidth = RelativeRect.getWidth() - 6;
-    wchar_t c;
 
     for (s32 i=0; i<size; ++i)
     {
-        c = Text[i];
+        wchar_t c = Text[i];
         bool lineBreak = false;
 
         if (c == L'\r') // Mac or Windows breaks
@@ -1543,6 +1597,7 @@ void CGUIEditBox::inputChar(wchar_t c)
 
 void CGUIEditBox::calculateScrollPos()
 {
+#ifndef SERVER_ONLY
     if (!AutoScroll)
         return;
 
@@ -1589,6 +1644,18 @@ void CGUIEditBox::calculateScrollPos()
         VScrollPos = 0;
 
     // todo: adjust scrollbar
+#if defined(_IRR_COMPILE_WITH_X11_DEVICE_)
+    if (irr_driver->getDevice()->getType() == irr::EIDT_X11)
+    {
+        CIrrDeviceLinux* dl = dynamic_cast<CIrrDeviceLinux*>(
+                                                       irr_driver->getDevice());
+        if (dl)
+        {
+            dl->setIMELocation(calculateICPos());
+        }
+    }
+#endif
+#endif   // SERVER_ONLY
 }
 
 //! set text markers
