@@ -34,6 +34,7 @@ class DeviceConfig;
 class InputDevice;
 class GamePadDevice;
 class KeyboardDevice;
+class MultitouchDevice;
 
 
 enum PlayerAssignMode
@@ -66,6 +67,7 @@ private:
     PtrVector<GamePadDevice, HOLD>     m_gamepads;
     PtrVector<KeyboardConfig, HOLD>    m_keyboard_configs;
     PtrVector<GamepadConfig, HOLD>     m_gamepad_configs;
+    MultitouchDevice*                  m_multitouch_device;
 
     /** The list of all joysticks that were found and activated. */
     core::array<SJoystickInfo>         m_irrlicht_gamepads;
@@ -101,6 +103,7 @@ public:
 
 
     DeviceManager();
+    ~DeviceManager();
 
     // ---- Assign mode ----
     PlayerAssignMode    getAssignMode() const               { return m_assign_mode; }
@@ -129,6 +132,11 @@ public:
     KeyboardConfig*     getKeyboardConfig(const int i)      { return m_keyboard_configs.get(i); }
     KeyboardDevice*     getKeyboardFromBtnID(const int btnID);
 
+    // ---- Multitouch device ----
+    MultitouchDevice*   getMultitouchDevice()    { return m_multitouch_device; }
+    void                clearMultitouchDevices();
+    void                updateMultitouchDevice();
+
 
     /**
       * \brief Delete the given config and removes DeviceManager references to it.
