@@ -236,10 +236,7 @@ core::stringw GamepadConfig::getBindingAsString(const PlayerAction action) const
                                                     // I18N: name of stick on gamepads
                                                     : _("Left thumb up");
                                                     // I18N: name of stick on gamepads
-            case 2: return (ad==Input::AD_POSITIVE) ? _("Left trigger")
-                                                    // I18N: name of stick on gamepads
-                                                    : _("Right trigger");
-                                                    // I18N: name of stick on gamepads
+            case 2: return _("Left trigger");       // I18N: name of trigger on gamepads
             case 3: return (ad==Input::AD_POSITIVE) ? _("Right thumb down")
                                                     // I18N: name of stick on gamepads
                                                     : _("Right thumb up");
@@ -248,7 +245,8 @@ core::stringw GamepadConfig::getBindingAsString(const PlayerAction action) const
                                                     // I18N: name of stick on gamepads
                                                     : _("Right thumb left");
                                                     // I18N: name of buttons on gamepads
-            case Input::HAT_H_ID: return (ad == Input::AD_POSITIVE) ? _("DPad up") 
+            case 5: return _("Right trigger");      // I18N: name of trigger on gamepads
+            case Input::HAT_H_ID: return (ad == Input::AD_POSITIVE) ? _("DPad up")
                                                     // I18N: name of buttons on gamepads
                                                                     : _("DPad down");
                                                     // I18N: name of buttons on gamepads
