@@ -19,21 +19,24 @@
 #include "input/input_manager.hpp"
 
 #include "config/user_config.hpp"
-#include "graphics/camera.hpp"
+#include "graphics/camera_fps.hpp"
 #include "graphics/irr_driver.hpp"
 #include "guiengine/engine.hpp"
 #include "guiengine/event_handler.hpp"
 #include "guiengine/modaldialog.hpp"
 #include "guiengine/screen.hpp"
+#include "guiengine/screen_keyboard.hpp"
 #include "input/device_manager.hpp"
 #include "input/gamepad_device.hpp"
 #include "input/keyboard_device.hpp"
+#include "input/multitouch_device.hpp"
 #include "input/input.hpp"
 #include "karts/controller/controller.hpp"
 #include "karts/abstract_kart.hpp"
 #include "modes/demo_world.hpp"
 #include "modes/profile_world.hpp"
 #include "modes/world.hpp"
+#include "network/rewind_manager.hpp"
 #include "physics/physics.hpp"
 #include "race/history.hpp"
 #include "replay/replay_recorder.hpp"
@@ -100,17 +103,19 @@ InputManager::~InputManager()
 void InputManager::handleStaticAction(int key, int value)
 {
     static bool control_is_pressed = false;
+    static bool shift_is_pressed   = false;
+
     World *world = World::getWorld();
 
     // When no players... a cutscene
     if (race_manager->getNumPlayers() == 0 && world != NULL && value > 0 &&
-        (key == KEY_SPACE || key == KEY_RETURN))
+        (key == IRR_KEY_SPACE || key == IRR_KEY_RETURN))
     {
         world->onFirePressed(NULL);
     }
 
 
-    if (world != NULL && UserConfigParams::m_artist_debug_mode && 
+    if (world != NULL && UserConfigParams::m_artist_debug_mode &&
         control_is_pressed && value > 0)
     {
         if (Debug::handleStaticAction(key))
@@ -123,35 +128,39 @@ void InputManager::handleStaticAction(int key, int value)
 #ifdef DEBUG
         // Special debug options for profile mode: switch the
         // camera to show a different kart.
-        case KEY_KEY_1:
-        case KEY_KEY_2:
-        case KEY_KEY_3:
-        case KEY_KEY_4:
-        case KEY_KEY_5:
-        case KEY_KEY_6:
-        case KEY_KEY_7:
-        case KEY_KEY_8:
-        case KEY_KEY_9:
+        case IRR_KEY_1:
+        case IRR_KEY_2:
+        case IRR_KEY_3:
+        case IRR_KEY_4:
+        case IRR_KEY_5:
+        case IRR_KEY_6:
+        case IRR_KEY_7:
+        case IRR_KEY_8:
+        case IRR_KEY_9:
         {
             if(!ProfileWorld::isProfileMode() || !world) break;
-            int kart_id = key - KEY_KEY_1;
+            int kart_id = key - IRR_KEY_1;
             if(kart_id<0 || kart_id>=(int)world->getNumKarts()) break;
             Camera::getCamera(0)->setKart(world->getKart(kart_id));
             break;
         }
 #endif
 
-        case KEY_CONTROL:
-        case KEY_RCONTROL:
-        case KEY_LCONTROL:
-        case KEY_RMENU:
-        case KEY_LMENU:
-        case KEY_LWIN:
+        case IRR_KEY_CONTROL:
+        case IRR_KEY_RCONTROL:
+        case IRR_KEY_LCONTROL:
+        case IRR_KEY_RMENU:
+        case IRR_KEY_LMENU:
+        case IRR_KEY_LWIN:
             control_is_pressed = value!=0;
             break;
+        case IRR_KEY_LSHIFT:
+        case IRR_KEY_RSHIFT:
+        case IRR_KEY_SHIFT:
+            shift_is_pressed = value!=0; break;
 
         // Flying up and down
-        case KEY_KEY_I:
+        case IRR_KEY_I:
         {
             if (!world || !UserConfigParams::m_artist_debug_mode) break;
 
@@ -161,7 +170,7 @@ void InputManager::handleStaticAction(int key, int value)
             kart->flyUp();
             break;
         }
-        case KEY_KEY_K:
+        case IRR_KEY_K:
         {
             if (!world || !UserConfigParams::m_artist_debug_mode) break;
 
@@ -172,72 +181,66 @@ void InputManager::handleStaticAction(int key, int value)
             break;
         }
         // Moving the first person camera
-        case KEY_KEY_W:
+        case IRR_KEY_W:
         {
-            if (world && UserConfigParams::m_artist_debug_mode &&
-                Camera::isFPS()                                   )
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (world && UserConfigParams::m_artist_debug_mode && cam  )
             {
-                Camera *cam = Camera::getActiveCamera();
                 core::vector3df vel(cam->getLinearVelocity());
                 vel.Z = value ? cam->getMaximumVelocity() : 0;
                 cam->setLinearVelocity(vel);
             }
             break;
         }
-        case KEY_KEY_S:
+        case IRR_KEY_S:
         {
-            if (world && UserConfigParams::m_artist_debug_mode && 
-                Camera::isFPS()                                   )
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (world && UserConfigParams::m_artist_debug_mode && cam)
             {
-                Camera *cam = Camera::getActiveCamera();
                 core::vector3df vel(cam->getLinearVelocity());
                 vel.Z = value ? -cam->getMaximumVelocity() : 0;
                 cam->setLinearVelocity(vel);
             }
             break;
         }
-        case KEY_KEY_D:
+        case IRR_KEY_D:
         {
-            if (world && !UserConfigParams::m_artist_debug_mode &&
-                Camera::isFPS()                                     )
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (world && !UserConfigParams::m_artist_debug_mode && cam)
             {
-                Camera *cam = Camera::getActiveCamera();
                 core::vector3df vel(cam->getLinearVelocity());
                 vel.X = value ? -cam->getMaximumVelocity() : 0;
                 cam->setLinearVelocity(vel);
             }
             break;
         }
-        case KEY_KEY_A:
+        case IRR_KEY_A:
         {
-            if (world && UserConfigParams::m_artist_debug_mode &&
-                Camera::isFPS()                                   )
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (world && UserConfigParams::m_artist_debug_mode && cam)
             {
-                Camera *cam = Camera::getActiveCamera();
                 core::vector3df vel(cam->getLinearVelocity());
                 vel.X = value ? cam->getMaximumVelocity() : 0;
                 cam->setLinearVelocity(vel);
             }
             break;
         }
-        case KEY_KEY_R:
+        case IRR_KEY_R:
         {
-            if (world && UserConfigParams::m_artist_debug_mode &&
-                Camera::isFPS()                                   )
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (world && UserConfigParams::m_artist_debug_mode && cam)
             {
-                Camera *cam = Camera::getActiveCamera();
                 core::vector3df vel(cam->getLinearVelocity());
                 vel.Y = value ? cam->getMaximumVelocity() : 0;
                 cam->setLinearVelocity(vel);
             }
             break;
         }
-        case KEY_KEY_F:
+        case IRR_KEY_F:
         {
-            if (world && UserConfigParams::m_artist_debug_mode &&
-                Camera::isFPS()                                   )
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (world && UserConfigParams::m_artist_debug_mode && cam)
             {
-                Camera *cam = Camera::getActiveCamera();
                 core::vector3df vel(cam->getLinearVelocity());
                 vel.Y = value ? -cam->getMaximumVelocity() : 0;
                 cam->setLinearVelocity(vel);
@@ -245,39 +248,60 @@ void InputManager::handleStaticAction(int key, int value)
             break;
         }
         // Rotating the first person camera
-        case KEY_KEY_Q:
+        case IRR_KEY_Q:
         {
-            if (world && UserConfigParams::m_artist_debug_mode &&
-                Camera::isFPS()                                   )
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (world && UserConfigParams::m_artist_debug_mode && cam )
             {
-                Camera *active_cam = Camera::getActiveCamera();
-                active_cam->setAngularVelocity(value ?
+                cam->setAngularVelocity(value ?
                     UserConfigParams::m_fpscam_max_angular_velocity : 0.0f);
             }
             break;
         }
-        case KEY_KEY_E:
+        case IRR_KEY_E:
         {
-            if (world && UserConfigParams::m_artist_debug_mode &&
-                Camera::isFPS()                                   )
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (world && UserConfigParams::m_artist_debug_mode && cam)
             {
-                Camera *active_cam = Camera::getActiveCamera();
-                active_cam->setAngularVelocity(value ?
+                cam->setAngularVelocity(value ?
                     -UserConfigParams::m_fpscam_max_angular_velocity : 0);
             }
             break;
         }
 
-        case KEY_SNAPSHOT:
-        case KEY_PRINT:
+        case IRR_KEY_SNAPSHOT:
+        case IRR_KEY_PRINT:
             // on windows we don't get a press event, only release.  So
             // save on release only (to avoid saving twice on other platforms)
-            if (value ==0 )
-                irr_driver->requestScreenshot();
+            if (value == 0)
+            {
+                if (control_is_pressed)
+                {
+                    const bool is_recording = irr_driver->isRecording();
+                    irr_driver->setRecording(!is_recording);
+                }
+                else
+                {
+                    irr_driver->requestScreenshot();
+                }
+            }
             break;
+        case IRR_KEY_F11:
+            if(value && shift_is_pressed && world && RewindManager::isEnabled())
+            {
+                printf("Enter rewind to time:");
+                char s[256];
+                fgets(s, 256, stdin);
+                float t;
+                StringUtils::fromString(s,t);
+                RewindManager::get()->rewindTo(t);
+                Log::info("Rewind", "Rewinding from %f to %f",
+                          world->getTime(), t);
+            }
+            break;
+
             /*
-        case KEY_F1:
-            if (UserConfigParams::m_artist_debug_mode && world)
+            else if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
 
@@ -293,7 +317,7 @@ void InputManager::handleStaticAction(int key, int value)
 #endif
             }
             break;
-        case KEY_F2:
+        case IRR_KEY_F2:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
@@ -301,42 +325,42 @@ void InputManager::handleStaticAction(int key, int value)
                 kart->setPowerup(PowerupManager::POWERUP_PLUNGER, 10000);
             }
             break;
-        case KEY_F3:
+        case IRR_KEY_F3:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
                 kart->setPowerup(PowerupManager::POWERUP_CAKE, 10000);
             }
             break;
-        case KEY_F4:
+        case IRR_KEY_F4:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
                 kart->setPowerup(PowerupManager::POWERUP_SWITCH, 10000);
             }
             break;
-        case KEY_F5:
+        case IRR_KEY_F5:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
                 kart->setPowerup(PowerupManager::POWERUP_BOWLING, 10000);
             }
             break;
-        case KEY_F6:
+        case IRR_KEY_F6:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
                 kart->setPowerup(PowerupManager::POWERUP_BUBBLEGUM, 10000);
             }
             break;
-        case KEY_F7:
+        case IRR_KEY_F7:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
                 kart->setPowerup(PowerupManager::POWERUP_ZIPPER, 10000);
             }
             break;
-        case KEY_F8:
+        case IRR_KEY_F8:
             if (UserConfigParams::m_artist_debug_mode && value && world)
             {
                 if (control_is_pressed)
@@ -359,7 +383,7 @@ void InputManager::handleStaticAction(int key, int value)
                 }
             }
             break;
-        case KEY_F9:
+        case IRR_KEY_F9:
             if (UserConfigParams::m_artist_debug_mode && world)
             {
                 AbstractKart* kart = world->getLocalPlayerKart(0);
@@ -372,7 +396,7 @@ void InputManager::handleStaticAction(int key, int value)
             }
             break;
             */
-        case KEY_F10:
+        case IRR_KEY_F10:
             if(world && value)
             {
                 if(control_is_pressed)
@@ -382,7 +406,7 @@ void InputManager::handleStaticAction(int key, int value)
             }
             break;
             /*
-        case KEY_F11:
+        case IRR_KEY_F11:
             if (UserConfigParams::m_artist_debug_mode && value &&
                 control_is_pressed && world)
             {
@@ -390,7 +414,7 @@ void InputManager::handleStaticAction(int key, int value)
             }
             break;
             */
-        case KEY_F12:
+        case IRR_KEY_F12:
             if(value)
                 UserConfigParams::m_display_fps =
                     !UserConfigParams::m_display_fps;
@@ -464,11 +488,13 @@ void InputManager::inputSensing(Input::InputType type, int deviceID,
         // We have to save the direction in which the axis was moved.
         // This is done by storing it as a sign (and since button can
         // be zero, we add one before changing the sign).
-        int input_id = value>=0 ? 1+button : -(1+button);
+        int input_button_id = value>=0 ? 1+button : -(1+button);
+        std::tuple<int, int> input_id(deviceID, input_button_id);
+        std::tuple<int, int> input_id_inv(deviceID, -input_button_id);
 
         bool id_was_high         = m_sensed_input_high_gamepad.find(input_id)
                                    != m_sensed_input_high_gamepad.end();
-        bool inverse_id_was_high = m_sensed_input_high_gamepad.find(-input_id)
+        bool inverse_id_was_high = m_sensed_input_high_gamepad.find(input_id_inv)
                                    != m_sensed_input_high_gamepad.end();
         bool id_was_zero         = m_sensed_input_zero_gamepad.find(button)
                                    != m_sensed_input_zero_gamepad.end();
@@ -632,13 +658,13 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
     {
         action = PA_BEFORE_FIRST;
 
-        if      (button == KEY_UP)     action = PA_MENU_UP;
-        else if (button == KEY_DOWN)   action = PA_MENU_DOWN;
-        else if (button == KEY_LEFT)   action = PA_MENU_LEFT;
-        else if (button == KEY_RIGHT)  action = PA_MENU_RIGHT;
-        else if (button == KEY_SPACE)  action = PA_MENU_SELECT;
-        else if (button == KEY_RETURN) action = PA_MENU_SELECT;
-        else if (button == KEY_TAB)    
+        if      (button == IRR_KEY_UP)     action = PA_MENU_UP;
+        else if (button == IRR_KEY_DOWN)   action = PA_MENU_DOWN;
+        else if (button == IRR_KEY_LEFT)   action = PA_MENU_LEFT;
+        else if (button == IRR_KEY_RIGHT)  action = PA_MENU_RIGHT;
+        else if (button == IRR_KEY_SPACE)  action = PA_MENU_SELECT;
+        else if (button == IRR_KEY_RETURN) action = PA_MENU_SELECT;
+        else if (button == IRR_KEY_TAB)
         {
             if (shift_mask)
             {
@@ -650,9 +676,13 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
             }
         }
 
-        if (button == KEY_RETURN && GUIEngine::ModalDialog::isADialogActive())
+        if (button == IRR_KEY_RETURN)
         {
-            GUIEngine::ModalDialog::onEnterPressed();
+            if (GUIEngine::ModalDialog::isADialogActive() &&
+                !GUIEngine::ScreenKeyboard::isActive())
+            {
+                GUIEngine::ModalDialog::onEnterPressed();
+            }
         }
 
         if (action != PA_BEFORE_FIRST)
@@ -719,6 +749,7 @@ void InputManager::dispatchInput(Input::InputType type, int deviceID,
         // ... when in-game
         if (StateManager::get()->getGameState() == GUIEngine::GAME &&
              !GUIEngine::ModalDialog::isADialogActive()            &&
+             !GUIEngine::ScreenKeyboard::isActive()                &&
              !race_manager->isWatchingReplay() )
         {
             if (player == NULL)
@@ -944,7 +975,7 @@ EventPropagation InputManager::input(const SEvent& event)
         if (event.KeyInput.PressedDown)
         {
             // escape is a little special
-            if (key == KEY_ESCAPE)
+            if (key == IRR_KEY_ESCAPE)
             {
                 StateManager::get()->escapePressed();
                 return EVENT_BLOCK;
@@ -955,11 +986,12 @@ EventPropagation InputManager::input(const SEvent& event)
             // single letter). Same for spacebar. Same for letters.
             if (GUIEngine::isWithinATextBox())
             {
-                if (key == KEY_BACK || key == KEY_SPACE || key == KEY_SHIFT)
+                if (key == IRR_KEY_BACK || key == IRR_KEY_SPACE || 
+                    key == IRR_KEY_SHIFT)
                 {
                     return EVENT_LET;
                 }
-                if (key >= KEY_KEY_0 && key <= KEY_KEY_Z)
+                if (key >= IRR_KEY_0 && key <= IRR_KEY_Z)
                 {
                     return EVENT_LET;
                 }
@@ -987,11 +1019,12 @@ EventPropagation InputManager::input(const SEvent& event)
             // single letter). Same for spacebar. Same for letters.
             if (GUIEngine::isWithinATextBox())
             {
-                if (key == KEY_BACK || key == KEY_SPACE || key == KEY_SHIFT)
+                if (key == IRR_KEY_BACK || key == IRR_KEY_SPACE || 
+                    key == IRR_KEY_SHIFT)
                 {
                     return EVENT_LET;
                 }
-                if (key >= KEY_KEY_0 && key <= KEY_KEY_Z)
+                if (key >= IRR_KEY_0 && key <= IRR_KEY_Z)
                 {
                     return EVENT_LET;
                 }
@@ -1002,6 +1035,30 @@ EventPropagation InputManager::input(const SEvent& event)
             return EVENT_BLOCK; // Don't propagate key up events
         }
     }
+    else if (event.EventType == EET_TOUCH_INPUT_EVENT)
+    {
+        MultitouchDevice* device = m_device_manager->getMultitouchDevice();
+        unsigned int id = (unsigned int)event.TouchInput.ID;
+
+        if (device != NULL && id < device->m_events.size())
+        {
+            device->m_events[id].id = id;
+            device->m_events[id].x = event.TouchInput.X;
+            device->m_events[id].y = event.TouchInput.Y;
+
+            if (event.TouchInput.Event == ETIE_PRESSED_DOWN)
+            {
+                device->m_events[id].touched = true;
+            }
+            else if (event.TouchInput.Event == ETIE_LEFT_UP)
+            {
+                device->m_events[id].touched = false;
+            }
+
+            m_device_manager->updateMultitouchDevice();
+            device->updateDeviceState(id);
+        }
+    }
     // Use the mouse to change the looking direction when first person view is activated
     else if (event.EventType == EET_MOUSE_INPUT_EVENT)
     {
@@ -1009,13 +1066,13 @@ EventPropagation InputManager::input(const SEvent& event)
 
         if (type == EMIE_MOUSE_MOVED)
         {
-            if (Camera::isFPS())
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (cam)
             {
-                Camera *cam = Camera::getActiveCamera();
                 // Center of the screen
-                core::vector2df screen_size = irr_driver->getCurrentScreenSize();
-                int mid_x = (int) screen_size.X / 2;
-                int mid_y = (int) screen_size.Y / 2;
+                core::dimension2du screen_size = irr_driver->getActualScreenSize();
+                int mid_x = (int) screen_size.Width / 2;
+                int mid_y = (int) screen_size.Height / 2;
                 // Relative mouse movement
                 int diff_x = event.MouseInput.X - m_mouse_val_x;
                 int diff_y = event.MouseInput.Y - m_mouse_val_y;
@@ -1064,12 +1121,12 @@ EventPropagation InputManager::input(const SEvent& event)
         }
         else if (type == EMIE_MOUSE_WHEEL)
         {
-            if (Camera::isFPS())
+            CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+            if (cam)
             {
                 // Use scrolling to change the maximum speed
                 // Only test if it's more or less than 0 as it seems to be not
                 // reliable accross more platforms.
-                Camera *cam = Camera::getActiveCamera();
                 if (event.MouseInput.Wheel < 0)
                 {
                     float vel = cam->getMaximumVelocity() - 3;
@@ -1084,6 +1141,31 @@ EventPropagation InputManager::input(const SEvent& event)
             }
         }
 
+        // Simulate touch event on non-android devices
+        #if !defined(ANDROID)
+        MultitouchDevice* device = m_device_manager->getMultitouchDevice();
+
+        if (device != NULL && (type == EMIE_LMOUSE_PRESSED_DOWN ||
+            type == EMIE_LMOUSE_LEFT_UP || type == EMIE_MOUSE_MOVED))
+        {
+            device->m_events[0].id = 0;
+            device->m_events[0].x = event.MouseInput.X;
+            device->m_events[0].y = event.MouseInput.Y;
+
+            if (type == EMIE_LMOUSE_PRESSED_DOWN)
+            {
+                device->m_events[0].touched = true;
+            }
+            else if (type == EMIE_LMOUSE_LEFT_UP)
+            {
+                device->m_events[0].touched = false;
+            }
+
+            m_device_manager->updateMultitouchDevice();
+            device->updateDeviceState(0);
+        }
+        #endif
+
         /*
         EMIE_LMOUSE_PRESSED_DOWN    Left mouse button was pressed down.
         EMIE_RMOUSE_PRESSED_DOWN    Right mouse button was pressed down.
@@ -1097,6 +1179,37 @@ EventPropagation InputManager::input(const SEvent& event)
                             how fast.
          */
     }
+    else if (event.EventType == EET_ACCELEROMETER_EVENT)
+    {
+        MultitouchDevice* device = m_device_manager->getMultitouchDevice();
+
+        if (device && device->isAccelerometerActive())
+        {
+            m_device_manager->updateMultitouchDevice();
+
+            for (unsigned int i = 0; i < device->getButtonsCount(); i++)
+            {
+                MultitouchButton* button = device->getButton(i);
+
+                if (button->type != BUTTON_STEERING)
+                    continue;
+                    
+                float factor = UserConfigParams::m_multitouch_tilt_factor;
+                factor = std::max(factor, 0.1f);
+
+                if (UserConfigParams::m_multitouch_accelerometer == 1)
+                {
+                    button->axis_x = (float)-event.AccelerometerEvent.X / factor;
+                    device->handleControls(button);
+                }
+                else if (UserConfigParams::m_multitouch_accelerometer == 2)
+                {
+                    button->axis_x = (float)event.AccelerometerEvent.Y / factor;
+                    device->handleControls(button);
+                }
+            }
+        }
+    }
 
     // block events in all modes but initial menus (except in text boxes to
     // allow typing, and except in modal dialogs in-game)
@@ -1105,6 +1218,7 @@ EventPropagation InputManager::input(const SEvent& event)
     if (getDeviceManager()->getAssignMode() != NO_ASSIGN &&
         !GUIEngine::isWithinATextBox() &&
         (!GUIEngine::ModalDialog::isADialogActive() &&
+        !GUIEngine::ScreenKeyboard::isActive() &&
         StateManager::get()->getGameState() == GUIEngine::GAME))
     {
         return EVENT_BLOCK;
