@@ -19,7 +19,6 @@
 #ifndef HEADER_ABSTRACT_KART_HPP
 #define HEADER_ABSTRACT_KART_HPP
 
-#include <EMaterialTypes.h>
 #include <memory>
 
 #include "items/powerup_manager.hpp"
@@ -46,10 +45,13 @@ class KartModel;
 class KartProperties;
 class Material;
 class Powerup;
+class SFXBuffer;
 class Skidding;
 class SlipStream;
 class TerrainInfo;
 
+enum KartRenderType: unsigned int;
+
 /** An abstract interface for the actual karts. Some functions are actually
  *  implemented here in order to allow inlining.
  * \ingroup karts
@@ -100,7 +102,7 @@ public:
                                 int world_kart_id,
                                 int position, const btTransform& init_transform,
                                 PerPlayerDifficulty difficulty,
-                                video::E_RENDER_TYPE rt);
+                                KartRenderType krt);
     virtual       ~AbstractKart();
     virtual core::stringw getName() const;
     virtual void   reset();
@@ -109,16 +111,13 @@ public:
     // Functions related to controlling the kart
     // ------------------------------------------------------------------------
     /** Returns the current steering value for this kart. */
-    float getSteerPercent() const { return m_controls.m_steer;  }
+    float getSteerPercent() const { return m_controls.getSteer();  }
     // ------------------------------------------------------------------------
     /** Returns all controls of this kart. */
     KartControl&  getControls() { return m_controls; }
     // ------------------------------------------------------------------------
     /** Returns all controls of this kart - const version. */
     const KartControl& getControls() const { return m_controls; }
-    // ------------------------------------------------------------------------
-    /** Sets the kart controls. Used e.g. by replaying history. */
-    void setControls(const KartControl &c) { m_controls = c; }
 
     // ========================================================================
     // Access to the kart properties.
@@ -168,10 +167,7 @@ public:
     // Access to the graphical kart model.
     // ------------------------------------------------------------------------
     /** Returns this kart's kart model. */
-    KartModel* getKartModel() { return m_kart_model;      }
-    // ------------------------------------------------------------------------
-    /** Returns this kart's kart model. */
-    const KartModel* getKartModel() const { return m_kart_model;      }
+    KartModel* getKartModel() const { return m_kart_model;      }
     // ------------------------------------------------------------------------
     /** Returns the length of the kart. */
     float getKartLength() const { return m_kart_length; }
@@ -277,6 +273,10 @@ public:
      *  like Ghost. */
     virtual float getSpeed() const = 0;
     // ------------------------------------------------------------------------
+    /** Returns the exponentially smoothened speed of the kart in 
+     *  which is removes shaking from camera. */
+    virtual float getSmoothedSpeed() const = 0;
+    // ------------------------------------------------------------------------
     /** Returns the current maximum speed for this kart, this includes all
      *  bonus and maluses that are currently applied. */
     virtual float getCurrentMaxSpeed() const = 0;
@@ -439,6 +439,10 @@ public:
     // ------------------------------------------------------------------------
     virtual void crashed(const Material *m, const Vec3 &normal) = 0;
     // ------------------------------------------------------------------------
+    /** Returns the normal of the terrain the kart is over atm. This is
+     *  defined even if the kart is flying. */
+    virtual const Vec3& getNormal() const = 0;
+    // ------------------------------------------------------------------------
     /** Returns the height of the terrain. we're currently above */
     virtual float getHoT() const = 0;
     // ------------------------------------------------------------------------
@@ -466,7 +470,8 @@ public:
     // ------------------------------------------------------------------------
     /** Returns whether this kart is jumping. */
     virtual bool isJumping() const = 0;
-
+    // ------------------------------------------------------------------------
+    virtual void playSound(SFXBuffer* buffer) = 0;
 };   // AbstractKart
 
 
