@@ -28,23 +28,23 @@
 #include "karts/abstract_kart.hpp"
 #include "karts/controller/controller.hpp"
 #include "karts/kart.hpp"
+#include "karts/kart_model.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/skidding.hpp"
 #include "physics/btKart.hpp"
 #include "utils/log.hpp"
+#include "race/race_manager.hpp"
 
 #include <iostream>
 
-KartGFX::KartGFX(const AbstractKart *kart)
+KartGFX::KartGFX(const AbstractKart *kart, bool is_day)
 {
-    //if(!UserConfigParams::m_graphical_effects)
-    //{
-    //    for(unsigned int i=0; i<KGFX_COUNT; i++)
-    //        m_all_emitters.push_back(NULL);
-    //    return;
-    //}
-
+    m_nitro_light = NULL;
+    m_skidding_light_1 = NULL;
+    m_skidding_light_2 = NULL;
     m_kart = kart;
+    m_wheel_toggle = 0;
+    m_skid_level = 0;
 
     const KartModel *km = m_kart->getKartModel();
     const float length = km->getLength();
@@ -52,34 +52,42 @@ KartGFX::KartGFX(const AbstractKart *kart)
     scene::ISceneNode *node = m_kart->getNode();
     // Create nitro light
     core::vector3df location(0.0f, 0.5f, -0.5f*length - 0.05f);
-    m_nitro_light = irr_driver->addLight(location, /*force*/ 0.4f,
-                                         /*radius*/5.0f, 0.0f, 0.4f, 1.0f,
-                                         false, node);
-    m_nitro_light->setVisible(false);
-#ifdef DEBUG
-    m_nitro_light->setName( ("nitro emitter (" + m_kart->getIdent()
-                                               + ")").c_str() );
+#ifndef SERVER_ONLY
+    if (CVS->isGLSL())
+    {
+        m_nitro_light = irr_driver->addLight(location, /*force*/ 0.4f,
+                                             /*radius*/ 5.0f, 0.0f, 0.4f, 1.0f,
+                                             false, node);
+        m_nitro_light->setVisible(false);
+    #ifdef DEBUG
+        m_nitro_light->setName( ("nitro emitter (" + m_kart->getIdent()
+                                                   + ")").c_str() );
+    #endif
+    
+        // Create skidding lights
+        // For the first skidding level
+        m_skidding_light_1 = 
+            irr_driver->addLight(core::vector3df(0.0f, 0.1f, -0.5f * length - 
+                                 0.05f), /* force */ 0.3f, /*radius*/ 3.0f,
+                                 1.0f, 0.6f, 0.0f, false, node);
+        m_skidding_light_1->setVisible(false);
+        m_skidding_light_1->setName(("skidding emitter 1 (" + m_kart->getIdent() 
+                                                            + ")").c_str() );
+    
+        // For the second skidding level
+        m_skidding_light_2 =
+            irr_driver->addLight(core::vector3df(0.0f, 0.1f, -0.5f * length - 
+                                 0.05f), /* force */0.4f, /*radius*/4.0f,
+                                 1.0f, 0.0f, 0.0f, false, node);
+        m_skidding_light_2->setVisible(false);
+        m_skidding_light_2->setName(("skidding emitter 2 (" + m_kart->getIdent()
+                                                            + ")").c_str() );
+        m_nitro_light->grab();
+        m_skidding_light_1->grab();
+        m_skidding_light_2->grab();
+    }
 #endif
 
-    // Create skidding lights
-    // For the first skidding level
-    m_skidding_light_1 = 
-        irr_driver->addLight(core::vector3df(0.0f, 0.1f, -0.5f*length - 0.05f),
-                             /* force */ 0.3f,  /* radius */ 3.0f,
-                             1.0f, 0.6f, 0.0f, false, node);
-    m_skidding_light_1->setVisible(false);
-    m_skidding_light_1->setName( ("skidding emitter 1 (" + m_kart->getIdent() 
-                                                         + ")").c_str() );
-
-    // For the second skidding level
-    m_skidding_light_2 =
-        irr_driver->addLight(core::vector3df(0.0f, 0.1f, -0.5f*length - 0.05f),
-                             /* force */0.4f, /*radius*/ 4.0f,
-                             1.0f, 0.0f, 0.0f, false, node);
-    m_skidding_light_2->setVisible(false);
-    m_skidding_light_2->setName( ("skidding emitter 2 (" + m_kart->getIdent() 
-                                                         + ")").c_str() );
-
     // Create particle effects
     Vec3 rear_left(kart->getWheelGraphicsPosition(3).getX(), 0.05f,
                    kart->getWheelGraphicsPosition(3).getZ()-0.1f    );
@@ -109,6 +117,18 @@ KartGFX::KartGFX(const AbstractKart *kart)
     addEffect(KGFX_SKID1R,      "skid1.xml",       rear_right,       true );
     addEffect(KGFX_SKID2L,      "skid2.xml",       rear_left,        true );
     addEffect(KGFX_SKID2R,      "skid2.xml",       rear_right,       true );
+    if (!kart->getKartModel()->getExhaustXML().empty())
+    {
+        const std::string& ex = kart->getKartModel()->getExhaustXML();
+        addEffect(KGFX_EXHAUST1, ex, rear_nitro_right, false);
+        addEffect(KGFX_EXHAUST2, ex, rear_nitro_left, false);
+    }
+    else
+    {
+        m_all_emitters.push_back(NULL);
+        m_all_emitters.push_back(NULL);
+    }
+
 }   // KartGFX
 
 // ----------------------------------------------------------------------------
@@ -121,13 +141,15 @@ KartGFX::~KartGFX()
         if(m_all_emitters[i])
             delete m_all_emitters[i];
     }   // for i < KGFX_COUNT
-    
+
+#ifndef SERVER_ONLY    
     if (CVS->isGLSL())
     {
         m_nitro_light->drop();
         m_skidding_light_1->drop();
         m_skidding_light_2->drop();
     }
+#endif
 
 }   // ~KartGFX
 
@@ -141,8 +163,11 @@ KartGFX::~KartGFX()
 void KartGFX::addEffect(KartGFXType type, const std::string &file_name,
                         const Vec3 &position, bool important)
 {
-    if (!UserConfigParams::m_graphical_effects &&
-        (!important || m_kart->getType() == RaceManager::KT_AI))
+#ifndef SERVER_ONLY
+    if (((UserConfigParams::m_graphical_effects < 2 || !CVS->isGLSL()) &&
+        (!important || m_kart->getType() == RaceManager::KT_AI ||
+        m_kart->getType() == RaceManager::KT_SPARE_TIRE)) ||
+        UserConfigParams::m_graphical_effects < 1)
     {
         m_all_emitters.push_back(NULL);
         return;
@@ -185,6 +210,7 @@ void KartGFX::addEffect(KartGFXType type, const std::string &file_name,
         m_skid_kind1 = kind;
     else if (type==KGFX_SKID2L || type==KGFX_SKID2R)
         m_skid_kind2 = kind;
+#endif
 }   // addEffect
 
 // ----------------------------------------------------------------------------
@@ -193,14 +219,15 @@ void KartGFX::addEffect(KartGFXType type, const std::string &file_name,
 void KartGFX::reset()
 {
     m_wheel_toggle = 1;
+#ifndef SERVER_ONLY
     for(unsigned int i=0; i<m_all_emitters.size(); i++)
     {
         if(m_all_emitters[i])
         {
             m_all_emitters[i]->setCreationRateAbsolute(0);
-            m_all_emitters[i]->clearParticles();
         }
     }
+#endif
 }   // reset
 
 // ----------------------------------------------------------------------------
@@ -212,7 +239,9 @@ void KartGFX::setSkidLevel(const unsigned int level)
 {
     assert(level >= 1);
     assert(level <= 2);
+    m_skid_level = level;
     const ParticleKind *pk = level==1 ? m_skid_kind1 : m_skid_kind2;
+#ifndef SERVER_ONLY
     if(m_all_emitters[KGFX_SKID1L])
         m_all_emitters[KGFX_SKID1L]->setParticleType(pk);
     if(m_all_emitters[KGFX_SKID1R])
@@ -221,6 +250,7 @@ void KartGFX::setSkidLevel(const unsigned int level)
     // set to indicate that the bonus is now available.
     setCreationRateRelative(KartGFX::KGFX_SKIDL, 0.0f);
     setCreationRateRelative(KartGFX::KGFX_SKIDR, 0.0f);
+#endif
 }   // setSkidLevel
 
 // ----------------------------------------------------------------------------
@@ -231,10 +261,12 @@ void KartGFX::setSkidLevel(const unsigned int level)
  */
 void KartGFX::setParticleKind(const KartGFXType type, const ParticleKind *pk)
 {
+#ifndef SERVER_ONLY
     ParticleEmitter *pe = m_all_emitters[KGFX_TERRAIN];
     if(!pe) return;
 
     pe->setParticleType(pk);
+#endif
 }   // setParticleKind
 
 // ----------------------------------------------------------------------------
@@ -244,9 +276,11 @@ void KartGFX::setParticleKind(const KartGFXType type, const ParticleKind *pk)
  */
 void KartGFX::setXYZ(const KartGFXType type, const Vec3 &xyz)
 {
+#ifndef SERVER_ONLY
     ParticleEmitter *pe = m_all_emitters[KGFX_TERRAIN];
     if(!pe) return;
     pe->setPosition(xyz);
+#endif
 }   // setXYZ
 
 // ----------------------------------------------------------------------------
@@ -257,8 +291,15 @@ void KartGFX::setXYZ(const KartGFXType type, const Vec3 &xyz)
  */
 void KartGFX::setCreationRateAbsolute(KartGFXType type, float f)
 {
-    if(m_all_emitters[type])
-        m_all_emitters[type]->setCreationRateAbsolute(f);
+#ifndef SERVER_ONLY
+    if (!m_all_emitters[type])
+        return;
+        
+    if (m_all_emitters[type]->getCreationRateFloat() == f)
+        return;
+
+    m_all_emitters[type]->setCreationRateAbsolute(f);
+#endif
 }   // setCreationRateAbsolute
 
 // ----------------------------------------------------------------------------
@@ -271,6 +312,7 @@ void KartGFX::setCreationRateAbsolute(KartGFXType type, float f)
  */
 void KartGFX::setCreationRateRelative(KartGFXType type, float f)
 {
+#ifndef SERVER_ONLY
     if(m_all_emitters[type])
     {
         if(f<0)
@@ -278,6 +320,7 @@ void KartGFX::setCreationRateRelative(KartGFXType type, float f)
         else
             m_all_emitters[type]->setCreationRateRelative(f);
     }
+#endif
 }   // setCreationRateRelative
 
 // ----------------------------------------------------------------------------
@@ -290,8 +333,10 @@ void KartGFX::setCreationRateRelative(KartGFXType type, float f)
  */
 void KartGFX::resizeBox(KartGFXType type, float new_size)
 {
+#ifndef SERVER_ONLY
     if(m_all_emitters[type])
         m_all_emitters[type]->resizeBox(std::max(0.25f, new_size));
+#endif
 }   // resizeBox
 
 // ----------------------------------------------------------------------------
@@ -303,6 +348,7 @@ void KartGFX::resizeBox(KartGFXType type, float new_size)
  */
 void KartGFX::updateTerrain(const ParticleKind *pk)
 {
+#ifndef SERVER_ONLY
    ParticleEmitter *pe = m_all_emitters[KGFX_TERRAIN];
     if(!pe) return;
 
@@ -324,7 +370,7 @@ void KartGFX::updateTerrain(const ParticleKind *pk)
     bool on_ground       = m_kart->isOnGround() &&
                            m_kart->getSkidding()->getGraphicalJumpOffset()==0;
     if (skidding > 1.0f && on_ground)
-        rate = fabsf(m_kart->getControls().m_steer) > 0.8 ? skidding - 1 : 0;
+        rate = fabsf(m_kart->getControls().getSteer()) > 0.8 ? skidding - 1 : 0;
     else if (speed >= 0.5f && on_ground)
         rate = speed/m_kart->getKartProperties()->getEngineMaxSpeed();
     else
@@ -335,6 +381,7 @@ void KartGFX::updateTerrain(const ParticleKind *pk)
     // m_skidding can be > 2, and speed > maxSpeed (if powerups are used).
     if(rate>1.0f) rate = 1.0f;
     pe->setCreationRateRelative(rate);
+#endif
 }   // updateTerrain
 
 // ----------------------------------------------------------------------------
@@ -359,6 +406,7 @@ void KartGFX::update(float dt)
  */
 void KartGFX::updateNitroGraphics(float nitro_frac)
 {
+#ifndef SERVER_ONLY
     // Upate particle effects (creation rate, and emitter size
     // depending on speed)
     // --------------------------------------------------------
@@ -368,7 +416,9 @@ void KartGFX::updateNitroGraphics(float nitro_frac)
         setCreationRateRelative(KartGFX::KGFX_NITRO2, nitro_frac);
         setCreationRateRelative(KartGFX::KGFX_NITROSMOKE1, nitro_frac);
         setCreationRateRelative(KartGFX::KGFX_NITROSMOKE2, nitro_frac);
-        m_nitro_light->setVisible(true);
+        
+        if (CVS->isGLSL())
+            m_nitro_light->setVisible(true);
     }
     else
     {
@@ -376,9 +426,15 @@ void KartGFX::updateNitroGraphics(float nitro_frac)
         setCreationRateAbsolute(KartGFX::KGFX_NITRO2,      0);
         setCreationRateAbsolute(KartGFX::KGFX_NITROSMOKE1, 0);
         setCreationRateAbsolute(KartGFX::KGFX_NITROSMOKE2, 0);
-        m_nitro_light->setVisible(false);
+        
+        if (CVS->isGLSL())
+            m_nitro_light->setVisible(false);
     }
-
+    
+    // Exhaust is always emitting
+    setCreationRateRelative(KartGFX::KGFX_EXHAUST1, 1.0);
+    setCreationRateRelative(KartGFX::KGFX_EXHAUST2, 1.0);
+#endif
 }  // updateGraphics
 
 // ----------------------------------------------------------------------------
@@ -388,14 +444,20 @@ void KartGFX::updateNitroGraphics(float nitro_frac)
  */
 void KartGFX::updateSkidLight(unsigned int level)
 {
-    m_skidding_light_1->setVisible(level == 1);
-    m_skidding_light_2->setVisible(level > 1);
+#ifndef SERVER_ONLY
+    if (CVS->isGLSL())
+    {
+        m_skidding_light_1->setVisible(level == 1);
+        m_skidding_light_2->setVisible(level > 1);
+    }
+#endif
 }   // updateSkidLight
 
 // ----------------------------------------------------------------------------
 void KartGFX::getGFXStatus(int* nitro, bool* zipper,
                            int* skidding, bool* red_skidding) const
 {
+#ifndef SERVER_ONLY
     int n = 0;
     bool z = false;
     int s = 0;
@@ -414,35 +476,40 @@ void KartGFX::getGFXStatus(int* nitro, bool* zipper,
     if (m_all_emitters[KGFX_SKIDL])
     {
         s = m_all_emitters[KGFX_SKIDL]->getCreationRate();
-        r = m_skidding_light_2->isVisible();
+        r = m_skid_level == 2;
     }
 
     *nitro = n;
     *zipper = z;
     *skidding = s;
     *red_skidding = r;
-
+#endif
 }   // getGFXStatus
 
 // ----------------------------------------------------------------------------
 void KartGFX::setGFXFromReplay(int nitro, bool zipper,
                                int skidding, bool red_skidding)
 {
+#ifndef SERVER_ONLY
     if (nitro > 0)
     {
         setCreationRateAbsolute(KartGFX::KGFX_NITRO1,      (float)nitro);
         setCreationRateAbsolute(KartGFX::KGFX_NITRO2,      (float)nitro);
         setCreationRateAbsolute(KartGFX::KGFX_NITROSMOKE1, (float)nitro);
         setCreationRateAbsolute(KartGFX::KGFX_NITROSMOKE2, (float)nitro);
-        m_nitro_light->setVisible(true);
+        
+        if (CVS->isGLSL())
+            m_nitro_light->setVisible(true);
     }
-    else if (m_nitro_light->isVisible() && nitro == 0)
+    else
     {
         setCreationRateAbsolute(KartGFX::KGFX_NITRO1,      0.0f);
         setCreationRateAbsolute(KartGFX::KGFX_NITRO2,      0.0f);
         setCreationRateAbsolute(KartGFX::KGFX_NITROSMOKE1, 0.0f);
         setCreationRateAbsolute(KartGFX::KGFX_NITROSMOKE2, 0.0f);
-        m_nitro_light->setVisible(false);
+        
+        if (CVS->isGLSL())
+            m_nitro_light->setVisible(false);
     }
 
     if (zipper)
@@ -450,43 +517,47 @@ void KartGFX::setGFXFromReplay(int nitro, bool zipper,
 
     if (skidding > 0)
     {
-        if (!m_skidding_light_1->isVisible() && !red_skidding)
-        {
-            if (m_all_emitters[KGFX_SKID1L])
-                m_all_emitters[KGFX_SKID1L]->setParticleType(m_skid_kind1);
-            if (m_all_emitters[KGFX_SKID1R])
-                m_all_emitters[KGFX_SKID1R]->setParticleType(m_skid_kind1);
+        const ParticleKind* skid_kind = red_skidding ? m_skid_kind2 
+                                                     : m_skid_kind1;
 
-            m_skidding_light_1->setVisible(true);
-            m_skidding_light_2->setVisible(false);
-        }
-        if (!m_skidding_light_2->isVisible() && red_skidding)
-        {
-            if (m_all_emitters[KGFX_SKID1L])
-                m_all_emitters[KGFX_SKID1L]->setParticleType(m_skid_kind2);
-            if (m_all_emitters[KGFX_SKID1R])
-                m_all_emitters[KGFX_SKID1R]->setParticleType(m_skid_kind2);
+        if (m_all_emitters[KGFX_SKID1L])
+            m_all_emitters[KGFX_SKID1L]->setParticleType(skid_kind);
+        if (m_all_emitters[KGFX_SKID1R])
+            m_all_emitters[KGFX_SKID1R]->setParticleType(skid_kind);
 
-            m_skidding_light_1->setVisible(false);
-            m_skidding_light_2->setVisible(true);
+        if (CVS->isGLSL())
+        {
+            m_skidding_light_1->setVisible(!red_skidding);
+            m_skidding_light_2->setVisible(red_skidding);
         }
+        
         setCreationRateAbsolute(KartGFX::KGFX_SKIDL, (float)skidding);
         setCreationRateAbsolute(KartGFX::KGFX_SKIDR, (float)skidding);
     }
-    else if ((m_skidding_light_1->isVisible() ||
-        m_skidding_light_2->isVisible()) && skidding == 0)
+    else
     {
         setCreationRateAbsolute(KartGFX::KGFX_SKIDL, 0.0f);
         setCreationRateAbsolute(KartGFX::KGFX_SKIDR, 0.0f);
-        m_skidding_light_1->setVisible(false);
-        m_skidding_light_2->setVisible(false);
+        
+        if (CVS->isGLSL())
+        {
+            m_skidding_light_1->setVisible(false);
+            m_skidding_light_2->setVisible(false);
+        }
     }
+#endif
 }   // setGFXFromReplay
 
 // ----------------------------------------------------------------------------
 void KartGFX::setGFXInvisible()
 {
-    m_nitro_light->setVisible(false);
-    m_skidding_light_1->setVisible(false);
-    m_skidding_light_2->setVisible(false);
+#ifndef SERVER_ONLY
+    if (CVS->isGLSL())
+    {
+        m_nitro_light->setVisible(false);
+        m_skidding_light_1->setVisible(false);
+        m_skidding_light_2->setVisible(false);
+        m_kart->getKartModel()->toggleHeadlights(false);
+    }
+#endif
 }   // setGFXInvisible
