@@ -21,9 +21,8 @@
 #include "addons/addon.hpp"
 #include "config/stk_config.hpp"
 #include "config/player_manager.hpp"
-#include "graphics/glwrap.hpp"
-#include "graphics/irr_driver.hpp"
 #include "graphics/material_manager.hpp"
+#include "graphics/stk_tex_manager.hpp"
 #include "io/file_manager.hpp"
 #include "karts/cached_characteristic.hpp"
 #include "karts/combined_characteristic.hpp"
@@ -125,8 +124,10 @@ void KartProperties::copyForPlayer(const KartProperties *source)
     // So all pointer variables need to be separately allocated and assigned.
     if (source->m_characteristic)
     {
+        // Remove the shared reference by creating a new pointer
         m_characteristic.reset(new XmlCharacteristic());
-        *m_characteristic = *source->m_characteristic;
+        m_characteristic->copyFrom(source->getCharacteristic());
+
         // Combine the characteristics for this object. We can't copy it because
         // this object has other pointers (to m_characteristic).
         combineCharacteristics();
@@ -223,11 +224,12 @@ void KartProperties::load(const std::string &filename, const std::string &node)
 
     // Load material
     std::string materials_file = m_root+"materials.xml";
-    file_manager->pushModelSearchPath  (m_root);
-    file_manager->pushTextureSearchPath(m_root);
+    std::string unique_id = StringUtils::insertValues("karts/%s", m_ident.c_str());
+    file_manager->pushModelSearchPath(m_root);
+    file_manager->pushTextureSearchPath(m_root, unique_id);
 
-    irr_driver->setTextureErrorMessage("Error while loading kart '%s':",
-                                       m_name);
+    STKTexManager::getInstance()
+        ->setTextureErrorMessage("Error while loading kart '%s':", m_name);
 
     // addShared makes sure that these textures/material infos stay in memory
     material_manager->addSharedMaterial(materials_file);
@@ -242,15 +244,20 @@ void KartProperties::load(const std::string &filename, const std::string &node)
                                                     /*make_permanent*/true,
                                                     /*complain_if_not_found*/true,
                                                     /*strip_path*/false);
-    if(m_minimap_icon_file!="")
-        m_minimap_icon = irr_driver->getTexture(m_root+m_minimap_icon_file);
+    if (m_minimap_icon_file!="")
+    {
+        m_minimap_icon = STKTexManager::getInstance()
+            ->getTexture(m_root+m_minimap_icon_file);
+    }
     else
         m_minimap_icon = NULL;
 
+#ifndef SERVER_ONLY
     if (m_minimap_icon == NULL)
     {
-        m_minimap_icon = getUnicolorTexture(m_color);
+        m_minimap_icon = STKTexManager::getInstance()->getUnicolorTexture(m_color);
     }
+#endif
 
     // Only load the model if the .kart file has the appropriate version,
     // otherwise warnings are printed.
@@ -291,14 +298,33 @@ void KartProperties::load(const std::string &filename, const std::string &node)
     // used.
     m_wheel_base = fabsf(m_kart_model->getLength() - 2*0.25f);
 
-    m_shadow_texture = irr_driver->getTexture(m_shadow_file);
+    m_shadow_texture = STKTexManager::getInstance()->getTexture(m_shadow_file);
 
-    irr_driver->unsetTextureErrorMessage();
+    STKTexManager::getInstance()->unsetTextureErrorMessage();
     file_manager->popTextureSearchPath();
     file_manager->popModelSearchPath();
 
 }   // load
 
+// ----------------------------------------------------------------------------
+/** Returns a pointer to the KartModel object.
+ *  \param krt The KartRenderType, like default, red, blue or transparent.
+ *  see the RenderInfo include for details
+ */
+KartModel* KartProperties::getKartModelCopy(KartRenderType krt) const
+{
+    return m_kart_model->makeCopy(krt);
+}  // getKartModelCopy
+
+// ----------------------------------------------------------------------------
+/** Sets the name of a mesh to be used for this kart.
+ *  \param hat_name Name of the mesh.
+ */
+void KartProperties::setHatMeshName(const std::string &hat_name)
+{
+    m_kart_model->setHatMeshName(hat_name);
+}  // setHatMeshName
+
 //-----------------------------------------------------------------------------
 void KartProperties::combineCharacteristics()
 {
@@ -365,11 +391,6 @@ void KartProperties::getAllData(const XMLNode * root)
         m_ai_properties[RaceManager::DIFFICULTY_BEST]->load(best);
     }
 
-    if(const XMLNode *speed_weighted_objects_node = root->getNode("speed-weighted-objects"))
-    {
-        m_speed_weighted_object_properties.loadFromXMLNode(speed_weighted_objects_node);
-    }
-
     if(const XMLNode *friction_node = root->getNode("friction"))
         friction_node->get("slip", &m_friction_slip);
 
@@ -473,8 +494,6 @@ void KartProperties::checkAllSet(const std::string &filename)
     CHECK_NEG(m_restitution,                "collision restitution"         );
     CHECK_NEG(m_physical_wheel_position,    "collision physical-wheel-position");
 
-    m_speed_weighted_object_properties.checkAllSet();
-
     for(unsigned int i=0; i<RaceManager::DIFFICULTY_COUNT; i++)
         m_ai_properties[i]->checkAllSet(filename);
 }   // checkAllSet
@@ -590,6 +609,12 @@ float KartProperties::getStabilityTrackConnectionAccel() const
     return m_cached_characteristic->getStabilityTrackConnectionAccel();
 }  // getStabilityTrackConnectionAccel
 
+// ----------------------------------------------------------------------------
+std::vector<float> KartProperties::getStabilityAngularFactor() const
+{
+    return m_cached_characteristic->getStabilityAngularFactor();
+}  // getStabilityAngularFactor
+
 // ----------------------------------------------------------------------------
 float KartProperties::getStabilitySmoothFlyingImpulse() const
 {
@@ -746,6 +771,18 @@ float KartProperties::getParachuteDurationOther() const
     return m_cached_characteristic->getParachuteDurationOther();
 }  // getParachuteDurationOther
 
+// ----------------------------------------------------------------------------
+float KartProperties::getParachuteDurationRankMult() const
+{
+    return m_cached_characteristic->getParachuteDurationRankMult();
+}  // getParachuteDurationRankMult
+
+// ----------------------------------------------------------------------------
+float KartProperties::getParachuteDurationSpeedMult() const
+{
+    return m_cached_characteristic->getParachuteDurationSpeedMult();
+}  // getParachuteDurationSpeedMult
+
 // ----------------------------------------------------------------------------
 float KartProperties::getParachuteLboundFraction() const
 {
@@ -764,6 +801,12 @@ float KartProperties::getParachuteMaxSpeed() const
     return m_cached_characteristic->getParachuteMaxSpeed();
 }  // getParachuteMaxSpeed
 
+// ----------------------------------------------------------------------------
+float KartProperties::getFrictionKartFriction() const
+{
+    return m_cached_characteristic->getFrictionKartFriction();
+}  // getFrictionKartFriction
+
 // ----------------------------------------------------------------------------
 float KartProperties::getBubblegumDuration() const
 {
@@ -1142,6 +1185,5 @@ bool KartProperties::getSkidEnabled() const
     return m_cached_characteristic->getSkidEnabled();
 }  // getSkidEnabled
 
-
 /* <characteristics-end kpgetter> */
 
