@@ -32,7 +32,6 @@ namespace irr
 using namespace irr;
 
 #include "audio/sfx_manager.hpp"
-#include "karts/kart_model.hpp"
 #include "io/xml_node.hpp"
 #include "race/race_manager.hpp"
 #include "utils/interpolation_array.hpp"
@@ -42,9 +41,12 @@ class AbstractCharacteristic;
 class AIProperties;
 class CachedCharacteristic;
 class CombinedCharacteristic;
+class KartModel;
 class Material;
 class XMLNode;
 
+enum KartRenderType: unsigned int;
+
 /**
  *  \brief This class stores the properties of a kart.
  *  This includes size, name, identifier, physical properties etc.
@@ -174,9 +176,6 @@ private:
     // -------------------
     float m_friction_slip;
 
-    /** Parameters for the speed-weighted objects */
-    SpeedWeightedObject::Properties   m_speed_weighted_object_properties;
-
     /** Shift of center of gravity. */
     Vec3  m_gravity_center_shift;
 
@@ -242,24 +241,13 @@ public:
     video::ITexture *getMinimapIcon  () const {return m_minimap_icon;         }
 
     // ------------------------------------------------------------------------
-    /** Returns a pointer to the KartModel object. */
-    KartModel*    getKartModelCopy
-                (video::E_RENDER_TYPE rt = video::ERT_DEFAULT) const
-                                          {return m_kart_model->makeCopy(rt); }
-
+    KartModel*    getKartModelCopy(KartRenderType krt) const;
     // ------------------------------------------------------------------------
     /** Returns a pointer to the main KartModel object. This copy
      *  should not be modified, not attachModel be called on it. */
     const KartModel& getMasterKartModel() const {return *m_kart_model;        }
-
     // ------------------------------------------------------------------------
-    /** Sets the name of a mesh to be used for this kart.
-     *  \param hat_name Name of the mesh.
-     */
-    void setHatMeshName(const std::string &hat_name)
-    {
-        m_kart_model->setHatMeshName(hat_name);
-    }   // setHatMeshName
+    void setHatMeshName(const std::string &hat_name);
     // ------------------------------------------------------------------------
     /** Returns the name of this kart.
         \note Pass it through fridibi as needed, this is the LTR name
@@ -320,13 +308,6 @@ public:
     /** Returns friction slip. */
     float getFrictionSlip           () const {return m_friction_slip;         }
 
-    // ------------------------------------------------------------------------
-    /** Returns parameters for the speed-weighted objects */
-    const SpeedWeightedObject::Properties& getSpeedWeightedObjectProperties() const
-    {
-        return m_speed_weighted_object_properties;
-    }
-
     // ------------------------------------------------------------------------
     /** Returns the wheel base (distance front to rear axis). */
     float getWheelBase              () const {return m_wheel_base;            }
@@ -427,6 +408,7 @@ public:
     float getStabilityChassisAngularDamping() const;
     float getStabilityDownwardImpulseFactor() const;
     float getStabilityTrackConnectionAccel() const;
+    std::vector<float> getStabilityAngularFactor() const;
     float getStabilitySmoothFlyingImpulse() const;
 
     InterpolationArray getTurnRadius() const;
@@ -463,10 +445,14 @@ public:
     float getParachuteFriction() const;
     float getParachuteDuration() const;
     float getParachuteDurationOther() const;
+    float getParachuteDurationRankMult() const;
+    float getParachuteDurationSpeedMult() const;
     float getParachuteLboundFraction() const;
     float getParachuteUboundFraction() const;
     float getParachuteMaxSpeed() const;
 
+    float getFrictionKartFriction() const;
+
     float getBubblegumDuration() const;
     float getBubblegumSpeedFraction() const;
     float getBubblegumTorque() const;
@@ -541,6 +527,8 @@ public:
     bool getSkidEnabled() const;
 
     /* <characteristics-end kpdefs> */
+    
+    LEAK_CHECK()
 };   // KartProperties
 
 #endif
