@@ -31,6 +31,17 @@ XmlCharacteristic::XmlCharacteristic(const XMLNode *node) :
         load(node);
 }   // XmlCharacteristic constructor
 
+// ----------------------------------------------------------------------------
+/** Copies the characteristics from the specified other characteristic class
+ *  into this class.
+ */
+void XmlCharacteristic::copyFrom(const AbstractCharacteristic *other)
+{
+    const XmlCharacteristic *xc = dynamic_cast<const XmlCharacteristic*>(other);
+    assert(xc!=NULL);
+    m_values = xc->m_values;
+}   // operator=
+
 // ----------------------------------------------------------------------------
 /** process will execute the operation that is specified in the saved string.
  *  The format of the operations is specified in kart_characteristics.xml.
@@ -212,8 +223,9 @@ void XmlCharacteristic::processFloat(const std::string &processor, float *value,
     std::size_t pos2;
     while ((pos2 = processor.find_first_of(operators, pos)) != std::string::npos)
     {
-        parts.push_back(processor.substr(pos, pos2));
-        operations.push_back(processor.substr(pos2, pos2 + 1));
+        std::string s = processor.substr(pos, pos2);
+        parts.push_back(processor.substr(pos, pos2-pos));
+        operations.push_back(processor.substr(pos2, 1));
         pos = pos2 + 1;
     }
     parts.push_back(processor.substr(pos));
@@ -224,15 +236,15 @@ void XmlCharacteristic::processFloat(const std::string &processor, float *value,
     // If nothing preceeds the first operator, insert x
     if (parts[index].empty())
     {
-        if (!*is_set)
+        // - is a special case: We don't take e.g. "-5" as relative, it
+        // describes a negative number. So 
+        if (!*is_set && operations[index] == "-")
+            *value = 0;
+        else if (!*is_set)
         {
             Log::error("XmlCharacteristic::processFloat", "x is unknown");
             return;
         }
-        // - is a special case: We don't take e.g. "-5" as relative, it
-        // describes a negative number
-        else if (operations[index] == "-")
-            *value = 0;
         else
             *value = x;
     }
@@ -331,6 +343,8 @@ void XmlCharacteristic::load(const XMLNode *node)
             &m_values[STABILITY_DOWNWARD_IMPULSE_FACTOR]);
         sub_node->get("track-connection-accel",
             &m_values[STABILITY_TRACK_CONNECTION_ACCEL]);
+        sub_node->get("angular-factor",
+            &m_values[STABILITY_ANGULAR_FACTOR]);
         sub_node->get("smooth-flying-impulse",
             &m_values[STABILITY_SMOOTH_FLYING_IMPULSE]);
     }
@@ -423,6 +437,10 @@ void XmlCharacteristic::load(const XMLNode *node)
             &m_values[PARACHUTE_DURATION]);
         sub_node->get("duration-other",
             &m_values[PARACHUTE_DURATION_OTHER]);
+        sub_node->get("duration-rank-mult",
+            &m_values[PARACHUTE_DURATION_RANK_MULT]);
+        sub_node->get("duration-speed-mult",
+            &m_values[PARACHUTE_DURATION_SPEED_MULT]);
         sub_node->get("lbound-fraction",
             &m_values[PARACHUTE_LBOUND_FRACTION]);
         sub_node->get("ubound-fraction",
@@ -431,6 +449,12 @@ void XmlCharacteristic::load(const XMLNode *node)
             &m_values[PARACHUTE_MAX_SPEED]);
     }
 
+    if (const XMLNode *sub_node = node->getNode("friction"))
+    {
+        sub_node->get("kart-friction",
+            &m_values[FRICTION_KART_FRICTION]);
+    }
+
     if (const XMLNode *sub_node = node->getNode("bubblegum"))
     {
         sub_node->get("duration",
