@@ -61,6 +61,7 @@ CutsceneWorld::CutsceneWorld() : World()
     m_play_track_intro_sound = false;
     m_play_ready_set_go_sounds = false;
     m_fade_duration = 1.0f;
+    m_camera = NULL;
 }   // CutsceneWorld
 
 //-----------------------------------------------------------------------------
@@ -74,17 +75,18 @@ void CutsceneWorld::init()
 
     dynamic_cast<CutsceneGUI*>(m_race_gui)->setFadeLevel(1.0f);
 
-    getTrack()->startMusic();
+    Track::getCurrentTrack()->startMusic();
 
     m_duration = -1.0f;
 
     Camera* stk_cam = Camera::createCamera(NULL);
     m_camera = stk_cam->getCameraSceneNode();
-    m_camera->setFOV(0.61f);
+    m_camera->setFOV(stk_config->m_cutscene_fov);
     m_camera->bindTargetAndRotation(true); // no "look-at"
 
     // --- Build list of sounds to play at certain frames
-    PtrVector<TrackObject>& objects = m_track->getTrackObjectManager()->getObjects();
+    PtrVector<TrackObject>& objects = Track::getCurrentTrack()
+                                    ->getTrackObjectManager()->getObjects();
     for (TrackObject* curr : objects)
     {
         if (curr->getType() == "particle-emitter" &&
@@ -198,7 +200,8 @@ void CutsceneWorld::update(float dt)
     {
         //printf("INITIAL TIME for CutsceneWorld\n");
 
-        PtrVector<TrackObject>& objects = m_track->getTrackObjectManager()->getObjects();
+        PtrVector<TrackObject>& objects = Track::getCurrentTrack()
+                                        ->getTrackObjectManager()->getObjects();
         TrackObject* curr;
         for_in(curr, objects)
         {
@@ -212,7 +215,8 @@ void CutsceneWorld::update(float dt)
     {
         m_second_reset = false;
 
-        PtrVector<TrackObject>& objects = m_track->getTrackObjectManager()->getObjects();
+        PtrVector<TrackObject>& objects = Track::getCurrentTrack()
+                                        ->getTrackObjectManager()->getObjects();
         TrackObject* curr;
         for_in(curr, objects)
         {
@@ -264,7 +268,8 @@ void CutsceneWorld::update(float dt)
 
     //printf("Estimated current frame : %f\n", curr_frame);
 
-    const std::vector<Subtitle>& subtitles = m_track->getSubtitles();
+    const std::vector<Subtitle>& subtitles = Track::getCurrentTrack()
+                                           ->getSubtitles();
     bool foundSubtitle = false;
     for (unsigned int n = 0; n < subtitles.size(); n++)
     {
@@ -286,7 +291,8 @@ void CutsceneWorld::update(float dt)
     World::update((float)dt);
     World::updateTrack((float)dt);
 
-    PtrVector<TrackObject>& objects = m_track->getTrackObjectManager()->getObjects();
+    PtrVector<TrackObject>& objects = Track::getCurrentTrack()
+                                    ->getTrackObjectManager()->getObjects();
     TrackObject* curr;
     for_in(curr, objects)
     {
@@ -301,6 +307,11 @@ void CutsceneWorld::update(float dt)
             rot2.setPitch(rot2.getPitch() + 90.0f);
             m_camera->setRotation(rot2.toIrrVector());
 
+            irr::core::vector3df up(0.0f, 0.0f, 1.0f);
+            irr::core::matrix4 matrix = anchorNode->getAbsoluteTransformation();
+            matrix.rotateVect(up);
+            m_camera->setUpVector(up);
+
             SFXManager::get()->positionListener(m_camera->getAbsolutePosition(),
                                           m_camera->getTarget() -
                                             m_camera->getAbsolutePosition(),
