@@ -23,6 +23,7 @@
 #include "graphics/camera.hpp"
 #include "items/powerup_manager.hpp"
 #include "karts/abstract_kart.hpp"
+#include "karts/controller/controller.hpp"
 #include "states_screens/race_gui_base.hpp"
 #include "tracks/track.hpp"
 #include "utils/translation.hpp"
@@ -76,6 +77,7 @@ FollowTheLeaderRace::~FollowTheLeaderRace()
 void FollowTheLeaderRace::reset()
 {
     LinearWorld::reset();
+    m_last_eliminated_time = 0.0f;
     m_leader_intervals.clear();
     m_leader_intervals    = stk_config->m_leader_intervals;
     for(unsigned int i=0; i<m_leader_intervals.size(); i++)
@@ -83,7 +85,7 @@ void FollowTheLeaderRace::reset()
             stk_config->m_leader_time_per_kart*race_manager->getNumberOfKarts();
     WorldStatus::setClockMode(WorldStatus::CLOCK_COUNTDOWN,
                               m_leader_intervals[0]);
-                              
+    
     m_is_over_delay = 2.0f;
 }   // reset
 
@@ -105,23 +107,14 @@ int FollowTheLeaderRace::getScoreForPosition(int p)
 const btTransform &FollowTheLeaderRace::getStartTransform(int index)
 {
     if (index == 0)   // Leader start position
-        return m_track->getStartTransform(index);
+        return Track::getCurrentTrack()->getStartTransform(index);
 
     // Otherwise the karts will start at the rear starting positions
     int start_index = stk_config->m_max_karts
                     - race_manager->getNumberOfKarts() + index;
-    return m_track->getStartTransform(start_index);
+    return Track::getCurrentTrack()->getStartTransform(start_index);
 }   // getStartTransform
 
-//-----------------------------------------------------------------------------
-/** Returns the original time at which the countdown timer started. This is
- *  used by the race_gui to display the music credits in FTL mode correctly.
- */
-float FollowTheLeaderRace::getClockStartTime() const
-{
-    return m_leader_intervals[0];
-}   // getClockStartTime
-
 //-----------------------------------------------------------------------------
 /** Called when a kart must be eliminated.
  */
@@ -259,7 +252,7 @@ void FollowTheLeaderRace::terminateRace()
     endSetKartPositions();
 
     // Mark all still racing karts to be finished.
-    for (int i = m_karts.size(); i>0; i--)
+    for (int i = (int)m_karts.size(); i>0; i--)
     {
         AbstractKart *kart = getKartAtPosition(i);
         if (kart->isEliminated() || kart->hasFinishedRace())
