@@ -17,24 +17,29 @@
 
 #include "modes/soccer_world.hpp"
 
+#include "main_loop.hpp"
 #include "audio/music_manager.hpp"
 #include "audio/sfx_base.hpp"
+#include "config/user_config.hpp"
 #include "io/file_manager.hpp"
+#include "graphics/central_settings.hpp"
 #include "graphics/irr_driver.hpp"
-#include "karts/abstract_kart.hpp"
+#include "graphics/render_info.hpp"
 #include "karts/kart.hpp"
 #include "karts/kart_model.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/rescue_animation.hpp"
 #include "karts/controller/local_player_controller.hpp"
-#include "karts/controller/soccer_ai.hpp"
+#include "karts/controller/network_player_controller.hpp"
 #include "physics/physics.hpp"
 #include "states_screens/race_gui_base.hpp"
 #include "tracks/track.hpp"
 #include "tracks/track_object_manager.hpp"
+#include "tracks/track_sector.hpp"
 #include "utils/constants.hpp"
 
 #include <IMeshSceneNode.h>
+#include <numeric>
 #include <string>
 //-----------------------------------------------------------------------------
 /** Constructor. Sets up the clock mode etc.
@@ -51,9 +56,11 @@ SoccerWorld::SoccerWorld() : WorldWithRank()
         WorldStatus::setClockMode(CLOCK_CHRONO);
     }
 
+    m_frame_count = 0;
     m_use_highscores = false;
     m_red_ai = 0;
     m_blue_ai = 0;
+    m_ball_track_sector = NULL;
 }   // SoccerWorld
 
 //-----------------------------------------------------------------------------
@@ -62,6 +69,9 @@ SoccerWorld::SoccerWorld() : WorldWithRank()
 SoccerWorld::~SoccerWorld()
 {
     m_goal_sound->deleteSFX();
+
+    delete m_ball_track_sector;
+    m_ball_track_sector = NULL;
 }   // ~SoccerWorld
 
 //-----------------------------------------------------------------------------
@@ -81,7 +91,14 @@ void SoccerWorld::init()
     m_goal_target = race_manager->getMaxGoal();
     m_goal_sound = SFXManager::get()->createSoundSource("goal_scored");
 
-    TrackObjectManager* tom = getTrack()->getTrackObjectManager();
+    Track *track = Track::getCurrentTrack();
+    if (track->hasNavMesh())
+    {
+        // Init track sector for ball if navmesh is found
+        m_ball_track_sector = new TrackSector();
+    }
+
+    TrackObjectManager* tom = track->getTrackObjectManager();
     assert(tom);
     PtrVector<TrackObject>& objects = tom->getObjects();
     for (unsigned int i = 0; i < objects.size(); i++)
@@ -134,11 +151,20 @@ void SoccerWorld::reset()
         m_goal_sound->stop();
     }
 
+    if (Track::getCurrentTrack()->hasNavMesh())
+    {
+        m_ball_track_sector->reset();
+    }
+
     initKartList();
-    resetAllPosition();
     m_ball->reset();
     m_bgd.reset();
 
+    // Make the player kart in profiling mode up
+    // ie make this kart less likely to affect gaming result
+    if (UserConfigParams::m_arena_ai_stats)
+        getKart(8)->flyUp();
+
 }   // reset
 
 //-----------------------------------------------------------------------------
@@ -156,9 +182,9 @@ const std::string& SoccerWorld::getIdent() const
 void SoccerWorld::update(float dt)
 {
     updateBallPosition(dt);
-    if (m_track->hasNavMesh())
+    if (Track::getCurrentTrack()->hasNavMesh())
     {
-        updateKartNodes();
+        updateSectorForKarts();
         updateAIData();
     }
 
@@ -184,9 +210,13 @@ void SoccerWorld::update(float dt)
                 // Reset all karts
                 for (unsigned int i = 0; i < m_karts.size(); i++)
                     moveKartAfterRescue(m_karts[i]);
+                if (UserConfigParams::m_arena_ai_stats)
+                    getKart(8)->flyUp();
             }
         }
     }
+    if (UserConfigParams::m_arena_ai_stats)
+        m_frame_count++;
 
 }   // update
 
@@ -200,6 +230,13 @@ void SoccerWorld::onCheckGoalTriggered(bool first_goal)
     m_goal_sound->play();
     if (m_ball_hitter != -1)
     {
+        if (UserConfigParams::m_arena_ai_stats)
+        {
+            const int elapsed_frame = m_goal_frame.empty() ? 0 :
+                std::accumulate(m_goal_frame.begin(), m_goal_frame.end(), 0);
+            m_goal_frame.push_back(m_frame_count - elapsed_frame);
+        }
+
         ScorerData sd;
         sd.m_id = m_ball_hitter;
         sd.m_correct_goal = isCorrectGoal(m_ball_hitter, first_goal);
@@ -284,28 +321,35 @@ void SoccerWorld::countdownReachedZero()
 //-----------------------------------------------------------------------------
 void SoccerWorld::initKartList()
 {
+#ifndef SERVER_ONLY
     const unsigned int kart_amount = (unsigned int)m_karts.size();
 
     //Loading the indicator textures
-    irr::video::ITexture *red =
-            irr_driver->getTexture(FileManager::GUI, "soccer_player_red.png");
-    irr::video::ITexture *blue =
-            irr_driver->getTexture(FileManager::GUI, "soccer_player_blue.png");
+    std::string red_path =
+            file_manager->getAsset(FileManager::GUI, "soccer_player_red.png");
+    std::string blue_path =
+            file_manager->getAsset(FileManager::GUI, "soccer_player_blue.png");
 
     //Assigning indicators
-    for(unsigned int i=0; i<kart_amount; i++)
+    for(unsigned int i = 0; i < kart_amount; i++)
     {
-        scene::ISceneNode *arrow_node;
-        float arrow_pos_height = m_karts[i]->getKartModel()->getHeight()+0.5f;
+        scene::ISceneNode *arrow_node = NULL;
+
+        KartModel* km = m_karts[i]->getKartModel();
+        // Color of karts can be changed using shaders if the model supports
+        if (km->supportColorization() && CVS->isGLSL()) continue;
+
+        float arrow_pos_height = km->getHeight() + 0.5f;
         SoccerTeam team = getKartTeam(i);
 
-        arrow_node = irr_driver->addBillboard(core::dimension2d<irr::f32>(0.3f,
-            0.3f), team == SOCCER_TEAM_BLUE ? blue : red, m_karts[i]
-            ->getNode(), true);
+        arrow_node = irr_driver->addBillboard(
+            core::dimension2d<irr::f32>(0.3f,0.3f),
+            team == SOCCER_TEAM_BLUE ? blue_path : red_path,
+            m_karts[i]->getNode());
 
         arrow_node->setPosition(core::vector3df(0, arrow_pos_height, 0));
     }
-
+#endif
 }   // initKartList
 
 //-----------------------------------------------------------------------------
@@ -369,9 +413,7 @@ AbstractKart *SoccerWorld::createKart(const std::string &kart_ident, int index,
     m_kart_position_map[index] = (unsigned)(pos_index - 1);
 
     AbstractKart *new_kart = new Kart(kart_ident, index, position, init_pos,
-              difficulty);
-            //difficulty, team == SOCCER_TEAM_BLUE ?
-            //video::ERT_BLUE : video::ERT_RED);
+            difficulty, team == SOCCER_TEAM_BLUE ? KRT_BLUE : KRT_RED);
     new_kart->init(race_manager->getKartType(index));
     Controller *controller = NULL;
 
@@ -383,11 +425,9 @@ AbstractKart *SoccerWorld::createKart(const std::string &kart_ident, int index,
         m_num_players ++;
         break;
     case RaceManager::KT_NETWORK_PLAYER:
-        break;  // Avoid compiler warning about enum not handled.
-        //controller = new NetworkController(kart_ident, position, init_pos,
-        //                          global_player_id);
-        //m_num_players++;
-        //break;
+        controller = new NetworkPlayerController(new_kart);
+        m_num_players++;
+        break;
     case RaceManager::KT_AI:
         controller = loadAIController(new_kart);
         break;
@@ -395,6 +435,8 @@ AbstractKart *SoccerWorld::createKart(const std::string &kart_ident, int index,
         break;
     case RaceManager::KT_LEADER:
         break;
+    case RaceManager::KT_SPARE_TIRE:
+        break;
     }
 
     new_kart->setController(controller);
@@ -402,24 +444,6 @@ AbstractKart *SoccerWorld::createKart(const std::string &kart_ident, int index,
     return new_kart;
 }   // createKart
 
-//-----------------------------------------------------------------------------
-/** Updates the m_kart_on_node value of each kart to localize it
- *  on the navigation mesh.
- */
-void SoccerWorld::updateKartNodes()
-{
-    if (isRaceOver()) return;
-
-    const unsigned int n = getNumKarts();
-    for (unsigned int i = 0; i < n; i++)
-    {
-        if (m_karts[i]->isEliminated()) continue;
-
-        m_kart_on_node[i] = BattleGraph::get()->pointToNode(m_kart_on_node[i],
-                            m_karts[i]->getXYZ(), false/*ignore_vertical*/);
-    }
-}   // updateKartNodes
-
 //-----------------------------------------------------------------------------
 /** Localize the ball on the navigation mesh.
  */
@@ -434,13 +458,11 @@ void SoccerWorld::updateBallPosition(float dt)
             m_ball_body->getLinearVelocity().getZ());
     }
 
-    if (m_track->hasNavMesh())
+    if (Track::getCurrentTrack()->hasNavMesh())
     {
-        m_ball_on_node  = BattleGraph::get()->pointToNode(m_ball_on_node,
-                          getBallPosition(), true/*ignore_vertical*/);
-
-        if (m_ball_on_node == BattleGraph::UNKNOWN_POLY &&
-            getPhase() == RACE_PHASE)
+        m_ball_track_sector
+            ->update(getBallPosition(), true/*ignore_vertical*/);
+        if (!m_ball_track_sector->isOnRoad() && getPhase() == RACE_PHASE)
         {
             m_ball_invalid_timer += dt;
             // Reset the ball and karts if out of navmesh after 2 seconds
@@ -450,6 +472,8 @@ void SoccerWorld::updateBallPosition(float dt)
                 m_ball->reset();
                 for (unsigned int i = 0; i < m_karts.size(); i++)
                     moveKartAfterRescue(m_karts[i]);
+                if (UserConfigParams::m_arena_ai_stats)
+                    getKart(8)->flyUp();
             }
         }
         else
@@ -459,12 +483,12 @@ void SoccerWorld::updateBallPosition(float dt)
 }   // updateBallPosition
 
 //-----------------------------------------------------------------------------
-void SoccerWorld::resetAllPosition()
+int SoccerWorld::getBallNode() const
 {
-    m_kart_on_node.clear();
-    m_kart_on_node.resize(m_karts.size(), BattleGraph::UNKNOWN_POLY);
-    m_ball_on_node = BattleGraph::UNKNOWN_POLY;
-}   // resetAllPosition
+    assert(m_ball_track_sector != NULL);
+    return m_ball_track_sector->getCurrentGraphNode();
+}   // getBallNode
+
 //-----------------------------------------------------------------------------
 SoccerTeam SoccerWorld::getKartTeam(unsigned int kart_id) const
 {
@@ -503,6 +527,10 @@ void SoccerWorld::updateAIData()
 
     for (unsigned int i = 0; i < m_karts.size(); ++i)
     {
+        if (UserConfigParams::m_arena_ai_stats &&
+            m_karts[i]->getController()->isPlayerController())
+            continue;
+
         if (getKartTeam(m_karts[i]->getWorldKartId()) == SOCCER_TEAM_RED)
         {
             Vec3 rd = m_karts[i]->getXYZ() - getBallPosition();
@@ -551,6 +579,7 @@ int SoccerWorld::getAttacker(SoccerTeam team) const
     // No attacker
     return -1;
 }   // getAttacker
+
 //-----------------------------------------------------------------------------
 int SoccerWorld::getTeamNum(SoccerTeam team) const
 {
@@ -575,6 +604,81 @@ unsigned int SoccerWorld::getRescuePositionIndex(AbstractKart *kart)
     return n->second;
 }   // getRescuePositionIndex
 
+//-----------------------------------------------------------------------------
+void SoccerWorld::enterRaceOverState()
+{
+    WorldWithRank::enterRaceOverState();
+
+    if (UserConfigParams::m_arena_ai_stats)
+    {
+        Log::verbose("Soccer AI profiling", "Total frames elapsed for a team"
+            " to win with 30 goals: %d", m_frame_count);
+
+        // Goal time statistics
+        std::sort(m_goal_frame.begin(), m_goal_frame.end());
+
+        const int mean = std::accumulate(m_goal_frame.begin(),
+            m_goal_frame.end(), 0) / (int)m_goal_frame.size();
+
+        // Prevent overflow if there is a large frame in vector
+        double squared_sum = 0;
+        for (const int &i : m_goal_frame)
+            squared_sum = squared_sum + (double(i - mean) * double(i - mean));
+
+        // Use sample st. deviation (n−1) as the profiling can't be run forever
+        const int stdev = int(sqrt(squared_sum / (m_goal_frame.size() - 1)));
+
+        int median = 0;
+        if (m_goal_frame.size() % 2 == 0)
+        {
+            median = (m_goal_frame[m_goal_frame.size() / 2 - 1] +
+                m_goal_frame[m_goal_frame.size() / 2]) / 2;
+        }
+        else
+        {
+            median = m_goal_frame[m_goal_frame.size() / 2];
+        }
+
+        Log::verbose("Soccer AI profiling", "Frames elapsed for each goal:"
+            " min: %d max: %d mean: %d median: %d standard deviation: %d",
+            m_goal_frame.front(), m_goal_frame.back(), mean, median, stdev);
+
+        // Goal calculation
+        int red_own_goal = 0;
+        int blue_own_goal = 0;
+        for (unsigned i = 0; i < m_red_scorers.size(); i++)
+        {
+            // Notice: if a team has own goal, the score will end up in the
+            // opposite team
+            if (!m_red_scorers[i].m_correct_goal)
+                blue_own_goal++;
+        }
+        for (unsigned i = 0; i < m_blue_scorers.size(); i++)
+        {
+            if (!m_blue_scorers[i].m_correct_goal)
+                red_own_goal++;
+        }
+
+        int red_goal = ((int(m_red_scorers.size()) - blue_own_goal) >= 0 ?
+            (int)m_red_scorers.size() - blue_own_goal : 0);
+        int blue_goal = ((int(m_blue_scorers.size()) - red_own_goal) >= 0 ?
+            (int)m_blue_scorers.size() - red_own_goal : 0);
+
+        Log::verbose("Soccer AI profiling", "Red goal: %d, Red own goal: %d,"
+            "Blue goal: %d, Blue own goal: %d", red_goal, red_own_goal,
+            blue_goal, blue_own_goal);
+
+        if (getScore(SOCCER_TEAM_BLUE) >= m_goal_target)
+            Log::verbose("Soccer AI profiling", "Blue team wins");
+        else
+            Log::verbose("Soccer AI profiling", "Red team wins");
+
+        delete this;
+        main_loop->abort();
+    }
+
+}   // enterRaceOverState
+
 //-----------------------------------------------------------------------------
 void SoccerWorld::setAITeam()
 {
@@ -595,6 +699,7 @@ void SoccerWorld::setAITeam()
         {
             race_manager->setKartSoccerTeam(i, SOCCER_TEAM_BLUE);
             team = SOCCER_TEAM_BLUE;
+            continue;
         }
 
         team == SOCCER_TEAM_BLUE ? blue_player++ : red_player++;
