@@ -18,13 +18,26 @@
 #include "modes/world_with_rank.hpp"
 
 #include "karts/abstract_kart.hpp"
+#include "karts/controller/spare_tire_ai.hpp"
 #include "karts/kart_properties.hpp"
 #include "race/history.hpp"
+#include "tracks/graph.hpp"
 #include "tracks/track.hpp"
+#include "tracks/track_sector.hpp"
 #include "utils/log.hpp"
 
 #include <iostream>
 
+//-----------------------------------------------------------------------------
+WorldWithRank::~WorldWithRank()
+{
+    for (unsigned int i = 0; i < m_kart_track_sector.size(); i++)
+    {
+        delete m_kart_track_sector[i];
+    }
+    m_kart_track_sector.clear();
+}   // ~WorldWithRank
+
 //-----------------------------------------------------------------------------
 void WorldWithRank::init()
 {
@@ -39,8 +52,27 @@ void WorldWithRank::init()
 #endif
     stk_config->getAllScores(&m_score_for_position, getNumKarts());
 
+    Track *track = Track::getCurrentTrack();
+    // Don't init track sector if navmesh is not found in arena
+    if ((track->isArena() || track->isSoccer()) && !track->hasNavMesh())
+        return;
+
+    for (unsigned int i = 0; i < m_karts.size(); i++)
+        m_kart_track_sector.push_back(new TrackSector());
+
 }   // init
 
+//-----------------------------------------------------------------------------
+void WorldWithRank::reset()
+{
+    World::reset();
+    for (unsigned int i = 0; i < m_kart_track_sector.size(); i++)
+    {
+        getTrackSector(i)->reset();
+        getTrackSector(i)->update(m_karts[i]->getXYZ());
+    }
+}   // reset
+
 //-----------------------------------------------------------------------------
 /** Returns the kart with a given position.
  *  \param p The position of the kart, 1<=p<=num_karts).
@@ -124,16 +156,6 @@ void WorldWithRank::endSetKartPositions()
 #endif
 }   // endSetKartPositions
 
-
-//-----------------------------------------------------------------------------
-/** WorldWithRank uses the start position as rescue positions. So return
- *  the number of start positions.
- */
-unsigned int WorldWithRank::getNumberOfRescuePositions() const
-{
-    return getTrack()->getNumberOfStartPositions();
-}   // getNumberOfRescuePositions
-
 //-----------------------------------------------------------------------------
 /** Determines the rescue position for a kart. The rescue position is the
  *  start position which is has the biggest accumulated distance to all other
@@ -146,7 +168,8 @@ unsigned int WorldWithRank::getNumberOfRescuePositions() const
 
 unsigned int WorldWithRank::getRescuePositionIndex(AbstractKart *kart)
 {
-    const int start_spots_amount = getTrack()->getNumberOfStartPositions();
+    const int start_spots_amount =
+                         Track::getCurrentTrack()->getNumberOfStartPositions();
     assert(start_spots_amount > 0);
 
     float largest_accumulated_distance_found = -1;
@@ -162,7 +185,7 @@ unsigned int WorldWithRank::getRescuePositionIndex(AbstractKart *kart)
         for(unsigned int k=0; k<getCurrentNumKarts(); k++)
         {
             if(kart->getWorldKartId()==k) continue;
-            float abs_distance2 = (getKart(k)->getXYZ()-v).length2_2d();
+            float abs_distance2 = (getKart(k)->getXYZ()-v).length2();
             const float CLEAR_SPAWN_RANGE2 = 5*5;
             if( abs_distance2 < CLEAR_SPAWN_RANGE2)
             {
@@ -184,16 +207,6 @@ unsigned int WorldWithRank::getRescuePositionIndex(AbstractKart *kart)
     return furthest_id_found;
 }   // getRescuePositionIndex
 
-// ----------------------------------------------------------------------------
-/** Returns the start transform with the give index.
- *  \param rescue_pos Index of the start position to be returned.
- *  \returns The transform of the corresponding start position.
- */
-btTransform WorldWithRank::getRescueTransform(unsigned int rescue_pos) const
-{
-    return getTrack()->getStartTransform(rescue_pos);
-}   // getRescueTransform
-
 //-----------------------------------------------------------------------------
 /** Returns the number of points for a kart at a specified position.
  *  \param p Position (starting with 1).
@@ -204,3 +217,46 @@ int WorldWithRank::getScoreForPosition(int p)
     assert(p - 1 <(int) m_score_for_position.size());
     return m_score_for_position[p - 1];
 }   // getScoreForPosition
+
+//-----------------------------------------------------------------------------
+/** Returns true if the kart is on a valid graph quad.
+ *  \param kart_index  Index of the kart.
+ */
+bool WorldWithRank::isOnRoad(unsigned int kart_index) const
+{
+    return getTrackSector(kart_index)->isOnRoad();
+}   // isOnRoad
+
+//-----------------------------------------------------------------------------
+/** Gets the sector a kart is on. This function returns UNKNOWN_SECTOR if the
+ *  kart_id is larger than the current kart sector. This is necessary in the
+ *  case that a collision with the track happens during resetAllKarts: at this
+ *  time m_kart_track_sector is not initialised (and has size 0), so it would
+ *  trigger this assert. While this normally does not happen, it is useful for
+ *  track designers that STK does not crash.
+ *  \param kart Kart for which to return the sector.
+ */
+int WorldWithRank::getSectorForKart(const AbstractKart *kart) const
+{
+    if (kart->getWorldKartId() >= m_kart_track_sector.size())
+        return Graph::UNKNOWN_SECTOR;
+    return getTrackSector(kart->getWorldKartId())->getCurrentGraphNode();
+}   // getSectorForKart
+
+//-----------------------------------------------------------------------------
+/** Localize each kart on the graph using its center xyz.
+ */
+void WorldWithRank::updateSectorForKarts()
+{
+    if (isRaceOver()) return;
+
+    const unsigned int n = getNumKarts();
+    assert(n == m_kart_track_sector.size());
+    for (unsigned int i = 0; i < n; i++)
+    {
+        SpareTireAI* sta =
+            dynamic_cast<SpareTireAI*>(m_karts[i]->getController());
+        if (!m_karts[i]->isEliminated() || (sta && sta->isMoving()))
+            getTrackSector(i)->update(m_karts[i]->getXYZ());
+    }
+}   // updateSectorForKarts
