@@ -53,9 +53,9 @@ Protocol::~Protocol()
 /** Returns a network string with the given type.
  *  \capacity Default preallocated size for the message.
  */
-NetworkString* Protocol::getNetworkString(int capacity)
+NetworkString* Protocol::getNetworkString(size_t capacity)
 {
-    return new NetworkString(m_type, capacity);
+    return new NetworkString(m_type, (int)capacity);
 }   // getNetworkString
 
 // ----------------------------------------------------------------------------
@@ -118,8 +118,7 @@ void Protocol::findAndTerminateProtocol(ProtocolType type)
     if (protocol)
         protocol->requestTerminate();
     else
-        Log::error("ClientLobbyRoomProtocol",
-                    "No protocol %d registered.", type);
+        Log::error("Protocol", "No protocol %d registered.", type);
 }   // findAndTerminateProtocol
 
 // ----------------------------------------------------------------------------
