@@ -26,7 +26,7 @@
 #include "network/protocols/hide_public_address.hpp"
 #include "network/protocols/request_connection.hpp"
 #include "network/protocols/ping_protocol.hpp"
-#include "network/protocols/client_lobby_room_protocol.hpp"
+#include "network/protocols/client_lobby.hpp"
 #include "network/protocol_manager.hpp"
 #include "network/servers_manager.hpp"
 #include "network/stk_host.hpp"
@@ -132,14 +132,7 @@ void ConnectToServer::asynchronousUpdate()
             }
             else
             {
-                // No quick connect, so we have a server to connect to.
-                // Find its address
-                m_current_protocol = new GetPeerAddress(m_host_id, this);
-                m_current_protocol->requestStart();
                 m_state = GOT_SERVER_ADDRESS;
-                // Pause this protocol till GetPeerAddress finishes.
-                // The callback then will unpause this protocol/
-                requestPause();
             }
         }
         break;
@@ -182,9 +175,10 @@ void ConnectToServer::asynchronousUpdate()
                     m_current_protocol->requestStart();
                     return;
                 }
-                if (m_server_address.getIP() 
-                      == NetworkConfig::get()->getMyAddress().getIP() ||
-                      NetworkConfig::get()->isLAN())
+                if( ( !NetworkConfig::m_disable_lan && 
+                       m_server_address.getIP() 
+                         == NetworkConfig::get()->getMyAddress().getIP() )  ||
+                      NetworkConfig::get()->isLAN()                            )
                 {
                     // We're in the same lan (same public ip address).
                     // The state will change to CONNECTING
@@ -244,7 +238,9 @@ void ConnectToServer::asynchronousUpdate()
                 // lobby room protocol if we're connected only
                 if(STKHost::get()->getPeers()[0]->isConnected())
                 {
-                    Protocol *p = new ClientLobbyRoomProtocol(m_server_address);
+                    ClientLobby *p = 
+                        LobbyProtocol::create<ClientLobby>();
+                    p->setAddress(m_server_address);
                     p->requestStart();
                 }
             }
@@ -270,11 +266,6 @@ void ConnectToServer::callback(Protocol *protocol)
             // STKHost, so we only need to unpause this protocol
             requestUnpause();
             break;
-        case GOT_SERVER_ADDRESS:
-            // Get the server address from the protocol.
-            m_server_address.copy(((GetPeerAddress*)protocol)->getAddress());
-            requestUnpause();
-            break;
         default:
             Log::error("ConnectToServer",
                        "Received unexpected callback while in state %d.",
@@ -296,7 +287,7 @@ void ConnectToServer::registerWithSTKServer()
     request->addParameter("address", addr.getIP());
     request->addParameter("port", addr.getPort());
     request->addParameter("private_port",
-                          NetworkConfig::get()->getPrivatePort());
+                          NetworkConfig::get()->getClientPort());
 
     Log::info("ConnectToServer", "Registering addr %s",
               addr.toString().c_str());
