@@ -11,10 +11,18 @@ class GameEventsProtocol : public Protocol
 {
 private:
     enum GameEventType {
-        GE_ITEM_COLLECTED     = 0x01,
-        GE_KART_FINISHED_RACE = 0x02
+        GE_CLIENT_STARTED_RSG = 0x01,
+        GE_ITEM_COLLECTED     = 0x02,
+        GE_KART_FINISHED_RACE = 0x03
     };   // GameEventType
 
+    /** Count how many clients have started 'ready'. The server
+     *  will only go to its 'ready' phase if all client shave done so.
+     *  This means the server time is far enough behind the clients
+     *  that at time T all client messages for time T have been 
+     *  received (short of latency spikes). */
+    int m_count_ready_clients;
+
 public:
              GameEventsProtocol();
     virtual ~GameEventsProtocol();
@@ -23,8 +31,10 @@ public:
     void collectedItem(Item* item, AbstractKart* kart);
     void collectedItem(const NetworkString &ns);
     void kartFinishedRace(AbstractKart *kart, float time);
+    void clientHasStarted();
+    void receivedClientHasStarted(Event *event);
     void kartFinishedRace(const NetworkString &ns);
-    virtual void setup() OVERRIDE {};
+    virtual void setup() OVERRIDE;
     virtual void update(float dt) OVERRIDE {};
     virtual void asynchronousUpdate() OVERRIDE{}
     // ------------------------------------------------------------------------
