@@ -22,8 +22,9 @@
 #include "LinearMath/btIDebugDraw.h"
 #include "BulletDynamics/ConstraintSolver/btContactConstraint.h"
 
+#include "graphics/material.hpp"
 #include "karts/kart.hpp"
-#include "modes/world.hpp"
+#include "karts/kart_model.hpp"
 #include "physics/triangle_mesh.hpp"
 #include "tracks/terrain_info.hpp"
 #include "tracks/track.hpp"
@@ -116,8 +117,7 @@ void btKart::reset()
         updateWheelTransform(i, true);
     }
     m_visual_wheels_touch_ground = false;
-    m_zipper_active              = false;
-    m_zipper_velocity            = btScalar(0);
+    m_zipper_speed               = btScalar(0);
     m_skid_angular_velocity      = 0;
     m_is_skidding                = false;
     m_allow_sliding              = false;
@@ -193,7 +193,8 @@ void btKart::resetSuspension()
 
 // ----------------------------------------------------------------------------
 void btKart::updateWheelTransformsWS(btWheelInfo& wheel,
-                                     bool interpolatedTransform)
+                                     bool interpolatedTransform,
+                                     float fraction)
 {
     wheel.m_raycastInfo.m_isInContact = false;
 
@@ -204,7 +205,7 @@ void btKart::updateWheelTransformsWS(btWheelInfo& wheel,
     }
 
     wheel.m_raycastInfo.m_hardPointWS =
-        chassisTrans( wheel.m_chassisConnectionPointCS );
+        chassisTrans( wheel.m_chassisConnectionPointCS*fraction );
     wheel.m_raycastInfo.m_wheelDirectionWS = chassisTrans.getBasis() *
                                                 wheel.m_wheelDirectionCS ;
     wheel.m_raycastInfo.m_wheelAxleWS      = chassisTrans.getBasis() *
@@ -214,7 +215,7 @@ void btKart::updateWheelTransformsWS(btWheelInfo& wheel,
 // ----------------------------------------------------------------------------
 /**
  */
-btScalar btKart::rayCast(unsigned int index)
+btScalar btKart::rayCast(unsigned int index, float fraction)
 {
     btWheelInfo &wheel = m_wheelInfo[index];
 
@@ -230,7 +231,7 @@ btScalar btKart::rayCast(unsigned int index)
         m_chassisBody->getBroadphaseHandle()->m_collisionFilterGroup = 0;
     }
 
-    updateWheelTransformsWS( wheel,false);
+    updateWheelTransformsWS( wheel,false, fraction);
 
     btScalar max_susp_len = wheel.getSuspensionRestLength()
                           + wheel.m_maxSuspensionTravel;
@@ -392,6 +393,16 @@ void btKart::updateVehicle( btScalar step )
         rayCast( i);
         if(m_wheelInfo[i].m_raycastInfo.m_isInContact)
             m_num_wheels_on_ground++;
+        else
+        {
+            // If the original raycast did not hit the ground,
+            // try a little bit (5%) closer to the centre of the chassis.
+            // Some tracks have very minor gaps that would otherwise
+            // trigger odd physical behaviour.
+            rayCast(i, 0.95f);
+            if (m_wheelInfo[i].m_raycastInfo.m_isInContact)
+                m_num_wheels_on_ground++;
+        }
     }
 
     // Test if the kart is falling so fast 
@@ -445,7 +456,9 @@ void btKart::updateVehicle( btScalar step )
     if(m_num_wheels_on_ground==0)
     {
         btVector3 kart_up    = getChassisWorldTransform().getBasis().getColumn(1);
-        btVector3 terrain_up(0,1,0);
+        btVector3 terrain_up =
+            m_kart->getMaterial() && m_kart->getMaterial()->hasGravity() ?
+            m_kart->getNormal() : Vec3(0, 1, 0);
         // Length of axis depends on the angle - i.e. the further awat
         // the kart is from being upright, the larger the applied impulse
         // will be, resulting in fast changes when the kart is on its
@@ -463,33 +476,6 @@ void btKart::updateVehicle( btScalar step )
         // Give a nicely balanced feeling for rebalancing the kart
         m_chassisBody->applyTorqueImpulse(axis * m_kart->getKartProperties()->getStabilitySmoothFlyingImpulse());
     }
-    
-    // Work around: make sure that either both wheels on one axis
-    // are on ground, or none of them. This avoids the problem of
-    // the kart suddenly getting additional angular velocity because
-    // e.g. only one rear wheel is on the ground and then the kart
-    // rotates very abruptly.
-    for(int i=0; i<m_wheelInfo.size(); i+=2)
-    {
-        if( m_wheelInfo[i  ].m_raycastInfo.m_isInContact !=
-            m_wheelInfo[i+1].m_raycastInfo.m_isInContact)
-        {
-            int wheel_air_index = i;
-            int wheel_ground_index = i+1;
-
-            if (m_wheelInfo[i].m_raycastInfo.m_isInContact)
-            {
-                wheel_air_index = i+1;
-                wheel_ground_index = i;
-            }
-
-            btWheelInfo& wheel_air = m_wheelInfo[wheel_air_index];
-            btWheelInfo& wheel_ground = m_wheelInfo[wheel_ground_index];
-
-            wheel_air.m_raycastInfo = wheel_ground.m_raycastInfo;
-        }
-    }   // for i=0; i<m_wheelInfo.size(); i+=2
-
 
     // Apply suspension forcen (i.e. upwards force)
     // --------------------------------------------
@@ -519,28 +505,6 @@ void btKart::updateVehicle( btScalar step )
     // ------------------------------------
     updateFriction( step);
 
-    for (int i=0;i<m_wheelInfo.size();i++)
-    {
-        btWheelInfo& wheel = m_wheelInfo[i];
-        btVector3 relpos   = wheel.m_raycastInfo.m_hardPointWS
-                           - getRigidBody()->getCenterOfMassPosition();
-        btVector3 vel      = getRigidBody()->getVelocityInLocalPoint(relpos);
-
-        if (wheel.m_raycastInfo.m_isInContact)
-        {
-            const btTransform& chassisWorldTransform =
-                                                 getChassisWorldTransform();
-
-            btVector3 fwd (
-                chassisWorldTransform.getBasis()[0][m_indexForwardAxis],
-                chassisWorldTransform.getBasis()[1][m_indexForwardAxis],
-                chassisWorldTransform.getBasis()[2][m_indexForwardAxis]);
-
-            btScalar proj = fwd.dot(wheel.m_raycastInfo.m_contactNormalWS);
-            fwd -= wheel.m_raycastInfo.m_contactNormalWS * proj;
-        } 
-    }
-
     // If configured, add a force to keep karts on the track
     // -----------------------------------------------------
     float dif = m_kart->getKartProperties()->getStabilityDownwardImpulseFactor();
@@ -668,13 +632,6 @@ void btKart::updateSuspension(btScalar deltaTime)
         btScalar length_diff    = (susp_length - current_length);
         if(m_kart->getKartProperties()->getSuspensionExpSpringResponse())
             length_diff *= fabsf(length_diff)/susp_length;
-        float f = (1.0f + fabsf(length_diff) / susp_length);
-        // Scale the length diff. This results that in uphill sections, when
-        // the suspension is more compressed (i.e. length is bigger), more
-        // force is used, which makes it much less likely for the kart to hit
-        // the terrain, while when driving on flat terrain (length small),
-        // there is hardly any difference
-        length_diff *= f*f;
         force = wheel_info.m_suspensionStiffness * length_diff
               * wheel_info.m_clippedInvContactDotSuspension;
 
@@ -748,7 +705,9 @@ btScalar btKart::calcRollingFriction(btWheelContactPoint& contactPoint)
     btScalar vrel = contactPoint.m_frictionDirectionWorld.dot(vel);
 
     // calculate j that moves us to zero relative velocity
-    btScalar j1 = -vrel * contactPoint.m_jacDiagABInv;
+    // Note that num_wheels_on_ground > 0 since this function is called
+    // for wheels that touch the ground/
+    btScalar j1 = -vrel * contactPoint.m_jacDiagABInv / m_num_wheels_on_ground;
     btSetMin(j1, maxImpulse);
     btSetMax(j1, -maxImpulse);
 
@@ -810,7 +769,7 @@ void btKart::updateFriction(btScalar timeStep)
             (btRigidBody*) wheelInfo.m_raycastInfo.m_groundObject;
         if(!groundObject) continue;
 
-        if(m_zipper_active && m_zipper_velocity > 0)
+        if(m_zipper_speed > 0)
         {
             if (wheel==2 || wheel==3)
             {
@@ -818,7 +777,7 @@ void btKart::updateFriction(btScalar timeStep)
                 // reached. So compute the impulse to accelerate the
                 // kart up to that speed:
                 m_forwardImpulse[wheel] =
-                    0.5f*(m_zipper_velocity -
+                    0.5f*(m_zipper_speed -
                           getRigidBody()->getLinearVelocity().length())
                     / m_chassisBody->getInvMass();
             }
@@ -847,7 +806,7 @@ void btKart::updateFriction(btScalar timeStep)
                 // bullet then tries to offset by applying a backward
                 // impulse - which is a bit too big, causing a impulse
                 // backwards, ... till the kart is shaking backwards and
-                // forwards. By onlyu applying half of the impulse in cae
+                // forwards. By only applying half of the impulse in case
                 // of low friction this goes away.
                 if(wheelInfo.m_brake && fabsf(rollingFriction)<10)
                     rollingFriction*=0.5f;
@@ -882,8 +841,8 @@ void btKart::updateFriction(btScalar timeStep)
     }   // for (int wheel=0; wheel<getNumWheels(); wheel++)
 
 
-    m_zipper_active   = false;
-    m_zipper_velocity = 0;
+    // Note: don't reset zipper speed, or the kart rewinder will
+    // get incorrect zipper information.
 
     // The kart just stopped skidding. Adjust the velocity so that
     // it points in the right direction.
@@ -1024,18 +983,18 @@ void btKart::debugDraw(btIDebugDraw* debugDrawer)
         int n = w.m_raycastInfo.m_triangle_index;
         if (n > -1)
         {
-            const TriangleMesh &tm = World::getWorld()->getTrack()->getTriangleMesh();
-            btVector3 *p1, *p2, *p3;
+            const TriangleMesh &tm = Track::getCurrentTrack()->getTriangleMesh();
+            btVector3 p1, p2, p3;
             tm.getTriangle(n, &p1, &p2, &p3);
-            const btVector3 *n1, *n2, *n3;
+            btVector3 n1, n2, n3;
             tm.getNormals(n, &n1, &n2, &n3);
             // Draw the normals at the vertices
-            debugDrawer->drawLine(*p1, *p1 + *n1, white);
-            debugDrawer->drawLine(*p2, *p2 + *n2, white);
-            debugDrawer->drawLine(*p3, *p3 + *n3, white);
+            debugDrawer->drawLine(p1, p1 + n1, white);
+            debugDrawer->drawLine(p2, p2 + n2, white);
+            debugDrawer->drawLine(p3, p3 + n3, white);
             // Also draw the triangle in white, it can make it easier
             // to identify which triangle a wheel is on
-            debugDrawer->drawTriangle(*p1, *p2, *p3, white, 1.0f);
+            debugDrawer->drawTriangle(p1, p2, p3, white, 1.0f);
         }
 
     }   // for i < getNumWheels
@@ -1061,8 +1020,12 @@ void btKart::setSliding(bool active)
  */
 void btKart::instantSpeedIncreaseTo(float speed)
 {
-    m_zipper_active   = true;
-    m_zipper_velocity = speed;
+    // Avoid that a speed 'increase' might cause a slowdown
+    if (m_chassisBody->getLinearVelocity().length2() > speed*speed)
+    {
+        return;
+    }
+    m_zipper_speed  = speed;
 }   // activateZipper
 
 // ----------------------------------------------------------------------------
