@@ -135,13 +135,31 @@ bool ReplayPlay::addReplayFile(const std::string& fn, bool custom_replay)
         is_end.trim();
         if (is_end == "kart_list_end") break;
         char s1[1024];
+        char display_name_encoded[1024];
 
-        if (sscanf(s,"kart: %s", s1) != 1)
+        int scanned = sscanf(s,"kart: %s %[^\n]", s1, display_name_encoded);
+        if (scanned < 1)
         {
             Log::warn("Replay", "Could not read ghost karts info!");
             break;
         }
+
         rd.m_kart_list.push_back(std::string(s1));
+        if (scanned == 2)
+        {
+            // If username of kart is present, use it
+            rd.m_name_list.push_back(StringUtils::xmlDecode(std::string(display_name_encoded)));
+            if (rd.m_name_list.size() == 1)
+            {
+                // First user is the game master and the "owner" of this replay file
+                rd.m_user_name = rd.m_name_list[0];
+            }
+        } else
+        { // scanned == 1
+            // If username is not present, kart display name will default to kart name
+            // (see GhostController::getName)
+            rd.m_name_list.push_back("");
+        }
     }
 
     int reverse = 0;
@@ -200,7 +218,7 @@ bool ReplayPlay::addReplayFile(const std::string& fn, bool custom_replay)
     assert(m_replay_file_list.size() > 0);
     // Force to use custom replay file immediately
     if (custom_replay)
-        m_current_replay_file = m_replay_file_list.size() - 1;
+        m_current_replay_file = (unsigned int)m_replay_file_list.size() - 1;
 
     return true;
 
@@ -249,11 +267,12 @@ void ReplayPlay::readKartData(FILE *fd, char *next_line)
 {
     char s[1024];
     const unsigned int kart_num = m_ghost_karts.size();
-    m_ghost_karts.push_back(new GhostKart(m_replay_file_list
-        [m_current_replay_file].m_kart_list.at(kart_num),
-        kart_num, kart_num + 1));
+    ReplayData &rd = m_replay_file_list[m_current_replay_file];
+    m_ghost_karts.push_back(new GhostKart(rd.m_kart_list.at(kart_num),
+                                          kart_num, kart_num + 1));
     m_ghost_karts[kart_num].init(RaceManager::KT_GHOST);
-    Controller* controller = new GhostController(getGhostKart(kart_num));
+    Controller* controller = new GhostController(getGhostKart(kart_num),
+                                                 rd.m_name_list[kart_num]);
     getGhostKart(kart_num)->setController(controller);
 
     unsigned int size;
