@@ -31,6 +31,7 @@
 #include "scriptengine/script_utils.hpp"
 #include "scriptengine/scriptstdstring.hpp"
 #include "scriptengine/scriptvec3.hpp"
+#include "scriptengine/scriptarray.hpp"
 #include <string.h>
 #include "states_screens/dialogs/tutorial_message_dialog.hpp"
 #include "tracks/track_object_manager.hpp"
@@ -77,6 +78,8 @@ namespace Scripting
     ScriptEngine::~ScriptEngine()
     {
         // Release the engine
+        m_pending_timeouts.clearAndDeleteAll();
+        m_engine->DiscardModule(MODULE_ID_MAIN_SCRIPT_FILE);
         m_engine->Release();
     }
 
@@ -105,7 +108,7 @@ namespace Scripting
         // Read the entire file
         std::string script;
         script.resize(len);
-        int c = fread(&script[0], len, 1, f);
+        size_t c = fread(&script[0], len, 1, f);
         fclose(f);
         if (c != 1)
         {
@@ -305,9 +308,20 @@ namespace Scripting
             // Find the function for the function we want to execute.
             //      This is how you call a normal function with arguments
             //      asIScriptFunction *func = engine->GetModule(0)->GetFunctionByDecl("void func(arg1Type, arg2Type)");
-            func = m_engine->GetModule(MODULE_ID_MAIN_SCRIPT_FILE)
-                        ->GetFunctionByDecl(function_name.c_str());
-        
+            asIScriptModule* module = m_engine->GetModule(MODULE_ID_MAIN_SCRIPT_FILE);
+
+            if (module == NULL)
+            {
+                if (warn_if_not_found)
+                    Log::warn("Scripting", "Scripting function was not found : %s (module not found)", function_name.c_str());
+                else
+                    Log::debug("Scripting", "Scripting function was not found : %s (module not found)", function_name.c_str());
+                m_functions_cache[function_name] = NULL; // remember that this function is unavailable
+                return;
+            }
+
+            func = module->GetFunctionByDecl(function_name.c_str());
+            
             if (func == NULL)
             {
                 if (warn_if_not_found)
@@ -378,7 +392,7 @@ namespace Scripting
                 Log::error("Scripting", "The script ended with an exception.");
 
                 // Write some information about the script exception
-                asIScriptFunction *func = ctx->GetExceptionFunction();
+                //asIScriptFunction *func = ctx->GetExceptionFunction();
                 //std::cout << "func: " << func->GetDeclaration() << std::endl;
                 //std::cout << "modl: " << func->GetModuleName() << std::endl;
                 //std::cout << "sect: " << func->GetScriptSectionName() << std::endl;
@@ -426,6 +440,7 @@ namespace Scripting
         // Register the script string type
         RegisterStdString(engine); //register std::string
         RegisterVec3(engine);      //register Vec3
+        RegisterScriptArray(engine, true);
 
         Scripting::Track::registerScriptFunctions(m_engine);
         Scripting::Challenges::registerScriptFunctions(m_engine);
@@ -520,7 +535,6 @@ namespace Scripting
     {
         if (m_callback_delegate != NULL)
         {
-            asIScriptEngine* engine = World::getWorld()->getScriptEngine()->getEngine();
             m_callback_delegate->Release();
         }
     }
