@@ -19,6 +19,8 @@
 #include "script_track.hpp"
 
 #include "animations/three_d_animation.hpp"
+#include "font/digit_face.hpp"
+#include "font/font_manager.hpp"
 #include "graphics/central_settings.hpp"
 #include "graphics/stk_text_billboard.hpp"
 #include "guiengine/scalable_font.hpp"
@@ -65,7 +67,8 @@ namespace Scripting
           */
         ::TrackObject* getTrackObject(std::string* libraryInstance, std::string* objID)
         {
-            return World::getWorld()->getTrack()->getTrackObjectManager()->getTrackObject(*libraryInstance, *objID);
+            return ::Track::getCurrentTrack()->getTrackObjectManager()
+                          ->getTrackObject(*libraryInstance, *objID);
         }
 
         /** Creates a trigger at the specified location */
@@ -82,33 +85,32 @@ namespace Scripting
             ::TrackObject* tobj = new ::TrackObject(posi, hpr, scale,
                 "none", newtrigger, false /* isDynamic */, NULL /* physics settings */);
             tobj->setID(*triggerID);
-            World::getWorld()->getTrack()->getTrackObjectManager()->insertObject(tobj);
+            ::Track::getCurrentTrack()->getTrackObjectManager()->insertObject(tobj);
         }
 
         void createTextBillboard(std::string* text, SimpleVec3* location)
         {
             core::stringw wtext = StringUtils::utf8ToWide(*text);
-            core::dimension2d<u32> textsize = GUIEngine::getHighresDigitFont()
-                ->getDimension(wtext.c_str());
-
-            assert(GUIEngine::getHighresDigitFont() != NULL);
+            DigitFace* digit_face = font_manager->getFont<DigitFace>();
+            core::dimension2d<u32> textsize = digit_face->getDimension(wtext.c_str());
 
             core::vector3df xyz(location->getX(), location->getY(), location->getZ());
-
+#ifndef SERVER_ONLY
             if (CVS->isGLSL())
             {
-                gui::ScalableFont* font = GUIEngine::getHighresDigitFont();
-                STKTextBillboard* tb = new STKTextBillboard(wtext.c_str(), font,
+                STKTextBillboard* tb = new STKTextBillboard(wtext.c_str(), digit_face,
                     GUIEngine::getSkin()->getColor("font::bottom"),
                     GUIEngine::getSkin()->getColor("font::top"),
                     irr_driver->getSceneManager()->getRootSceneNode(),
                     irr_driver->getSceneManager(), -1, xyz,
                     core::vector3df(1.5f, 1.5f, 1.5f));
 
-                World::getWorld()->getTrack()->addNode(tb);
+                ::Track::getCurrentTrack()->addNode(tb);
+                tb->drop();
             }
             else
             {
+                assert(GUIEngine::getHighresDigitFont() != NULL);
                 scene::ISceneManager* sm = irr_driver->getSceneManager();
                 scene::ISceneNode* sn =
                     sm->addBillboardTextSceneNode(GUIEngine::getHighresDigitFont(),
@@ -120,7 +122,25 @@ namespace Scripting
                         -1, // id
                         GUIEngine::getSkin()->getColor("font::bottom"),
                         GUIEngine::getSkin()->getColor("font::top"));
-                World::getWorld()->getTrack()->addNode(sn);
+                ::Track::getCurrentTrack()->addNode(sn);
+            }
+#endif
+        }
+
+        /** Function for re-enable a trigger after a specific timeout*/
+        void setTriggerReenableTimeout(std::string* triggerID, std::string* lib_id,
+                                       float reenable_time)
+        {
+            ::TrackObject* tobj = ::Track::getCurrentTrack()->getTrackObjectManager()
+                ->getTrackObject(*lib_id, *triggerID);
+            if (tobj != NULL)
+            {
+                TrackObjectPresentationActionTrigger* topat =
+                    tobj->getPresentation<TrackObjectPresentationActionTrigger>();
+                if (topat != NULL)
+                {
+                    topat->setReenableTimeout(reenable_time);
+                }
             }
         }
 
@@ -150,10 +170,25 @@ namespace Scripting
             return race_manager->getReverseTrack();
         }
 
+        int getMajorRaceMode()
+        {
+            return race_manager->getMajorMode();
+        }
+
+        int getMinorRaceMode()
+        {
+            return race_manager->getMinorMode();
+        }
+
+        bool isDuringDay()
+        {
+            return ::Track::getCurrentTrack()->getIsDuringDay();
+        }
+
         void setFog(float maxDensity, float start, float end, int r, int g, int b, float duration)
         {
             PropertyAnimator* animator = PropertyAnimator::get();
-            ::Track* track = World::getWorld()->getTrack();
+            ::Track* track = ::Track::getCurrentTrack();
             animator->add(
                 new AnimatedProperty(FOG_MAX, 1,
                     new double[1] { track->getFogMax() }, 
@@ -223,22 +258,52 @@ namespace Scripting
 
             /** Sets a loop for a skeletal animation */
             // TODO: can we use a type and avoid void* ?
-            void setLoop(int start, int end /** \cond DOXYGEN_IGNORE */, void *memory /** \endcond */)
+            void setFrameLoop(int start, int end /** \cond DOXYGEN_IGNORE */, void *memory /** \endcond */)
             {
-                ((TrackObjectPresentationMesh*)(memory))->setLoop(start, end);
+                if (memory)
+                {
+                    ((scene::IAnimatedMeshSceneNode*)(memory))->setFrameLoop(start, end);
+                }
+            }
+
+            /** Sets a loop once for a skeletal animation */
+            void setFrameLoopOnce(int start, int end /** \cond DOXYGEN_IGNORE */, void *memory /** \endcond */)
+            {
+                if (memory)
+                {
+                    ((scene::IAnimatedMeshSceneNode*)(memory))->setFrameLoopOnce(start, end);
+                }
+            }
+
+            /** Get current frame in a skeletal animation */
+            int getFrameNr(/** \cond DOXYGEN_IGNORE */void *memory /** \endcond */)
+            {
+                if (memory)
+                {
+                    return (int)((scene::IAnimatedMeshSceneNode*)(memory))->getFrameNr();
+                }
+                return -1;
+            }
+
+            /** Gets the animation set for a skeletal animation */
+            int getAnimationSet(/** \cond DOXYGEN_IGNORE */void *memory /** \endcond */)
+            {
+                if (memory)
+                {
+                    return ((scene::IAnimatedMeshSceneNode*)(memory))->getAnimationSet();
+                }
+                return -1;
             }
 
             /** Sets the current frame for a skeletal animation */
             void setCurrentFrame(int frame /** \cond DOXYGEN_IGNORE */, void *memory /** \endcond */)
             {
-                ((TrackObjectPresentationMesh*)(memory))->setCurrentFrame(frame);
+                if (memory)
+                {
+                    ((scene::IAnimatedMeshSceneNode*)(memory))->setCurrentFrame((float)frame);
+                }
             }
 
-            /** Get current frame in a skeletal animation */
-            int getCurrentFrame(/** \cond DOXYGEN_IGNORE */void *memory /** \endcond */)
-            {
-                return ((TrackObjectPresentationMesh*)(memory))->getCurrentFrame();
-            }
             /** @} */
         }
 
@@ -376,6 +441,8 @@ namespace Scripting
                 asFUNCTION(createTrigger), asCALL_CDECL); assert(r >= 0);
             r = engine->RegisterGlobalFunction("void createTextBillboard(const string &in, const Vec3 &in)",
                 asFUNCTION(createTextBillboard), asCALL_CDECL); assert(r >= 0);
+            r = engine->RegisterGlobalFunction("void setTriggerReenableTimeout(const string &in, const string &in, float reenable_time)",
+                asFUNCTION(setTriggerReenableTimeout), asCALL_CDECL); assert(r >= 0);
             r = engine->RegisterGlobalFunction("TrackObject@ getTrackObject(const string &in, const string &in)", asFUNCTION(getTrackObject), asCALL_CDECL); assert(r >= 0);
             r = engine->RegisterGlobalFunction("void exitRace()", asFUNCTION(exitRace), asCALL_CDECL); assert(r >= 0);
             r = engine->RegisterGlobalFunction("void pauseRace()", asFUNCTION(pauseRace), asCALL_CDECL); assert(r >= 0);
@@ -383,6 +450,9 @@ namespace Scripting
             r = engine->RegisterGlobalFunction("int getNumberOfKarts()", asFUNCTION(getNumberOfKarts), asCALL_CDECL); assert(r >= 0);
             r = engine->RegisterGlobalFunction("int getNumLocalPlayers()", asFUNCTION(getNumLocalPlayers), asCALL_CDECL); assert(r >= 0);
             r = engine->RegisterGlobalFunction("bool isReverse()", asFUNCTION(isTrackReverse), asCALL_CDECL); assert(r >= 0);
+            r = engine->RegisterGlobalFunction("int getMajorRaceMode()", asFUNCTION(getMajorRaceMode), asCALL_CDECL); assert(r >= 0);
+            r = engine->RegisterGlobalFunction("int getMinorRaceMode()", asFUNCTION(getMinorRaceMode), asCALL_CDECL); assert(r >= 0);
+            r = engine->RegisterGlobalFunction("bool isDuringDay()", asFUNCTION(isDuringDay), asCALL_CDECL); assert(r >= 0);
 
             // TrackObject
             r = engine->RegisterObjectMethod("TrackObject", "void setEnabled(bool status)", asMETHOD(::TrackObject, setEnabled), asCALL_THISCALL); assert(r >= 0);
@@ -401,9 +471,11 @@ namespace Scripting
             r = engine->RegisterObjectMethod("PhysicalObject", "void disable()", asMETHOD(PhysicalObject, disable), asCALL_THISCALL); assert(r >= 0);
             r = engine->RegisterObjectMethod("PhysicalObject", "void enable()", asMETHOD(PhysicalObject, enable), asCALL_THISCALL); assert(r >= 0);
 
-            // TrackObjectPresentationMesh (Mesh or Skeletal Animation)
-            r = engine->RegisterObjectMethod("Mesh", "void setLoop(int start, int end)", asFUNCTION(Mesh::setLoop), asCALL_CDECL_OBJLAST); assert(r >= 0);
-            r = engine->RegisterObjectMethod("Mesh", "int getCurrentFrame()", asFUNCTION(Mesh::getCurrentFrame), asCALL_CDECL_OBJLAST); assert(r >= 0);
+            // Animated Mesh
+            r = engine->RegisterObjectMethod("Mesh", "void setFrameLoop(int start, int end)", asFUNCTION(Mesh::setFrameLoop), asCALL_CDECL_OBJLAST); assert(r >= 0);
+            r = engine->RegisterObjectMethod("Mesh", "void setFrameLoopOnce(int start, int end)", asFUNCTION(Mesh::setFrameLoopOnce), asCALL_CDECL_OBJLAST); assert(r >= 0);
+            r = engine->RegisterObjectMethod("Mesh", "int getFrameNr()", asFUNCTION(Mesh::getFrameNr), asCALL_CDECL_OBJLAST); assert(r >= 0);
+            r = engine->RegisterObjectMethod("Mesh", "int getAnimationSet()", asFUNCTION(Mesh::getAnimationSet), asCALL_CDECL_OBJLAST); assert(r >= 0);
             r = engine->RegisterObjectMethod("Mesh", "void setCurrentFrame(int frame)", asFUNCTION(Mesh::setCurrentFrame), asCALL_CDECL_OBJLAST); assert(r >= 0);
             //r = engine->RegisterObjectMethod("Mesh", "void move(Vec3 &in)", asFUNCTION(movePresentation), asCALL_CDECL_OBJLAST); assert(r >= 0);
 
