@@ -1,20 +1,3 @@
-//  SuperTuxKart - a fun racing game with go-kart
-//  Copyright (C) 2014-2015 SuperTuxKart-Team
-//
-//  This program is free software; you can redistribute it and/or
-//  modify it under the terms of the GNU General Public License
-//  as published by the Free Software Foundation; either version 3
-//  of the License, or (at your option) any later version.
-//
-//  This program is distributed in the hope that it will be useful,
-//  but WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//  GNU General Public License for more details.
-//
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
 //
 // Script std::string
 //
@@ -28,13 +11,15 @@
 #ifndef SCRIPTSTDSTRING_H
 #define SCRIPTSTDSTRING_H
 
+// String must be included before angelscript.h to avoid some errors during
+// compilation with GetObject function
+#include <string>
+
 #ifndef ANGELSCRIPT_H 
 // Avoid having to inform include path if header is already include before
-#include "angelscript.h"
+#include <angelscript.h>
 #endif
 
-#include <string>
-
 //---------------------------
 // Compilation settings
 //
