@@ -277,7 +277,7 @@ void ArenasScreen::buildTrackList()
             }
             else
             {
-                w->addItem( curr->getName(), curr->getIdent(), curr->getScreenshotFile(), 0,
+                w->addItem( translations->fribidize(curr->getName()), curr->getIdent(), curr->getScreenshotFile(), 0,
                            IconButtonWidget::ICON_PATH_TYPE_ABSOLUTE );
             }
         }
@@ -329,7 +329,7 @@ void ArenasScreen::buildTrackList()
             }
             else
             {
-                w->addItem( curr->getName(), curr->getIdent(), curr->getScreenshotFile(), 0,
+                w->addItem( translations->fribidize(curr->getName()), curr->getIdent(), curr->getScreenshotFile(), 0,
                            IconButtonWidget::ICON_PATH_TYPE_ABSOLUTE );
             }
         }
@@ -342,7 +342,7 @@ void ArenasScreen::buildTrackList()
     if (m_unsupported_arena.size() > 0)
         w->setText( _P("%d arena unavailable in single player.",
                        "%d arenas unavailable in single player.",
-                       m_unsupported_arena.size()) );
+                       (int)m_unsupported_arena.size()) );
 }
 
 // ------------------------------------------------------------------------------------------------------
