@@ -56,7 +56,7 @@ class CreditsScreen : public GUIEngine::Screen,
 
     friend class GUIEngine::ScreenSingleton<CreditsScreen>;
     CreditsScreen();
-    bool getWideLine(std::ifstream& file, core::stringw* out);
+    bool getLineAsWide(std::ifstream& file, core::stringw* out);
 
     bool m_is_victory_music;
 
