@@ -29,7 +29,6 @@
 #include "input/input_manager.hpp"
 #include "io/file_manager.hpp"
 #include "modes/world.hpp"
-#include "network/network_manager.hpp"
 #include "race/grand_prix_manager.hpp"
 #include "race/race_manager.hpp"
 #include "tracks/track_manager.hpp"
@@ -64,8 +63,11 @@ core::stringw getLabel(RaceManager::Difficulty difficulty, const ChallengeData*
         label.append( _("Required Nitro Points: %i", c->getEnergy(difficulty)) );
     }
 
-    if (label.size() > 0) label.append(L"\n");
-    label.append(_("Number of AI Karts: %i", c->getNumKarts(difficulty) - 1));
+    if (!c->isGhostReplay())
+    {
+        if (label.size() > 0) label.append(L"\n");
+        label.append(_("Number of AI Karts: %i", c->getNumKarts(difficulty) - 1));
+    }
 
     return label;
 }
@@ -145,6 +147,8 @@ SelectChallengeDialog::SelectChallengeDialog(const float percentWidth,
         typeLbl->setText(_("Grand Prix"), false );
     else if (c->getData()->getEnergy(RaceManager::DIFFICULTY_EASY) > 0)
         typeLbl->setText(_("Nitro challenge"), false );
+    else if (c->getData()->isGhostReplay())
+        typeLbl->setText(_("Ghost replay race"), false );
     else
         typeLbl->setText( RaceManager::getNameOf(c->getData()->getMinorMode()), false );
 
@@ -191,8 +195,8 @@ GUIEngine::EventPropagation SelectChallengeDialog::processEvent(const std::strin
         assert(device != NULL);
 
         // Set up race manager appropriately
-        race_manager->setNumLocalPlayers(1);
-        race_manager->setLocalKartInfo(0, UserConfigParams::m_default_kart);
+        race_manager->setNumPlayers(1);
+        race_manager->setPlayerKart(0, UserConfigParams::m_default_kart);
         race_manager->setReverseTrack(false);
 
         //int id = StateManager::get()->createActivePlayer( unlock_manager->getCurrentPlayer(), device );
