@@ -177,8 +177,6 @@ void EditTrackScreen::init()
 // -----------------------------------------------------------------------------
 void EditTrackScreen::loadTrackList()
 {
-    bool belongs_to_group;
-
     DynamicRibbonWidget* tracks_widget = getWidget<DynamicRibbonWidget>("tracks");
     assert(tracks_widget != NULL);
 
@@ -187,7 +185,7 @@ void EditTrackScreen::loadTrackList()
     for (unsigned int i = 0; i < track_manager->getNumberOfTracks(); i++)
     {
         Track* t = track_manager->getTrack(i);
-        belongs_to_group = (m_track_group.empty()                ||
+        bool belongs_to_group = (m_track_group.empty()                ||
                           m_track_group == ALL_TRACKS_GROUP_ID ||
                           t->isInGroup(m_track_group)                );
         if (!t->isArena()    && !t->isSoccer() &&
