@@ -25,9 +25,11 @@
 #include "challenges/unlock_manager.hpp"
 #include "config/player_manager.hpp"
 #include "graphics/material_manager.hpp"
+#include "graphics/render_info.hpp"
 #include "guiengine/engine.hpp"
 #include "guiengine/scalable_font.hpp"
 #include "io/file_manager.hpp"
+#include "karts/kart_model.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/kart_properties_manager.hpp"
 #include "modes/cutscene_world.hpp"
@@ -215,13 +217,13 @@ void FeatureUnlockedCutScene::addTrophy(RaceManager::Difficulty difficulty)
     switch (difficulty)
     {
         case RaceManager::DIFFICULTY_EASY:
-            model = file_manager->getAsset(FileManager::MODEL,"trophy_bronze.b3d");
+            model = file_manager->getAsset(FileManager::MODEL,"trophy_bronze.spm");
             break;
         case RaceManager::DIFFICULTY_MEDIUM:
-            model = file_manager->getAsset(FileManager::MODEL,"trophy_silver.b3d");
+            model = file_manager->getAsset(FileManager::MODEL,"trophy_silver.spm");
             break;
         case RaceManager::DIFFICULTY_HARD:
-            model = file_manager->getAsset(FileManager::MODEL,"trophy_gold.b3d");
+            model = file_manager->getAsset(FileManager::MODEL,"trophy_gold.spm");
             break;
         default:
             assert(false);
@@ -295,7 +297,7 @@ void FeatureUnlockedCutScene::init()
         else if (m_unlocked_stuff[n].m_unlocked_kart != NULL)
         {
             KartModel *kart_model =
-                m_unlocked_stuff[n].m_unlocked_kart->getKartModelCopy();
+                m_unlocked_stuff[n].m_unlocked_kart->getKartModelCopy(KRT_DEFAULT);
             m_all_kart_models.push_back(kart_model);
             m_unlocked_stuff[n].m_root_gift_node = kart_model->attachModel(true, false);
             m_unlocked_stuff[n].m_scale = 5.0f;
@@ -322,7 +324,7 @@ void FeatureUnlockedCutScene::init()
         {
             video::SMaterial m;
             //m.MaterialType    = video::EMT_TRANSPARENT_ALPHA_CHANNEL;
-            m.BackfaceCulling = false;
+            m.BackfaceCulling = true;
             m.setTexture(0, m_unlocked_stuff[n].m_pictures[0]);
             m.AmbientColor  = video::SColor(255, 255, 255, 255);
             m.DiffuseColor  = video::SColor(255, 255, 255, 255);
@@ -340,7 +342,7 @@ void FeatureUnlockedCutScene::init()
             m_unlocked_stuff[n].m_root_gift_node = irr_driver->getSceneManager()->addEmptySceneNode();
             irr_driver->setAllMaterialFlags(mesh);
             m_unlocked_stuff[n].m_side_1 = irr_driver->addMesh(mesh, "unlocked_picture", m_unlocked_stuff[n].m_root_gift_node);
-            //mesh->drop();
+            mesh->drop();
 
             mesh = irr_driver->createTexturedQuadMesh(&m,
                 m_unlocked_stuff[n].m_w,
@@ -348,7 +350,7 @@ void FeatureUnlockedCutScene::init()
             irr_driver->setAllMaterialFlags(mesh);
             m_unlocked_stuff[n].m_side_2 = irr_driver->addMesh(mesh, "unlocked_picture",  m_unlocked_stuff[n].m_root_gift_node);
             m_unlocked_stuff[n].m_side_2->setRotation(core::vector3df(0.0f, 180.0f, 0.0f));
-            //mesh->drop();
+            mesh->drop();
 #ifdef DEBUG
             m_unlocked_stuff[n].m_root_gift_node->setName("unlocked track picture");
 #endif
