@@ -32,6 +32,7 @@
 #include "input/device_manager.hpp"
 #include "items/item_manager.hpp"
 #include "karts/abstract_characteristic.hpp"
+#include "karts/kart_model.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/kart_properties_manager.hpp"
 #include "modes/overworld.hpp"
@@ -148,8 +149,6 @@ EventPropagation FocusDispatcher::focused(const int player_id)
     return GUIEngine::EVENT_LET;
 }   // focused
 
-static FocusDispatcher  *g_dispatcher = NULL;
-
 #if 0
 #pragma mark -
 #pragma mark KartHoverListener
@@ -229,6 +228,7 @@ bool sameKart(const PlayerKartWidget& player1, const PlayerKartWidget& player2)
 
 KartSelectionScreen::KartSelectionScreen(const char* filename) : Screen(filename)
 {
+    m_dispatcher           = NULL;
     m_removed_widget       = NULL;
     m_multiplayer_message  = NULL;
     m_from_overworld       = false;
@@ -246,8 +246,8 @@ KartSelectionScreen* KartSelectionScreen::getRunningInstance()
 
 void KartSelectionScreen::loadedFromFile()
 {
-    g_dispatcher          = new FocusDispatcher(this);
-    m_first_widget        = g_dispatcher;
+    m_dispatcher          = new FocusDispatcher(this);
+    m_first_widget        = m_dispatcher;
     m_game_master_confirmed    = false;
     m_multiplayer_message = NULL;
     // Dynamically add tabs
@@ -305,6 +305,7 @@ void KartSelectionScreen::beforeAddingWidget()
 
 void KartSelectionScreen::init()
 {
+    m_instance_ptr = this;
     Screen::init();
     m_must_delete_on_back = false;
 
@@ -318,17 +319,17 @@ void KartSelectionScreen::init()
 
     // FIXME : The reserved id value is -1 when we switch from KSS to NKSS and vice-versa
 
-    g_dispatcher->setRootID(placeholder->m_reserved_id);
+    m_dispatcher->setRootID(placeholder->m_reserved_id);
 
     g_root_id = placeholder->m_reserved_id;
-    if (!m_widgets.contains(g_dispatcher))
+    if (!m_widgets.contains(m_dispatcher))
     {
-        m_widgets.push_back(g_dispatcher);
+        m_widgets.push_back(m_dispatcher);
 
         // this is only needed if the dispatcher wasn't already in
         // the list of widgets. If it already was, it was added along
         // other widgets.
-        g_dispatcher->add();
+        m_dispatcher->add();
     }
 
     m_game_master_confirmed = false;
@@ -426,7 +427,7 @@ void KartSelectionScreen::tearDown()
 void KartSelectionScreen::unloaded()
 {
     // these pointer is no more valid (have been deleted along other widgets)
-    g_dispatcher = NULL;
+    m_dispatcher = NULL;
 }
 
 // ----------------------------------------------------------------------------
@@ -439,7 +440,7 @@ bool KartSelectionScreen::joinPlayer(InputDevice* device)
         Log::info("KartSelectionScreen",  "joinPlayer() invoked");
     if (!m_multiplayer && !first_player) return false;
 
-    assert (g_dispatcher != NULL);
+    assert (m_dispatcher != NULL);
 
     DynamicRibbonWidget* w = getWidget<DynamicRibbonWidget>("karts");
     if (w == NULL)
@@ -809,15 +810,10 @@ void KartSelectionScreen::updateKartStats(uint8_t widget_id,
 
     const KartProperties *kp =
                     kart_properties_manager->getKart(selection);
+
     if (kp != NULL)
     {
-        // Scale the values so they look better
-        w->setValue(KartStatsWidget::SKILL_MASS, (int)
-            ((kp->getCombinedCharacteristic()->getMass() - 20) / 4));
-        w->setValue(KartStatsWidget::SKILL_SPEED, (int)
-            ((kp->getCombinedCharacteristic()->getEngineMaxSpeed() - 15) * 6));
-        w->setValue(KartStatsWidget::SKILL_POWER, (int)
-            ((kp->getAvgPower() - 30) / 20));
+        w->setValues(kp);
         w->update(0);
     }
 }
@@ -838,8 +834,10 @@ void KartSelectionScreen::updateKartWidgetModel(int widget_id,
             ItemManager::getItemModel(Item::ITEM_BONUS_BOX);
 
         w3->clearModels();
-        w3->addModel( model, Vec3(0.0f, -12.0f, 0.0f),
-                      Vec3(35.0f, 35.0f, 35.0f) );
+        core::matrix4 model_location;
+        model_location.setTranslation(core::vector3df(0.0f, -12.0f, 0.0f));
+        model_location.setScale(core::vector3df(35.0f, 35.0f, 35.0f));
+        w3->addModel(model, model_location);
         w3->update(0);
         m_kart_widgets[widget_id].m_kart_name
         ->setText( _("Random Kart"), false );
@@ -848,9 +846,11 @@ void KartSelectionScreen::updateKartWidgetModel(int widget_id,
     else if (StringUtils::startsWith(selection, ID_LOCKED) && !m_multiplayer)
     {
         w3->clearModels();
+        core::matrix4 model_location;
+        model_location.setScale(core::vector3df(15.0f, 15.0f, 15.0f));
         w3->addModel(irr_driver->getAnimatedMesh(
-                         file_manager->getAsset(FileManager::MODEL, "chest.b3d") )->getMesh(20),
-                     Vec3(0,0,0), Vec3(15.0f, 15.0f, 15.0f) );
+            file_manager->getAsset(FileManager::MODEL, "chest.spm"))
+            ->getMesh(20), model_location);
         w3->update(0);
 
         if (m_multiplayer)
@@ -879,22 +879,54 @@ void KartSelectionScreen::updateKartWidgetModel(int widget_id,
                 scale = 30.0f;
             }
 
+            core::matrix4 model_location;
+            model_location.setScale(core::vector3df(scale, scale, scale));
             w3->clearModels();
-            w3->addModel( kart_model.getModel(), Vec3(0,0,0),
-                Vec3(scale, scale, scale),
-                          kart_model.getBaseFrame() );
-            w3->addModel( kart_model.getWheelModel(0),
-                          kart_model.getWheelGraphicsPosition(0) );
-            w3->addModel( kart_model.getWheelModel(1),
-                          kart_model.getWheelGraphicsPosition(1) );
-            w3->addModel( kart_model.getWheelModel(2),
-                          kart_model.getWheelGraphicsPosition(2) );
-            w3->addModel( kart_model.getWheelModel(3),
-                          kart_model.getWheelGraphicsPosition(3) );
-            for (size_t i = 0; i < kart_model.getSpeedWeightedObjectsCount(); i++)
+            const bool has_win_anime =
+                UserConfigParams::m_show_steering_animations != 0 &&
+                (((kart_model.getFrame(KartModel::AF_WIN_LOOP_START) > -1 ||
+                kart_model.getFrame(KartModel::AF_WIN_START) > -1) &&
+                kart_model.getFrame(KartModel::AF_WIN_END) > -1) ||
+                (kart_model.getFrame(KartModel::AF_SELECTION_START) > -1 &&
+                kart_model.getFrame(KartModel::AF_SELECTION_END) > -1));
+            w3->addModel( kart_model.getModel(), model_location,
+                has_win_anime ?
+                kart_model.getFrame(KartModel::AF_SELECTION_START) > -1 ?
+                kart_model.getFrame(KartModel::AF_SELECTION_START) :
+                kart_model.getFrame(KartModel::AF_WIN_LOOP_START) > -1 ?
+                kart_model.getFrame(KartModel::AF_WIN_LOOP_START) :
+                kart_model.getFrame(KartModel::AF_WIN_START) :
+                kart_model.getBaseFrame(),
+                has_win_anime ?
+                kart_model.getFrame(KartModel::AF_SELECTION_END) > -1 ?
+                kart_model.getFrame(KartModel::AF_SELECTION_END) :
+                kart_model.getFrame(KartModel::AF_WIN_END) :
+                kart_model.getBaseFrame(),
+                false/*all_parts_colorized*/,
+                kart_model.getAnimationSpeed());
+
+            model_location.setScale(core::vector3df(1.0f, 1.0f, 1.0f));
+            for (unsigned i = 0; i < 4; i++)
             {
-                const SpeedWeightedObject&  obj = kart_model.getSpeedWeightedObject((int)i);
-                w3->addModel(obj.m_model, obj.m_position);
+                model_location.setTranslation(kart_model
+                    .getWheelGraphicsPosition(i).toIrrVector());
+                w3->addModel(kart_model.getWheelModel(i), model_location);
+            }
+
+            for (unsigned i = 0;
+                 i < kart_model.getSpeedWeightedObjectsCount(); i++)
+            {
+                const SpeedWeightedObject& obj =
+                    kart_model.getSpeedWeightedObject(i);
+                core::matrix4 swol = obj.m_location;
+                if (!obj.m_bone_name.empty())
+                {
+                    core::matrix4 inv =
+                        kart_model.getInverseBoneMatrix(obj.m_bone_name);
+                    swol = inv * obj.m_location;
+                }
+                w3->addModel(obj.m_model, swol, -1, -1, false, 0.0f,
+                    obj.m_bone_name);
             }
             //w3->update(0);
 
@@ -1158,13 +1190,12 @@ void KartSelectionScreen::allPlayersDone()
         if (selected_kart == RANDOM_KART_ID)
         {
             // don't select an already selected kart
-            int random_id;
             // to prevent infinite loop in case they are all locked
             int count = 0;
             bool done = false;
             do
             {
-                random_id = random.get(item_count);
+                int random_id = random.get(item_count);
                 // valid kart if it can bt used, and is either not locked,
                 // or it's a multiplayer race.
                 if (items[random_id].m_code_name != ID_DONT_USE &&
