@@ -37,6 +37,7 @@
 #include "modes/cutscene_world.hpp"
 #include "modes/overworld.hpp"
 #include "modes/demo_world.hpp"
+#include "network/network_config.hpp"
 #include "online/request_manager.hpp"
 #include "states_screens/addons_screen.hpp"
 #include "states_screens/credits.hpp"
@@ -75,6 +76,7 @@ bool MainMenuScreen::m_enable_online = false;
 MainMenuScreen::MainMenuScreen() : Screen("main_menu.stkgui")
 {
     m_online_string = _("Online");
+    //I18N: Used as a verb, appears on the main menu (login button)
     m_login_string = _("Login");
 }   // MainMenuScreen
 
@@ -84,6 +86,16 @@ void MainMenuScreen::loadedFromFile()
 {
     LabelWidget* w = getWidget<LabelWidget>("info_addons");
     w->setScrollSpeed(15);
+    
+    RibbonWidget* rw_top = getWidget<RibbonWidget>("menu_toprow");
+    assert(rw_top != NULL);
+    
+    if (track_manager->getTrack("overworld") == NULL ||
+        track_manager->getTrack("introcutscene") == NULL ||
+        track_manager->getTrack("introcutscene2") == NULL)
+    {
+        rw_top->removeChildNamed("story");
+    }
 
 #if DEBUG_MENU_ITEM != 1
     RibbonWidget* rw = getWidget<RibbonWidget>("menu_bottomrow");
@@ -96,6 +108,13 @@ void MainMenuScreen::loadedFromFile()
 #endif
 }   // loadedFromFile
 
+// ----------------------------------------------------------------------------
+
+void MainMenuScreen::beforeAddingWidget()
+{
+
+}
+
 // ----------------------------------------------------------------------------
 //
 void MainMenuScreen::init()
@@ -274,7 +293,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
         race_manager->startSingleRace("gpwin", 999, false);
         GrandPrixWin* scene = GrandPrixWin::getInstance();
         scene->push();
-        const std::string winners[] = { "elephpant", "nolok", "pidgin" };
+        const std::string winners[] = { "kiki", "nolok", "pidgin" };
         scene->setKarts(winners);
     }
     else if (selection == "test_gplose")
@@ -289,7 +308,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
         scene->push();
         std::vector<std::string> losers;
         losers.push_back("nolok");
-        losers.push_back("elephpant");
+        losers.push_back("kiki");
         //losers.push_back("wilber");
         //losers.push_back("tux");
         scene->setKarts(losers);
@@ -336,7 +355,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
                 track_manager->getTrack("lighthouse")
                              ->getScreenshotFile().c_str()));
             textures.push_back(irr_driver->getTexture(
-                track_manager->getTrack("startrack")
+                track_manager->getTrack("snowtuxpeak")
                              ->getScreenshotFile().c_str()));
             textures.push_back(irr_driver->getTexture(
                 track_manager->getTrack("sandtrack")
@@ -385,7 +404,8 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
 #endif
     if (selection == "new")
     {
-        KartSelectionScreen* s = OfflineKartSelectionScreen::getInstance(); //FIXME : that was for tests
+        NetworkConfig::get()->unsetNetworking();
+        KartSelectionScreen* s = OfflineKartSelectionScreen::getInstance();
         s->setMultiplayer(false);
         s->setFromOverworld(false);
         s->push();
@@ -393,6 +413,7 @@ void MainMenuScreen::eventCallback(Widget* widget, const std::string& name,
     else if (selection == "multiplayer")
     {
         KartSelectionScreen* s = OfflineKartSelectionScreen::getInstance();
+        NetworkConfig::get()->unsetNetworking();
         s->setMultiplayer(true);
         s->setFromOverworld(false);
         s->push();
