@@ -28,7 +28,6 @@
 #include "graphics/irr_driver.hpp"
 #include "graphics/material.hpp"
 #include "graphics/material_manager.hpp"
-#include "graphics/post_processing.hpp"
 #include "graphics/referee.hpp"
 #include "guiengine/scalable_font.hpp"
 #include "io/file_manager.hpp"
@@ -41,8 +40,9 @@
 #include "karts/kart_properties.hpp"
 #include "karts/kart_properties_manager.hpp"
 #include "karts/rescue_animation.hpp"
-#include "modes/follow_the_leader.hpp"
+#include "modes/linear_world.hpp"
 #include "modes/world.hpp"
+#include "states_screens/race_gui_multitouch.hpp"
 #include "tracks/track.hpp"
 #include "utils/constants.hpp"
 
@@ -69,28 +69,27 @@ RaceGUIBase::RaceGUIBase()
     m_string_go             = _("Go!");
     //I18N: Shown when a goal is scored
     m_string_goal           = _("GOAL!");
-    // Make the two materials permanent (in case that they are not listed
-    // in the textures/materials.xml file).
-    m_music_icon            = material_manager->getMaterial("notes.png",
-                                                            /*full path*/false,
-                                                            /*permanent*/true);
-    if(!m_music_icon->getTexture())
-        Log::fatal("RaceGuiBase", "Can't find 'notes.png' texture, aborting.");
 
-    m_plunger_face          = material_manager->getMaterial("plungerface.png",
-                                                            /*full path*/false,
-                                                            /*permanent*/true);
-    if(!m_plunger_face->getTexture())
+    m_music_icon = irr_driver->getTexture("notes.png");
+    if (!m_music_icon)
+    {
+        Log::fatal("RaceGuiBase", "Can't find 'notes.png' texture, aborting.");
+    }
+
+    m_plunger_face = irr_driver->getTexture("plungerface.png");
+    if (!m_plunger_face)
+    {
         Log::fatal("RaceGuiBase",
                    "Can't find 'plungerface.png' texture, aborting.");
+    }
 
     //read frame picture for icons in the mini map.
-    m_icons_frame           = material_manager->getMaterial("icons-frame.png",
-                                                            /*full_path*/false,
-                                                            /*permanent*/true);
-    if(!m_icons_frame->getTexture())
+    m_icons_frame = irr_driver->getTexture("icons-frame.png");
+    if (!m_icons_frame)
+    {
         Log::fatal("RaceGuiBase",
                    "Can't find 'icons-frame.png' texture, aborting.");
+    }
 
     m_gauge_full            = irr_driver->getTexture(file_manager->getAsset(FileManager::GUI,"gauge_full.png"));
     m_gauge_full_bright     = irr_driver->getTexture(file_manager->getAsset(FileManager::GUI,"gauge_full_bright.png"));
@@ -100,6 +99,7 @@ RaceGUIBase::RaceGUIBase()
     m_icons_inertia         = 2;
 
     m_referee               = NULL;
+    m_multitouch_gui        = NULL;
 }   // RaceGUIBase
 
 // ----------------------------------------------------------------------------
@@ -137,18 +137,27 @@ void RaceGUIBase::reset()
     {
         const AbstractKart *kart = World::getWorld()->getKart(i);
         m_referee_pos[i] = kart->getTrans()(Referee::getStartOffset());
-        m_referee_rotation[i] = Referee::getStartRotation()
-                              + Vec3(0, kart->getHeading()*RAD_TO_DEGREE, 0);
+        Vec3 hpr;
+        btQuaternion q = btQuaternion(kart->getTrans().getBasis().getColumn(1),
+            Referee::getStartRotation().getY() * DEGREE_TO_RAD) *
+            kart->getTrans().getRotation();
+        hpr.setHPR(q);
+        m_referee_rotation[i] = hpr.toIrrHPR();
     }
 
-
     m_referee_height = 10.0f;
     m_referee->attachToSceneNode();
+    m_referee->selectReadySetGo(0); // set red color
     m_plunger_move_time = 0;
     m_plunger_offset    = core::vector2di(0,0);
     m_plunger_speed     = core::vector2df(0,0);
     m_plunger_state     = PLUNGER_STATE_INIT;
     clearAllMessages();
+    
+    if (m_multitouch_gui != NULL)
+    {
+        m_multitouch_gui->reset();
+    }
 }   // reset
 
 //-----------------------------------------------------------------------------
@@ -303,6 +312,7 @@ void RaceGUIBase::drawPowerupIcons(const AbstractKart* kart,
                                    const core::recti &viewport,
                                    const core::vector2df &scaling)
 {
+#ifndef SERVER_ONLY
     // If player doesn't have any powerups or has completed race, do nothing.
     const Powerup* powerup = kart->getPowerup();
     if (powerup->getType() == PowerupManager::POWERUP_NOTHING
@@ -352,6 +362,7 @@ void RaceGUIBase::drawPowerupIcons(const AbstractKart* kart,
             pos, video::SColor(255, 255, 255, 255));
         font->setScale(1.0f);
     }
+#endif
 }   // drawPowerupIcons
 
 // ----------------------------------------------------------------------------
@@ -419,8 +430,8 @@ void RaceGUIBase::preRenderCallback(const Camera *camera)
     if(m_referee && camera->getKart())
     {
         unsigned int world_id = camera->getKart()->getWorldKartId();
-        Vec3 xyz = m_referee_pos[world_id];
-        xyz.setY(xyz.getY()+m_referee_height);
+        Vec3 xyz = m_referee_pos[world_id] +
+            camera->getKart()->getNormal() * m_referee_height;
         m_referee->setPosition(xyz);
         m_referee->setRotation(m_referee_rotation[world_id]);
     }
@@ -429,7 +440,15 @@ void RaceGUIBase::preRenderCallback(const Camera *camera)
 // ----------------------------------------------------------------------------
 void RaceGUIBase::renderPlayerView(const Camera *camera, float dt)
 {
-
+    const core::recti &viewport = camera->getViewport();
+    const core::vector2df scaling = camera->getScaling();
+    const AbstractKart* kart = camera->getKart();
+    if(!kart) return;
+    
+    if (m_multitouch_gui != NULL)
+    {
+        m_multitouch_gui->drawMultitouchSteering(kart, viewport, scaling);
+    }
 }   // renderPlayerView
 
 
@@ -452,20 +471,13 @@ void RaceGUIBase::addMessage(const core::stringw &msg,
  */
 void RaceGUIBase::drawGlobalMusicDescription()
 {
+#ifndef SERVER_ONLY
      // show no music description when it's off
     if (!UserConfigParams::m_music) return;
 
     gui::IGUIFont*       font = GUIEngine::getFont();
 
-    float race_time = World::getWorld()->getTime();
-    // In the modes that the clock counts backwards, convert the
-    // countdown time to time since start:
-    if(race_manager->getMinorMode()==RaceManager::MINOR_MODE_FOLLOW_LEADER)
-        race_time = ((FollowTheLeaderRace*)World::getWorld())->getClockStartTime()
-                  - race_time;
-    else if (race_manager->getMinorMode()==RaceManager::MINOR_MODE_SOCCER &&
-             race_manager->hasTimeTarget())
-        race_time = race_manager->getTimeTarget() - World::getWorld()->getTime();
+    float race_time = World::getWorld()->getTimeSinceStart();
 
     // ---- Manage pulsing effect
     // 3.0 is the duration of ready/set (TODO: don't hardcode)
@@ -546,16 +558,15 @@ void RaceGUIBase::drawGlobalMusicDescription()
     int iconSizeX = (int)(ICON_SIZE*resize + x_pulse*resize*resize);
     int iconSizeY = (int)(ICON_SIZE*resize + y_pulse*resize*resize);
 
-    video::ITexture *t = m_music_icon->getTexture();
     core::rect<s32> dest(noteX-iconSizeX/2+20,
                          noteY-iconSizeY/2+ICON_SIZE/2,
                          noteX+iconSizeX/2+20,
                          noteY+iconSizeY/2+ICON_SIZE/2);
     const core::rect<s32> source(core::position2d<s32>(0,0),
-                                 t->getSize());
+                                 m_music_icon->getSize());
 
-    draw2DImage(t, dest, source,
-                                              NULL, NULL, true);
+    draw2DImage(m_music_icon, dest, source, NULL, NULL, true);
+#endif
 }   // drawGlobalMusicDescription
 
 //-----------------------------------------------------------------------------
@@ -627,6 +638,7 @@ void RaceGUIBase::drawGlobalReadySetGo()
  */
 void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
 {
+#ifndef SERVER_ONLY
     // For now, don't draw player icons when in soccer mode
     const RaceManager::MinorRaceModeType  minor_mode = race_manager->getMinorMode();
     if(minor_mode == RaceManager::MINOR_MODE_SOCCER)
@@ -643,8 +655,11 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
         y_space = irr_driver->getActualScreenSize().Height - y_base;
     }
 
+    unsigned int sta = race_manager->getNumSpareTireKarts();
+    const unsigned int num_karts = race_manager->getNumberOfKarts() - sta;
+
     // -2 because that's the spacing further on
-    int ICON_PLAYER_WIDTH = y_space / race_manager->getNumberOfKarts() - 2;
+    int ICON_PLAYER_WIDTH = y_space / num_karts - 2;
 
     int icon_width_max = (int)(50*(irr_driver->getActualScreenSize().Width/800.0f));
     int icon_width_min = (int)(35*(irr_driver->getActualScreenSize().Height/600.0f));
@@ -669,10 +684,11 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
     int ICON_WIDTH = ICON_PLAYER_WIDTH * 4 / 5;
 
     WorldWithRank *world    = (WorldWithRank*)(World::getWorld());
+
     //initialize m_previous_icons_position
     if(m_previous_icons_position.size()==0)
     {
-        for(unsigned int i=0; i<race_manager->getNumberOfKarts(); i++)
+        for(unsigned int i=0; i<num_karts; i++)
         {
             const AbstractKart *kart = world->getKart(i);
             int position = kart->getPosition();
@@ -691,12 +707,19 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
     int previous_y=y_base-ICON_PLAYER_WIDTH-2;
 
     gui::ScalableFont* font = GUIEngine::getFont();
-    const unsigned int kart_amount = world->getNumKarts();
+    const unsigned int kart_amount = world->getNumKarts() - sta;
 
     //where is the limit to hide last icons
-    int y_icons_limit=irr_driver->getActualScreenSize().Height-bottom_margin-ICON_PLAYER_WIDTH;
+    int y_icons_limit = irr_driver->getActualScreenSize().Height - 
+                                            bottom_margin - ICON_PLAYER_WIDTH;
     if (race_manager->getNumLocalPlayers() == 3)
-        y_icons_limit=irr_driver->getActualScreenSize().Height-ICON_WIDTH;
+    {
+        y_icons_limit = irr_driver->getActualScreenSize().Height - ICON_WIDTH;
+    }
+    else if (m_multitouch_gui != NULL)
+    {
+        y_icons_limit = irr_driver->getActualScreenSize().Height / 2;
+    }
 
     world->getKartsDisplayInfo(&m_kart_display_infos);
 
@@ -730,7 +753,7 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
             LinearWorld *linear_world = (LinearWorld*)(World::getWorld());
 
             float distance = linear_world->getDistanceDownTrackForKart(kart_id)
-                           + linear_world->getTrack()->getTrackLength()*lap;
+                           + Track::getCurrentTrack()->getTrackLength()*lap;
             if ((position>1) &&
                 (previous_distance-distance<m_dist_show_overlap) &&
                 (!kart->hasFinishedRace())                          )
@@ -820,9 +843,8 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
                                    100+(int)(100*cos(M_PI/2*i+World::getWorld()->getTime()*2)));
             }
             const core::rect<s32> rect(core::position2d<s32>(0,0),
-                                       m_icons_frame->getTexture()->getSize());
-            draw2DImage(
-                                                      m_icons_frame->getTexture(), pos, rect,NULL, colors, true);
+                                       m_icons_frame->getSize());
+            draw2DImage(m_icons_frame, pos, rect,NULL, colors, true);
         }
 
         // Fixes crash bug, why are certain icons not showing up?
@@ -929,6 +951,7 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
         }
 
     } //next position
+#endif
 }   // drawGlobalPlayerIcons
 
 // ----------------------------------------------------------------------------
@@ -938,6 +961,7 @@ void RaceGUIBase::drawGlobalPlayerIcons(int bottom_margin)
  */
 void RaceGUIBase::drawPlungerInFace(const Camera *camera, float dt)
 {
+#ifndef SERVER_ONLY
     const AbstractKart *kart = camera->getKart();
     if (kart->getBlockedByPlungerTime()<=0)
     {
@@ -1012,16 +1036,16 @@ void RaceGUIBase::drawPlungerInFace(const Camera *camera, float dt)
     int plunger_x = viewport.UpperLeftCorner.X + screen_width/2
                   - plunger_size/2;
 
-    video::ITexture *t=m_plunger_face->getTexture();
     plunger_x += (int)m_plunger_offset.X;
     core::rect<s32> dest(plunger_x,              offset_y,
                          plunger_x+plunger_size, offset_y+plunger_size);
 
     const core::rect<s32> source(core::position2d<s32>(0,0),
-                                 t->getSize());
+                                 m_plunger_face->getSize());
 
-    draw2DImage(t, dest, source,
+    draw2DImage(m_plunger_face, dest, source,
                                               &viewport /* clip */,
                                               NULL /* color */,
                                               true /* alpha */     );
+#endif   // !SERVER_ONLY
 }   // drawPlungerInFace
