@@ -45,7 +45,7 @@
 #include "modes/soccer_world.hpp"
 #include "modes/world_with_rank.hpp"
 #include "network/protocol_manager.hpp"
-#include "network/protocols/client_lobby_room_protocol.hpp"
+#include "network/protocols/client_lobby.hpp"
 #include "race/highscores.hpp"
 #include "scriptengine/property_animator.hpp"
 #include "states_screens/feature_unlocked.hpp"
@@ -135,7 +135,7 @@ void RaceResultGUI::init()
     else
     {
         m_start_track = 0;
-        m_end_track = tracks.size();
+        m_end_track = (int)tracks.size();
     }
 }   // init
 
@@ -143,7 +143,7 @@ void RaceResultGUI::init()
 void RaceResultGUI::tearDown()
 {
     Screen::tearDown();
-    m_font->setMonospaceDigits(m_was_monospace);
+    //m_font->setMonospaceDigits(m_was_monospace);
 
     if (m_finish_sound != NULL &&
         m_finish_sound->getStatus() == SFXBase::SFX_PLAYING)
@@ -217,7 +217,7 @@ void RaceResultGUI::enableAllButtons()
 
         middle->setText(_("Restart"));
         middle->setVisible(true);
-
+        middle->setFocusForPlayer(PLAYER_ID_GAME_MASTER);
         if (race_manager->raceWasStartedFromOverworld())
         {
             top->setVisible(false);
@@ -230,8 +230,6 @@ void RaceResultGUI::enableAllButtons()
             bottom->setText(_("Back to the menu"));
         }
         bottom->setVisible(true);
-
-        bottom->setFocusForPlayer(PLAYER_ID_GAME_MASTER);
     }
 }   // enableAllButtons
 
@@ -330,7 +328,7 @@ void RaceResultGUI::eventCallback(GUIEngine::Widget* widget,
             }
             return;
         }
-        Log::fatal("RaceResultGUI", "Incorrect event '%s' when things are unlocked.",
+        Log::warn("RaceResultGUI", "Incorrect event '%s' when things are unlocked.",
             name.c_str());
     }
 
@@ -341,10 +339,9 @@ void RaceResultGUI::eventCallback(GUIEngine::Widget* widget,
         if (name == "middle") // Continue button (return to server lobby)
         {
             // Signal to the server that this client is back in the lobby now.
-            Protocol* protocol =
-                ProtocolManager::getInstance()->getProtocol(PROTOCOL_LOBBY_ROOM);
-            ClientLobbyRoomProtocol* clrp =
-                static_cast<ClientLobbyRoomProtocol*>(protocol);
+            Protocol* protocol = LobbyProtocol::get();
+            ClientLobby* clrp =
+                dynamic_cast<ClientLobby*>(protocol);
             if(clrp)
                 clrp->doneWithResults();
             backToLobby();
@@ -374,8 +371,10 @@ void RaceResultGUI::eventCallback(GUIEngine::Widget* widget,
                 MessageDialog::MESSAGE_DIALOG_CONFIRM, this, false);
         }
         else if (!getWidget(name.c_str())->isVisible())
-            Log::fatal("RaceResultGUI", "Incorrect event '%s' when things are unlocked.",
+        {
+            Log::warn("RaceResultGUI", "Incorrect event '%s' when things are unlocked.",
                 name.c_str());
+        }
         return;
     }
 
@@ -386,10 +385,6 @@ void RaceResultGUI::eventCallback(GUIEngine::Widget* widget,
     {
         race_manager->exitRace();
         race_manager->setAIKartOverride("");
-        // FIXME: why is this call necessary here? tearDown should be
-        // automatically called when the screen is left. Note that the
-        // NetworkKartSelectionScreen::getInstance()->tearDown(); caused #1347
-        KartSelectionScreen::getRunningInstance()->tearDown();
         Screen* newStack[] = { MainMenuScreen::getInstance(),
                               RaceSetupScreen::getInstance(),
                               NULL };
@@ -403,11 +398,6 @@ void RaceResultGUI::eventCallback(GUIEngine::Widget* widget,
     {
         race_manager->exitRace();
         race_manager->setAIKartOverride("");
-        // FIXME: why is this call necessary here? tearDown should be
-        // automatically called when the screen is left. Note that the
-        // NetworkKartSelectionScreen::getInstance()->tearDown(); caused #1347
-        //if (KartSelectionScreen::getRunningInstance() != NULL)
-        //    KartSelectionScreen::getRunningInstance()->tearDown();
         StateManager::get()->resetAndGoToScreen(MainMenuScreen::getInstance());
 
         if (race_manager->raceWasStartedFromOverworld())
@@ -416,7 +406,7 @@ void RaceResultGUI::eventCallback(GUIEngine::Widget* widget,
         }
     }
     else
-        Log::fatal("RaceResultGUI", "Incorrect event '%s' for normal race.",
+        Log::warn("RaceResultGUI", "Incorrect event '%s' for normal race.",
             name.c_str());
     return;
 }   // eventCallback
@@ -464,22 +454,22 @@ void RaceResultGUI::backToLobby()
 
         m_font = GUIEngine::getFont();
         assert(m_font);
-        m_was_monospace = m_font->getMonospaceDigits();
-        m_font->setMonospaceDigits(true);
+        //m_was_monospace = m_font->getMonospaceDigits();
+        //m_font->setMonospaceDigits(true);
         WorldWithRank *rank_world = (WorldWithRank*)World::getWorld();
 
         unsigned int first_position = 1;
+        unsigned int sta = race_manager->getNumSpareTireKarts();
         if (race_manager->getMinorMode() == RaceManager::MINOR_MODE_FOLLOW_LEADER)
             first_position = 2;
 
         // Use only the karts that are supposed to be displayed (and
         // ignore e.g. the leader in a FTL race).
-        unsigned int num_karts = race_manager->getNumberOfKarts() - first_position + 1;
+        unsigned int num_karts = race_manager->getNumberOfKarts() - first_position + 1 - sta;
 
         // In FTL races the leader kart is not displayed
         m_all_row_infos.resize(num_karts);
 
-
         // Determine the kart to display in the right order,
         // and the maximum width for the kart name column
         // -------------------------------------------------
@@ -487,14 +477,14 @@ void RaceResultGUI::backToLobby()
         float max_finish_time = 0;
 
         for (unsigned int position = first_position;
-        position <= race_manager->getNumberOfKarts(); position++)
+        position <= race_manager->getNumberOfKarts() - sta; position++)
         {
             const AbstractKart *kart = rank_world->getKartAtPosition(position);
 
             // Save a pointer to the current row_info entry
             RowInfo *ri = &(m_all_row_infos[position - first_position]);
             ri->m_is_player_kart = kart->getController()->isLocalPlayerController();
-            ri->m_kart_name = getKartDisplayName(kart);
+            ri->m_kart_name = kart->getController()->getName();
 
             video::ITexture *icon =
                 kart->getKartProperties()->getIconMaterial()->getTexture();
@@ -565,14 +555,14 @@ void RaceResultGUI::backToLobby()
 
         // Determine width of new points column
 
-        m_font->setMonospaceDigits(true);
+        //m_font->setMonospaceDigits(true);
         core::dimension2du r_new_p = m_font->getDimension(L"+99");
 
         m_width_new_points = r_new_p.Width;
 
         // Determine width of overall points column
         core::dimension2du r_all_p = m_font->getDimension(L"999");
-        m_font->setMonospaceDigits(false);
+        //m_font->setMonospaceDigits(false);
 
         m_width_all_points = r_all_p.Width;
 
@@ -668,6 +658,7 @@ void RaceResultGUI::backToLobby()
      */
     void RaceResultGUI::renderGlobal(float dt)
     {
+#ifndef SERVER_ONLY
         bool isSoccerWorld = race_manager->getMinorMode() == RaceManager::MINOR_MODE_SOCCER;
 
         m_timer += dt;
@@ -837,8 +828,9 @@ void RaceResultGUI::backToLobby()
         if (race_manager->getMajorMode() != RaceManager::MAJOR_MODE_GRAND_PRIX ||
             m_animation_state == RR_RACE_RESULT)
         {
-            displayHighScores();
+            displayPostRaceInfo();
         }
+#endif
     }   // renderGlobal
 
     //-----------------------------------------------------------------------------
@@ -847,6 +839,7 @@ void RaceResultGUI::backToLobby()
      */
     void RaceResultGUI::determineGPLayout()
     {
+#ifndef SERVER_ONLY
         unsigned int num_karts = race_manager->getNumberOfKarts();
         std::vector<int> old_rank(num_karts, 0);
         for (unsigned int kart_id = 0; kart_id < num_karts; kart_id++)
@@ -860,7 +853,7 @@ void RaceResultGUI::backToLobby()
             ri->m_kart_icon =
                 kart->getKartProperties()->getIconMaterial()->getTexture();
             ri->m_is_player_kart = kart->getController()->isLocalPlayerController();
-            ri->m_kart_name = getKartDisplayName(kart);
+            ri->m_kart_name = kart->getController()->getName();
 
             // In FTL karts do have a time, which is shown even when the kart
             // is eliminated
@@ -909,31 +902,9 @@ void RaceResultGUI::backToLobby()
             int p = race_manager->getKartScore(i);
             ri->m_new_overall_points = p;
         }   // i < num_karts
+#endif
     }   // determineGPLayout
 
-    //-----------------------------------------------------------------------------
-    /** Returns a string to display next to a kart. For a player that's the name
-     *  of the player, for an AI kart it's the name of the driver. 
-     */
-    core::stringw RaceResultGUI::getKartDisplayName(const AbstractKart *kart) const
-    {
-        const EndController *ec = 
-            dynamic_cast<const EndController*>(kart->getController());
-        // If the race was given up, there is no end controller for the
-        // players, so this case needs to be handled separately
-        if(ec && ec->isLocalPlayerController())
-            return ec->getName();
-        else
-        {
-            // No end controller, check explicitely for a player controller
-            const PlayerController *pc = 
-                dynamic_cast<const PlayerController*>(kart->getController());
-            // Check if the kart is a player controller to get the real name
-            if(pc) return pc->getName();
-        }
-        return translations->fribidize(kart->getName());
-    }   // getKartDisplayName
-
     //-----------------------------------------------------------------------------
     /** Displays the race results for a single kart.
      *  \param n Index of the kart to be displayed.
@@ -942,6 +913,7 @@ void RaceResultGUI::backToLobby()
     void RaceResultGUI::displayOneEntry(unsigned int x, unsigned int y,
         unsigned int n, bool display_points)
     {
+#ifndef SERVER_ONLY
         RowInfo *ri = &(m_all_row_infos[n]);
         video::SColor color = ri->m_is_player_kart
             ? video::SColor(255, 255, 0, 0)
@@ -1012,12 +984,13 @@ void RaceResultGUI::backToLobby()
             m_font->draw(point_inc_string, dest_rect, color, false, false, NULL,
                 true /* ignoreRTL */);
         }
+#endif
     }   // displayOneEntry
 
     //-----------------------------------------------------------------------------
     void RaceResultGUI::displaySoccerResults()
     {
-
+#ifndef SERVER_ONLY
         //Draw win text
         core::stringw result_text;
         static video::SColor color = video::SColor(255, 255, 255, 255);
@@ -1090,7 +1063,15 @@ void RaceResultGUI::backToLobby()
         current_y += rect.Height / 2 + rect.Height / 4;
         font = GUIEngine::getSmallFont();
         std::vector<SoccerWorld::ScorerData> scorers = sw->getScorers(SOCCER_TEAM_RED);
+        while (scorers.size() > 10)
+        {
+            scorers.erase(scorers.begin());
+        }
         std::vector<float> score_times = sw->getScoreTimes(SOCCER_TEAM_RED);
+        while (score_times.size() > 10)
+        {
+            score_times.erase(score_times.begin());
+        }
         irr::video::ITexture* scorer_icon;
 
         int prev_y = current_y;
@@ -1111,6 +1092,7 @@ void RaceResultGUI::backToLobby()
             if (own_goal)
             {
                 result_text.append(" ");
+                //I18N: indicates a player that scored in their own goal in result screen
                 result_text.append(_("(Own Goal)"));
             }
 
@@ -1141,7 +1123,15 @@ void RaceResultGUI::backToLobby()
         current_y = prev_y;
         current_x += UserConfigParams::m_width / 2 - red_icon->getSize().Width / 2;
         scorers = sw->getScorers(SOCCER_TEAM_BLUE);
+        while (scorers.size() > 10)
+        {
+            scorers.erase(scorers.begin());
+        }
         score_times = sw->getScoreTimes(SOCCER_TEAM_BLUE);
+        while (score_times.size() > 10)
+        {
+            score_times.erase(score_times.begin());
+        }
         for (unsigned int i = 0; i < scorers.size(); i++)
         {
             const bool own_goal = !(scorers.at(i).m_correct_goal);
@@ -1185,6 +1175,7 @@ void RaceResultGUI::backToLobby()
             draw2DImage(scorer_icon, dest_rect, source_rect,
                 NULL, NULL, true);
         }
+#endif
     }
 
     //-----------------------------------------------------------------------------
@@ -1324,27 +1315,31 @@ void RaceResultGUI::backToLobby()
     // ----------------------------------------------------------------------------
     void RaceResultGUI::cleanupGPProgress()
     {
-        for (size_t i = 0; i < m_gp_progress_widgets.size(); i++)
+        for (unsigned int i = 0; i < m_gp_progress_widgets.size(); i++)
             m_widgets.remove(m_gp_progress_widgets.get(i));
         m_gp_progress_widgets.clearAndDeleteAll();
     }   // cleanupGPProgress
 
     // ----------------------------------------------------------------------------
-    void RaceResultGUI::displayHighScores()
+    void RaceResultGUI::displayPostRaceInfo()
     {
+#ifndef SERVER_ONLY
         // This happens in demo world
         if (!World::getWorld())
             return;
 
         Highscores* scores = World::getWorld()->getHighscores();
+
+        video::SColor white_color = video::SColor(255, 255, 255, 255);
+
+        int x = (int)(UserConfigParams::m_width*0.65f);
+        int y = m_top;
+
+        int current_y = y;
+
         // In some case for exemple FTL they will be no highscores
         if (scores != NULL)
         {
-            video::SColor white_color = video::SColor(255, 255, 255, 255);
-
-            int x = (int)(UserConfigParams::m_width*0.65f);
-            int y = m_top;
-
             // First draw title
             GUIEngine::getFont()->draw(_("Highscores"),
                 core::recti(x, y, 0, 0),
@@ -1377,7 +1372,7 @@ void RaceResultGUI::backToLobby()
                 }
 
                 int current_x = x;
-                int current_y = y + (int)((i + 1) * m_distance_between_rows * 1.5f);
+                current_y = y + (int)((i + 1) * m_distance_between_rows * 1.5f);
 
                 const KartProperties* prop = kart_properties_manager->getKart(kart_name);
                 if (prop != NULL)
@@ -1418,6 +1413,37 @@ void RaceResultGUI::backToLobby()
                     false, false, NULL, true /* ignoreRTL */);
             }
         }
+
+        if (race_manager->getMinorMode() != RaceManager::MINOR_MODE_SOCCER)
+        {
+            // display lap count
+            if (race_manager->modeHasLaps())
+            {
+                core::stringw laps = _("Laps: %i", race_manager->getNumLaps());
+                current_y += int(m_distance_between_rows * 0.8f * 2);
+                GUIEngine::getFont()->draw(laps, core::recti(x, current_y, 0, 0),
+                    white_color, false, false, nullptr, true);
+            }
+            // display difficulty
+            const core::stringw& difficulty_name =
+                race_manager->getDifficultyName(race_manager->getDifficulty());
+            core::stringw difficulty_string = _("Difficulty: %s", difficulty_name);
+            current_y += int(m_distance_between_rows * 0.8f);
+            GUIEngine::getFont()->draw(difficulty_string, core::recti(x, current_y, 0, 0),
+                white_color, false, false, nullptr, true);
+            // show fastest lap
+            if (race_manager->modeHasLaps())
+            {
+                float best_lap_time = static_cast<LinearWorld*>(World::getWorld())->getFastestLap();
+                core::stringw best_lap_string = _("Best lap time: %s",
+                    StringUtils::timeToString(best_lap_time).c_str());
+                current_y += int(m_distance_between_rows * 0.8f);
+                GUIEngine::getFont()->draw(best_lap_string,
+                    core::recti(x, current_y, 0, 0), white_color, false, false,
+                    nullptr, true);
+            }   // if mode has laps
+        }   // if not soccer mode
+#endif
     }
 
     // ----------------------------------------------------------------------------
