@@ -23,7 +23,7 @@
 
 #include "guiengine/screen.hpp"
 #include "guiengine/widgets/spinner_widget.hpp"
-
+#include "input/input.hpp"
 namespace GUIEngine
 {
     class CheckBoxWidget;
@@ -103,20 +103,20 @@ private:
 
 public:
     /** \brief implement callback from parent class GUIEngine::Screen */
-    virtual void loadedFromFile();
+    virtual void loadedFromFile() OVERRIDE;
 
     /** \brief implement callback from parent class GUIEngine::Screen */
     virtual void eventCallback(GUIEngine::Widget* widget,
-                               const std::string& name, const int playerID);
+                               const std::string& name, const int playerID) OVERRIDE;
 
     /** \brief implement callback from parent class GUIEngine::Screen */
-    virtual void init();
+    virtual void init() OVERRIDE;
 
     /** \brief implement callback from parent class GUIEngine::Screen */
-    virtual void tearDown();
+    virtual void tearDown() OVERRIDE;
 
     /** \brief implement optional callback from parent class GUIEngine::Screen */
-    virtual void unloaded();
+    virtual void unloaded() OVERRIDE;
 
     void setNewAccountData(bool online, bool auto_login,
                            const core::stringw &online_name="",
@@ -125,6 +125,12 @@ public:
     void loginError(const irr::core::stringw &error_message);
     void logoutSuccessful();
     void logoutError(const irr::core::stringw &error_message);
+
+    virtual GUIEngine::EventPropagation filterActions(PlayerAction action,
+        int deviceID,
+        const unsigned int value,
+        Input::InputType type,
+        int playerId) OVERRIDE;
 };   // class BaseUserScreen
 
 // ============================================================================
@@ -149,9 +155,9 @@ private:
 public:
     friend class GUIEngine::ScreenSingleton<TabbedUserScreen>;
 
-    virtual void init();
+    virtual void init() OVERRIDE;
     virtual void eventCallback(GUIEngine::Widget* widget,
-                               const std::string& name, const int playerID);
+                               const std::string& name, const int playerID) OVERRIDE;
 };   // class TabbedUserScreen
 
 #endif
