@@ -31,7 +31,7 @@ public:
   virtual ~FileSystem() {}
 
   virtual std::vector<std::string>    open_directory(const std::string& pathname) =0;
-  virtual std::auto_ptr<std::istream> open_file(const std::string& filename)      =0;
+  virtual std::unique_ptr<std::istream> open_file(const std::string& filename)      =0;
 };
 
 } // namespace tinygettext
