@@ -25,7 +25,9 @@
 #include <string>
 #include <vector>
 
+class AbstractKart;
 class CheckStructure;
+class Flyable;
 class Track;
 class XMLNode;
 class Vec3;
@@ -45,9 +47,12 @@ private:
           ~CheckManager();
 public:
     void   add(CheckStructure* strct) { m_all_checks.push_back(strct); }
+    void   addFlyableToCannons(Flyable *flyable);
+    void   removeFlyableFromCannons(Flyable *flyable);
     void   load(const XMLNode &node);
     void   update(float dt);
     void   reset(const Track &track);
+    void   resetAfterKartMove(AbstractKart *kart);
     unsigned int getLapLineIndex() const;
     int    getChecklineTriggering(const Vec3 &from, const Vec3 &to) const;
     // ------------------------------------------------------------------------
