@@ -20,15 +20,20 @@
 
 #include "animations/three_d_animation.hpp"
 #include "graphics/irr_driver.hpp"
+#include "graphics/lod_node.hpp"
+#include "graphics/material.hpp"
+#include "graphics/material_manager.hpp"
+#include "graphics/render_info.hpp"
 #include "io/file_manager.hpp"
 #include "io/xml_node.hpp"
 #include "input/device_manager.hpp"
 #include "items/item_manager.hpp"
-#include "modes/world.hpp"
 #include "physics/physical_object.hpp"
 #include "race/race_manager.hpp"
 #include "scriptengine/script_engine.hpp"
-#include "utils/helpers.hpp"
+#include "tracks/model_definition_loader.hpp"
+
+#include <IAnimatedMeshSceneNode.h>
 #include <ISceneManager.h>
 
 /** A track object: any additional object on the track. This object implements
@@ -67,6 +72,7 @@ TrackObject::TrackObject(const core::vector3df& xyz, const core::vector3df& hpr,
     m_animator        = NULL;
     m_physical_object = NULL;
     m_parent_library  = NULL;
+    m_render_info     = NULL;
     m_interaction     = interaction;
     m_presentation    = presentation;
     m_is_driveable    = false;
@@ -98,6 +104,7 @@ void TrackObject::init(const XMLNode &xml_node, scene::ISceneNode* parent,
     m_init_xyz   = core::vector3df(0,0,0);
     m_init_hpr   = core::vector3df(0,0,0);
     m_init_scale = core::vector3df(1,1,1);
+    m_render_info = NULL;
     m_enabled    = true;
     m_initially_visible = false;
     m_presentation = NULL;
@@ -151,6 +158,7 @@ void TrackObject::init(const XMLNode &xml_node, scene::ISceneNode* parent,
     }
     else if (xml_node.getName() == "library")
     {
+        xml_node.get("name", &m_name);
         m_presentation = new TrackObjectPresentationLibraryNode(this, xml_node, model_def_loader);
     }
     else if (type == "sfx-emitter")
@@ -165,7 +173,7 @@ void TrackObject::init(const XMLNode &xml_node, scene::ISceneNode* parent,
         std::string action;
         xml_node.get("action", &action);
         m_name = action; //adds action as name so that it can be found by using getName()
-        m_presentation = new TrackObjectPresentationActionTrigger(xml_node);
+        m_presentation = new TrackObjectPresentationActionTrigger(xml_node, parent_library);
     }
     else if (type == "billboard")
     {
@@ -177,13 +185,77 @@ void TrackObject::init(const XMLNode &xml_node, scene::ISceneNode* parent,
     }
     else
     {
+        // Colorization settings
+        std::string model_name;
+        xml_node.get("model", &model_name);
+        bool colorizable = false;
+        scene::IMesh* mesh = NULL;
+        // Only non-lod groups can use dynamic hue for different parts of mesh
+        bool use_dynamic_hue = true;
+        float static_hue = 0.0f;
+        if (model_name.size() > 0)
+        {
+            mesh = irr_driver->getMesh(model_name);
+            if (mesh != NULL)
+            {
+                unsigned int n = mesh->getMeshBufferCount();
+                for (unsigned int i = 0; i < n; i++)
+                {
+                    scene::IMeshBuffer *mb = mesh->getMeshBuffer(i);
+                    Material* m = material_manager->getMaterialFor(mb
+                        ->getMaterial().getTexture(0), mb);
+                    colorizable = colorizable || m->isColorizable();
+                    if (colorizable) break;
+                }
+            }
+        }
+        else
+        {
+            std::string group_name = "";
+            xml_node.get("lod_group", &group_name);
+            // Try to get the first mesh from lod groups
+            mesh = model_def_loader.getFirstMeshFor(group_name);
+            if (mesh != NULL)
+            {
+                use_dynamic_hue = false;
+                unsigned int n = mesh->getMeshBufferCount();
+                for (unsigned int i = 0; i < n; i++)
+                {
+                    scene::IMeshBuffer *mb = mesh->getMeshBuffer(i);
+                    Material* m = material_manager->getMaterialFor(mb
+                        ->getMaterial().getTexture(0), mb);
+                    if (m->isColorizable())
+                    {
+                        // Use the first texture to determine static hue
+                        // Other texture that is non-colorizable will be
+                        // untouched, otherwise they will be colorized the
+                        // same hue
+                        colorizable = true;
+                        static_hue = m->getRandomHue();
+                        break;
+                    }
+                }
+            }
+        }
+
+
+        // If at least one material is colorizable, add RenderInfo for it
+        if (colorizable)
+        {
+            m_render_info = new RenderInfo();
+            if (use_dynamic_hue)
+                m_render_info->setDynamicHue(mesh);
+            else
+                m_render_info->setHue(static_hue);
+        }
+
         scene::ISceneNode *glownode = NULL;
         bool is_movable = false;
         if (lod_instance)
         {
             m_type = "lod";
             TrackObjectPresentationLOD* lod_node =
-                new TrackObjectPresentationLOD(xml_node, parent, model_def_loader);
+                new TrackObjectPresentationLOD(xml_node, parent, model_def_loader, m_render_info);
             m_presentation = lod_node;
 
             LODNode* node = (LODNode*)lod_node->getNode();
@@ -207,7 +279,8 @@ void TrackObject::init(const XMLNode &xml_node, scene::ISceneNode* parent,
             m_type = "mesh";
             m_presentation = new TrackObjectPresentationMesh(xml_node,
                                                              m_enabled,
-                                                             parent);
+                                                             parent,
+                                                             m_render_info);
             scene::ISceneNode* node = ((TrackObjectPresentationMesh *)m_presentation)->getNode();
             if (type == "movable" && parent != NULL)
             {
@@ -262,8 +335,7 @@ void TrackObject::init(const XMLNode &xml_node, scene::ISceneNode* parent,
         {
             float power = 1;
             xml_node.get("bloompower", &power);
-            power = clampf(power, 0.5f, 10);
-
+            btClamp(power, 0.5f, 10.0f);
             irr_driver->addForcedBloomNode(glownode, power);
         }
     }
@@ -293,7 +365,8 @@ void TrackObject::onWorldReady()
     else if (m_visibility_condition.size() > 0)
     {
         unsigned char result = -1;
-        Scripting::ScriptEngine* script_engine = World::getWorld()->getScriptEngine();
+        Scripting::ScriptEngine* script_engine = 
+                                        Scripting::ScriptEngine::getInstance();
 
         std::ostringstream fn_signature;
         std::vector<std::string> arguments;
@@ -303,8 +376,8 @@ void TrackObject::onWorldReady()
             // There are arguments to pass to the function
             // TODO: For the moment we only support string arguments
             // TODO: this parsing could be improved
-            unsigned first = m_visibility_condition.find("(");
-            unsigned last = m_visibility_condition.find_last_of(")");
+            unsigned first = (unsigned)m_visibility_condition.find("(");
+            unsigned last = (unsigned)m_visibility_condition.find_last_of(")");
             std::string fn_name = m_visibility_condition.substr(0, first);
             std::string str_arguments = m_visibility_condition.substr(first + 1, last - first - 1);
             arguments = StringUtils::split(str_arguments, ',');
@@ -333,7 +406,7 @@ void TrackObject::onWorldReady()
                 {
                     ctx->SetArgObject(i, &arguments[i]);
                 }
-                ctx->SetArgObject(arguments.size(), self);
+                ctx->SetArgObject((int)arguments.size(), self);
             },
             [&](asIScriptContext* ctx) { result = ctx->GetReturnByte(); });
 
@@ -354,6 +427,7 @@ TrackObject::~TrackObject()
     delete m_presentation;
     delete m_animator;
     delete m_physical_object;
+    delete m_render_info;
 }   // ~TrackObject
 
 // ----------------------------------------------------------------------------
@@ -590,3 +664,35 @@ void TrackObject::moveTo(const Scripting::SimpleVec3* pos, bool isAbsoluteCoord)
             isAbsoluteCoord);
     }
 }
+
+// ----------------------------------------------------------------------------
+scene::IAnimatedMeshSceneNode* TrackObject::getMesh()
+{
+    if (getPresentation<TrackObjectPresentationLOD>())
+    {
+        LODNode* ln = dynamic_cast<LODNode*>
+            (getPresentation<TrackObjectPresentationLOD>()->getNode());
+        if (ln && !ln->getAllNodes().empty())
+        {
+            scene::IAnimatedMeshSceneNode* an =
+                dynamic_cast<scene::IAnimatedMeshSceneNode*>
+                (ln->getFirstNode());
+            if (an)
+            {
+                return an;
+            }
+        }
+    }
+    else if (getPresentation<TrackObjectPresentationMesh>())
+    {
+        scene::IAnimatedMeshSceneNode* an =
+            dynamic_cast<scene::IAnimatedMeshSceneNode*>
+            (getPresentation<TrackObjectPresentationMesh>()->getNode());
+        if (an)
+        {
+            return an;
+        }
+    }
+    Log::debug("TrackObject", "No animated mesh");
+    return NULL;
+}   // getMesh
