@@ -19,23 +19,35 @@
 #include "debug.hpp"
 
 #include "config/user_config.hpp"
-#include "graphics/camera.hpp"
+#include "font/bold_face.hpp"
+#include "font/digit_face.hpp"
+#include "font/font_manager.hpp"
+#include "font/regular_face.hpp"
+#include "graphics/camera_debug.hpp"
+#include "graphics/camera_fps.hpp"
+#include "karts/explosion_animation.hpp"
 #include "graphics/irr_driver.hpp"
 #include "graphics/light.hpp"
 #include "graphics/shaders.hpp"
+#include "graphics/stk_tex_manager.hpp"
+#include "guiengine/widgets/label_widget.hpp"
+#include "guiengine/widgets/text_box_widget.hpp"
 #include "items/powerup_manager.hpp"
 #include "items/attachment.hpp"
 #include "karts/abstract_kart.hpp"
 #include "karts/kart_properties.hpp"
 #include "karts/controller/controller.hpp"
+#include "modes/cutscene_world.hpp"
 #include "modes/world.hpp"
 #include "physics/irr_debug_drawer.hpp"
 #include "physics/physics.hpp"
 #include "race/history.hpp"
 #include "main_loop.hpp"
 #include "replay/replay_recorder.hpp"
+#include "scriptengine/script_engine.hpp"
 #include "states_screens/dialogs/debug_slider.hpp"
-#include "states_screens/dialogs/scripting_console.hpp"
+#include "states_screens/dialogs/general_text_field_dialog.hpp"
+#include "tracks/track_manager.hpp"
 #include "utils/constants.hpp"
 #include "utils/log.hpp"
 #include "utils/profiler.hpp"
@@ -76,7 +88,9 @@ enum DebugMenuCommand
     DEBUG_GRAPHICS_BULLET_2,
     DEBUG_GRAPHICS_BOUNDING_BOXES_VIZ,
     DEBUG_PROFILER,
-    DEBUG_PROFILER_GENERATE_REPORT,
+    DEBUG_PROFILER_WRITE_REPORT,
+    DEBUG_FONT_DUMP_GLYPH_PAGE,
+    DEBUG_FONT_RELOAD,
     DEBUG_FPS,
     DEBUG_SAVE_REPLAY,
     DEBUG_SAVE_HISTORY,
@@ -93,6 +107,7 @@ enum DebugMenuCommand
     DEBUG_ATTACHMENT_PARACHUTE,
     DEBUG_ATTACHMENT_BOMB,
     DEBUG_ATTACHMENT_ANVIL,
+    DEBUG_ATTACHMENT_EXPLOSION,
     DEBUG_GUI_TOGGLE,
     DEBUG_GUI_HIDE_KARTS,
     DEBUG_GUI_CAM_FREE,
@@ -116,7 +131,11 @@ enum DebugMenuCommand
     DEBUG_VISUAL_VALUES,
     DEBUG_PRINT_START_POS,
     DEBUG_ADJUST_LIGHTS,
-    DEBUG_SCRIPT_CONSOLE
+    DEBUG_SCRIPT_CONSOLE,
+    DEBUG_RUN_CUTSCENE,
+    DEBUG_TEXTURE_CONSOLE,
+    DEBUG_START_RECORDING,
+    DEBUG_STOP_RECORDING
 };   // DebugMenuCommand
 
 // -----------------------------------------------------------------------------
@@ -174,12 +193,9 @@ void changeCameraTarget(u32 num)
     {
         AbstractKart* kart = world->getKart(num - 1);
         if (kart == NULL) return;
-        if (kart->isEliminated()) return;
         cam->setMode(Camera::CM_NORMAL);
         cam->setKart(kart);
     }
-    else
-        return;
 
 }   // changeCameraTarget
 
@@ -220,186 +236,172 @@ LightNode* findNearestLight()
 
 // ----------------------------------------------------------------------------
 
-bool handleContextMenuAction(s32 cmdID)
+bool handleContextMenuAction(s32 cmd_id)
 {
 
     World *world = World::getWorld();
-    Physics *physics = world ? world->getPhysics() : NULL;
-    if (cmdID == DEBUG_GRAPHICS_RELOAD_SHADERS)
+    Physics *physics = Physics::getInstance();
+    switch(cmd_id)
     {
+    case DEBUG_GRAPHICS_RELOAD_SHADERS:
+#ifndef SERVER_ONLY
         Log::info("Debug", "Reloading shaders...");
+        ShaderFilesManager::getInstance()->clean();
         ShaderBase::updateShaders();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_RESET)
-    {
+#endif
+        break;
+    case DEBUG_GRAPHICS_RESET:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_WIREFRAME)
-    {
+        break;
+    case DEBUG_GRAPHICS_WIREFRAME:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleWireframe();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_MIPMAP_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_MIPMAP_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleMipVisualization();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_NORMALS_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_NORMALS_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleNormals();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_SSAO_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_SSAO_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleSSAOViz();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_RSM_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_RSM_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleRSM();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_RH_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_RH_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleRH();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_GI_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_GI_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleGI();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_SHADOW_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_SHADOW_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleShadowViz();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_LIGHT_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_LIGHT_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleLightViz();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_DISTORT_VIZ)
-    {
+        break;
+    case DEBUG_GRAPHICS_DISTORT_VIZ:
         if (physics)
             physics->setDebugMode(IrrDebugDrawer::DM_NONE);
 
         irr_driver->resetDebugModes();
         irr_driver->toggleDistortViz();
-    }
-    else if (cmdID == DEBUG_GRAPHICS_BULLET_1)
-    {
+        break;
+    case DEBUG_GRAPHICS_BULLET_1:
         irr_driver->resetDebugModes();
 
         if (!world) return false;
         physics->setDebugMode(IrrDebugDrawer::DM_KARTS_PHYSICS);
-    }
-    else if (cmdID == DEBUG_GRAPHICS_BULLET_2)
+        break;
+    case DEBUG_GRAPHICS_BULLET_2:
     {
         irr_driver->resetDebugModes();
 
-        if (!world) return false;
-        Physics *physics = world->getPhysics();
+        Physics *physics = Physics::getInstance();
+        if (!physics) return false;
         physics->setDebugMode(IrrDebugDrawer::DM_NO_KARTS_GRAPHICS);
+        break;
     }
-    else if (cmdID == DEBUG_GRAPHICS_BOUNDING_BOXES_VIZ)
-    {
+    case DEBUG_GRAPHICS_BOUNDING_BOXES_VIZ:
         irr_driver->resetDebugModes();
         irr_driver->toggleBoundingBoxesViz();
-    }
-    else if (cmdID == DEBUG_PROFILER)
-    {
-        UserConfigParams::m_profiler_enabled =
-            !UserConfigParams::m_profiler_enabled;
-    }
-    else if (cmdID == DEBUG_PROFILER_GENERATE_REPORT)
-    {
-        profiler.setCaptureReport(!profiler.getCaptureReport());
-    }
-    else if (cmdID == DEBUG_THROTTLE_FPS)
-    {
+        break;
+    case DEBUG_PROFILER:
+        profiler.toggleStatus();
+        break;
+    case DEBUG_PROFILER_WRITE_REPORT:
+        profiler.writeToFile();
+        break;
+    case DEBUG_THROTTLE_FPS:
         main_loop->setThrottleFPS(false);
-    }
-    else if (cmdID == DEBUG_FPS)
-    {
+        break;
+    case DEBUG_FONT_DUMP_GLYPH_PAGE:
+        font_manager->getFont<BoldFace>()->dumpGlyphPage("bold");
+        font_manager->getFont<DigitFace>()->dumpGlyphPage("digit");
+        font_manager->getFont<RegularFace>()->dumpGlyphPage("regular");
+        break;
+    case DEBUG_FONT_RELOAD:
+        font_manager->getFont<BoldFace>()->reset();
+        font_manager->getFont<DigitFace>()->reset();
+        font_manager->getFont<RegularFace>()->reset();
+        break;
+    case DEBUG_FPS:
         UserConfigParams::m_display_fps =
             !UserConfigParams::m_display_fps;
-    }
-    else if (cmdID == DEBUG_SAVE_REPLAY)
-    {
+        break;
+    case DEBUG_SAVE_REPLAY:
         ReplayRecorder::get()->save();
-    }
-    else if (cmdID == DEBUG_SAVE_HISTORY)
-    {
+        break;
+    case DEBUG_SAVE_HISTORY:
         history->Save();
-    }
-    else if (cmdID == DEBUG_POWERUP_BOWLING)
-    {
+        break;
+    case DEBUG_POWERUP_BOWLING:
         addPowerup(PowerupManager::POWERUP_BOWLING);
-    }
-    else if (cmdID == DEBUG_POWERUP_BUBBLEGUM)
-    {
+        break;
+    case DEBUG_POWERUP_BUBBLEGUM:
         addPowerup(PowerupManager::POWERUP_BUBBLEGUM);
-    }
-    else if (cmdID == DEBUG_POWERUP_CAKE)
-    {
+        break;
+    case DEBUG_POWERUP_CAKE:
         addPowerup(PowerupManager::POWERUP_CAKE);
-    }
-    else if (cmdID == DEBUG_POWERUP_PARACHUTE)
-    {
+        break;
+    case DEBUG_POWERUP_PARACHUTE:
         addPowerup(PowerupManager::POWERUP_PARACHUTE);
-    }
-    else if (cmdID == DEBUG_POWERUP_PLUNGER)
-    {
+        break;
+    case DEBUG_POWERUP_PLUNGER:
         addPowerup(PowerupManager::POWERUP_PLUNGER);
-    }
-    else if (cmdID == DEBUG_POWERUP_RUBBERBALL)
-    {
+        break;
+    case DEBUG_POWERUP_RUBBERBALL:
         addPowerup(PowerupManager::POWERUP_RUBBERBALL);
-    }
-    else if (cmdID == DEBUG_POWERUP_SWATTER)
-    {
+        break;
+    case DEBUG_POWERUP_SWATTER:
         addPowerup(PowerupManager::POWERUP_SWATTER);
-    }
-    else if (cmdID == DEBUG_POWERUP_SWITCH)
-    {
+        break;
+    case DEBUG_POWERUP_SWITCH:
         addPowerup(PowerupManager::POWERUP_SWITCH);
-    }
-    else if (cmdID == DEBUG_POWERUP_ZIPPER)
-    {
+        break;
+    case DEBUG_POWERUP_ZIPPER:
         addPowerup(PowerupManager::POWERUP_ZIPPER);
-    }
-    else if (cmdID == DEBUG_POWERUP_NITRO)
+        break;
+    case DEBUG_POWERUP_NITRO:
     {
         if (!world) return false;
         const unsigned int num_local_players =
@@ -409,27 +411,32 @@ bool handleContextMenuAction(s32 cmdID)
             AbstractKart* kart = world->getLocalPlayerKart(i);
             kart->setEnergy(100.0f);
         }
+        break;
     }
-    else if (cmdID == DEBUG_ATTACHMENT_ANVIL)
-    {
+    case DEBUG_ATTACHMENT_ANVIL:
         addAttachment(Attachment::ATTACH_ANVIL);
-    }
-    else if (cmdID == DEBUG_ATTACHMENT_BOMB)
-    {
+        break;
+    case DEBUG_ATTACHMENT_BOMB:
         addAttachment(Attachment::ATTACH_BOMB);
-    }
-    else if (cmdID == DEBUG_ATTACHMENT_PARACHUTE)
-    {
+        break;
+    case DEBUG_ATTACHMENT_PARACHUTE:
         addAttachment(Attachment::ATTACH_PARACHUTE);
-    }
-    else if (cmdID == DEBUG_GUI_TOGGLE)
+        break;
+    case DEBUG_ATTACHMENT_EXPLOSION:
+        for (unsigned int i = 0; i < race_manager->getNumLocalPlayers(); i++)
+        {
+            AbstractKart* kart = world->getLocalPlayerKart(i);
+            ExplosionAnimation::create(kart, kart->getXYZ(), true);
+        }
+        break;
+    case DEBUG_GUI_TOGGLE:
     {
         if (!world) return false;
         RaceGUIBase* gui = world->getRaceGUI();
         if (gui != NULL) gui->m_enabled = !gui->m_enabled;
+        break;
     }
-    else if (cmdID == DEBUG_GUI_HIDE_KARTS)
-    {
+    case DEBUG_GUI_HIDE_KARTS:
         if (!world) return false;
         for (unsigned int n = 0; n<world->getNumKarts(); n++)
         {
@@ -437,85 +444,91 @@ bool handleContextMenuAction(s32 cmdID)
             if (kart->getController()->isPlayerController())
                 kart->getNode()->setVisible(false);
         }
-    }
-    else if (cmdID == DEBUG_GUI_CAM_TOP)
-    {
-        Camera::setDebugMode(Camera::CM_DEBUG_TOP_OF_KART);
+        break;
+    case DEBUG_GUI_CAM_TOP:
+        CameraDebug::setDebugType(CameraDebug::CM_DEBUG_TOP_OF_KART);
+        Camera::changeCamera(0, Camera::CM_TYPE_DEBUG);
         irr_driver->getDevice()->getCursorControl()->setVisible(true);
-    }
-    else if (cmdID == DEBUG_GUI_CAM_WHEEL)
-    {
-        Camera::setDebugMode(Camera::CM_DEBUG_GROUND);
+        break;
+    case DEBUG_GUI_CAM_WHEEL:
+        CameraDebug::setDebugType(CameraDebug::CM_DEBUG_GROUND);
+        Camera::changeCamera(0, Camera::CM_TYPE_DEBUG);
         irr_driver->getDevice()->getCursorControl()->setVisible(true);
-    }
-    else if (cmdID == DEBUG_GUI_CAM_BEHIND_KART)
-    {
-        Camera::setDebugMode(Camera::CM_DEBUG_BEHIND_KART);
+        break;
+    case DEBUG_GUI_CAM_BEHIND_KART:
+        CameraDebug::setDebugType(CameraDebug::CM_DEBUG_BEHIND_KART);
+        Camera::changeCamera(0, Camera::CM_TYPE_DEBUG);
         irr_driver->getDevice()->getCursorControl()->setVisible(true);
-    }
-    else if (cmdID == DEBUG_GUI_CAM_SIDE_OF_KART)
-    {
-        Camera::setDebugMode(Camera::CM_DEBUG_SIDE_OF_KART);
+        break;
+    case DEBUG_GUI_CAM_SIDE_OF_KART:
+        CameraDebug::setDebugType(CameraDebug::CM_DEBUG_SIDE_OF_KART);
+        Camera::changeCamera(0, Camera::CM_TYPE_DEBUG);
         irr_driver->getDevice()->getCursorControl()->setVisible(true);
-    }
-    else if (cmdID == DEBUG_GUI_CAM_FREE)
+        break;
+    case DEBUG_GUI_CAM_FREE:
     {
-        Camera::setDebugMode(Camera::CM_DEBUG_FPS);
+        Camera *camera = Camera::getActiveCamera();
+        Camera::changeCamera(camera->getIndex(), Camera::CM_TYPE_FPS);
         irr_driver->getDevice()->getCursorControl()->setVisible(false);
         // Reset camera rotation
-        Camera *cam = Camera::getActiveCamera();
-        cam->setDirection(vector3df(0, 0, 1));
-        cam->setUpVector(vector3df(0, 1, 0));
+        CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+        if(cam)
+        {
+            cam->setDirection(vector3df(0, 0, 1));
+            cam->setUpVector(vector3df(0, 1, 0));
+        }
+        break;
     }
-    else if (cmdID == DEBUG_GUI_CAM_NORMAL)
+    case DEBUG_GUI_CAM_NORMAL:
     {
-        Camera::setDebugMode(Camera::CM_DEBUG_NONE);
+        Camera *camera = Camera::getActiveCamera();
+        Camera::changeCamera(camera->getIndex(), Camera::CM_TYPE_NORMAL);
         irr_driver->getDevice()->getCursorControl()->setVisible(true);
+        break;
     }
-    else if (cmdID == DEBUG_GUI_CAM_SMOOTH)
+    case DEBUG_GUI_CAM_SMOOTH:
     {
-        Camera *cam = Camera::getActiveCamera();
-        cam->setSmoothMovement(!cam->getSmoothMovement());
+        CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+        if(cam)
+        {
+            cam->setSmoothMovement(!cam->getSmoothMovement());
+        }
+        break;
     }
-    else if (cmdID == DEBUG_GUI_CAM_ATTACH)
+    case DEBUG_GUI_CAM_ATTACH:
     {
-        Camera *cam = Camera::getActiveCamera();
-        cam->setAttachedFpsCam(!cam->getAttachedFpsCam());
+        CameraFPS *cam = dynamic_cast<CameraFPS*>(Camera::getActiveCamera());
+        if(cam)
+        {
+            cam->setAttachedFpsCam(!cam->getAttachedFpsCam());
+        }
+        break;
     }
-    else if (cmdID == DEBUG_VIEW_KART_ONE)
-    {
+    case DEBUG_VIEW_KART_ONE:
         changeCameraTarget(1);
-    }
-    else if (cmdID == DEBUG_VIEW_KART_TWO)
-    {
+        break;
+    case DEBUG_VIEW_KART_TWO:
         changeCameraTarget(2);
-    }
-    else if (cmdID == DEBUG_VIEW_KART_THREE)
-    {
+        break;
+    case DEBUG_VIEW_KART_THREE:
         changeCameraTarget(3);
-    }
-    else if (cmdID == DEBUG_VIEW_KART_FOUR)
-    {
+        break;
+    case DEBUG_VIEW_KART_FOUR:
         changeCameraTarget(4);
-    }
-    else if (cmdID == DEBUG_VIEW_KART_FIVE)
-    {
+        break;
+    case DEBUG_VIEW_KART_FIVE:
         changeCameraTarget(5);
-    }
-    else if (cmdID == DEBUG_VIEW_KART_SIX)
-    {
+        break;
+    case DEBUG_VIEW_KART_SIX:
         changeCameraTarget(6);
-    }
-    else if (cmdID == DEBUG_VIEW_KART_SEVEN)
-    {
+        break;
+    case DEBUG_VIEW_KART_SEVEN:
         changeCameraTarget(7);
-    }
-    else if (cmdID == DEBUG_VIEW_KART_EIGHT)
-    {
+        break;
+    case DEBUG_VIEW_KART_EIGHT:
         changeCameraTarget(8);
-    }
-    else if (cmdID == DEBUG_PRINT_START_POS)
-    {
+        break;
+    case DEBUG_PRINT_START_POS:
         if (!world) return false;
         for (unsigned int i = 0; i<world->getNumKarts(); i++)
         {
@@ -526,10 +539,9 @@ bool handleContextMenuAction(s32 cmdID)
                 kart->getXYZ().getZ(), kart->getHeading()*RAD_TO_DEGREE
                 );
         }
-    }
-    else if (cmdID == DEBUG_VISUAL_VALUES)
+        break;
+    case DEBUG_VISUAL_VALUES:
     {
-#if !defined(__APPLE__)
         DebugSliderDialog *dsd = new DebugSliderDialog();
         dsd->setSliderHook("red_slider", 0, 255,
             [](){ return int(irr_driver->getAmbientLight().r * 255.f); },
@@ -564,11 +576,10 @@ bool handleContextMenuAction(s32 cmdID)
             [](){ return int(irr_driver->getSSAOSigma() * 10.f); },
             [](int v){irr_driver->setSSAOSigma(v / 10.f); }
         );
-#endif
     }
-    else if (cmdID == DEBUG_ADJUST_LIGHTS)
+    break;
+    case DEBUG_ADJUST_LIGHTS:
     {
-#if !defined(__APPLE__)
         // Some sliders use multipliers because the spinner widget
         // only supports integers
         DebugSliderDialog *dsd = new DebugSliderDialog();
@@ -622,13 +633,88 @@ bool handleContextMenuAction(s32 cmdID)
             [](int v){        findNearestLight()->setRadius(float(v)); }
         );
         dsd->changeLabel("SSAO Sigma", "[None]");
+        break;
+    }
+    case DEBUG_SCRIPT_CONSOLE:
+        new GeneralTextFieldDialog(L"Run Script", []
+            (const irr::core::stringw& text) {},
+            [] (GUIEngine::LabelWidget* lw, GUIEngine::TextBoxWidget* tb)->bool
+            {
+                Scripting::ScriptEngine* engine =
+                    Scripting::ScriptEngine::getInstance();
+                if (engine == NULL)
+                {
+                    Log::warn("Debug", "No scripting engine loaded!");
+                    return true;
+                }
+                engine->evalScript(StringUtils::wideToUtf8(tb->getText()));
+                tb->setText(L"");
+                // Don't close the console after each run
+                return false;
+            });
+        break;
+    case DEBUG_RUN_CUTSCENE:
+        new GeneralTextFieldDialog(
+            L"Enter the cutscene names (separate parts by space)", []
+            (const irr::core::stringw& text)
+            {
+                if (World::getWorld())
+                {
+                    Log::warn("Debug", "Please run cutscene in main menu");
+                    return;
+                }
+                if (text.empty()) return;
+                std::vector<std::string> parts =
+                    StringUtils::split(StringUtils::wideToUtf8(text), ' ');
+                for (const std::string& track : parts)
+                {
+                    Track* t = track_manager->getTrack(track);
+                    if (t == NULL)
+                    {
+                        Log::warn("Debug", "Cutscene %s not found!",
+                            track.c_str());
+                        return;
+                    }
+                }
+                CutsceneWorld::setUseDuration(true);
+                StateManager::get()->enterGameState();
+                race_manager->setMinorMode(RaceManager::MINOR_MODE_CUTSCENE);
+                race_manager->setNumKarts(0);
+                race_manager->setNumPlayers(0);
+                race_manager->startSingleRace(parts.front(), 999, false);
+                ((CutsceneWorld*)World::getWorld())->setParts(parts);
+            });
+        break;
+        case DEBUG_TEXTURE_CONSOLE:
+        new GeneralTextFieldDialog(
+            L"Enter the texture filename(s) (separate names by ;)"
+            " to be reloaded (empty to reload all)\n"
+            "Press tus; for texture usage stats (shown in console)", []
+            (const irr::core::stringw& text) {},
+            [] (GUIEngine::LabelWidget* lw, GUIEngine::TextBoxWidget* tb)->bool
+            {
+#ifndef SERVER_ONLY
+                core::stringw t = tb->getText();
+                STKTexManager* stktm = STKTexManager::getInstance();
+                if (t == "tus;")
+                {
+                    stktm->dumpAllTexture(false/*mesh_texture*/);
+                    stktm->dumpTextureUsage();
+                    return false;
+                }
+                lw->setText(stktm->reloadTexture(t), true);
 #endif
-    }
-    else if (cmdID == DEBUG_SCRIPT_CONSOLE)
-    {
-        ScriptingConsole* console = new ScriptingConsole();
-    }
-
+                // Don't close the dialog after each run
+                return false;
+            });
+        break;
+        case DEBUG_START_RECORDING:
+            irr_driver->setRecording(true);
+        break;
+        case DEBUG_STOP_RECORDING:
+            irr_driver->setRecording(false);
+        break;
+    }   // switch
     return false;
 }
 
@@ -640,11 +726,15 @@ bool onEvent(const SEvent &event)
     if(!UserConfigParams::m_artist_debug_mode)
         return true;    // keep handling the events
 
-    if(event.EventType == EET_MOUSE_INPUT_EVENT)
+    if (event.EventType == EET_MOUSE_INPUT_EVENT)
     {
         // Create the menu (only one menu at a time)
-        if(event.MouseInput.Event == EMIE_RMOUSE_PRESSED_DOWN &&
-             !g_debug_menu_visible)
+        #ifdef ANDROID
+        if (event.MouseInput.X < 30 && event.MouseInput.Y < 30 &&
+        #else
+        if (event.MouseInput.Event == EMIE_RMOUSE_PRESSED_DOWN &&
+        #endif
+            !g_debug_menu_visible)
         {
             irr_driver->getDevice()->getCursorControl()->setVisible(true);
 
@@ -691,6 +781,7 @@ bool onEvent(const SEvent &event)
             sub->addItem(L"Bomb", DEBUG_ATTACHMENT_BOMB);
             sub->addItem(L"Anvil", DEBUG_ATTACHMENT_ANVIL);
             sub->addItem(L"Parachute", DEBUG_ATTACHMENT_PARACHUTE);
+            sub->addItem(L"Explosion", DEBUG_ATTACHMENT_EXPLOSION);
 
             mnu->addItem(L"GUI >",-1,true, true);
             sub = mnu->getSubMenu(3);
@@ -705,8 +796,13 @@ bool onEvent(const SEvent &event)
             sub->addItem(L"Toggle smooth camera", DEBUG_GUI_CAM_SMOOTH);
             sub->addItem(L"Attach fps camera to kart", DEBUG_GUI_CAM_ATTACH);
 
-            mnu->addItem(L"Change camera target >",-1,true, true);
+            mnu->addItem(L"Recording >",-1,true, true);
             sub = mnu->getSubMenu(4);
+            sub->addItem(L"Start recording", DEBUG_START_RECORDING);
+            sub->addItem(L"Stop recording", DEBUG_STOP_RECORDING);
+
+            mnu->addItem(L"Change camera target >",-1,true, true);
+            sub = mnu->getSubMenu(5);
             sub->addItem(L"To kart one", DEBUG_VIEW_KART_ONE);
             sub->addItem(L"To kart two", DEBUG_VIEW_KART_TWO);
             sub->addItem(L"To kart three", DEBUG_VIEW_KART_THREE);
@@ -716,12 +812,17 @@ bool onEvent(const SEvent &event)
             sub->addItem(L"To kart seven", DEBUG_VIEW_KART_SEVEN);
             sub->addItem(L"To kart eight", DEBUG_VIEW_KART_EIGHT);
 
+            mnu->addItem(L"Font >",-1,true, true);
+            sub = mnu->getSubMenu(6);
+            sub->addItem(L"Dump glyph pages of fonts", DEBUG_FONT_DUMP_GLYPH_PAGE);
+            sub->addItem(L"Reload all fonts", DEBUG_FONT_RELOAD);
+
             mnu->addItem(L"Adjust values", DEBUG_VISUAL_VALUES);
 
             mnu->addItem(L"Profiler", DEBUG_PROFILER);
             if (UserConfigParams::m_profiler_enabled)
-                mnu->addItem(L"Toggle capture profiler report",
-                             DEBUG_PROFILER_GENERATE_REPORT);
+                mnu->addItem(L"Save profiler report",
+                             DEBUG_PROFILER_WRITE_REPORT);
             mnu->addItem(L"Do not limit FPS", DEBUG_THROTTLE_FPS);
             mnu->addItem(L"Toggle FPS", DEBUG_FPS);
             mnu->addItem(L"Save replay", DEBUG_SAVE_REPLAY);
@@ -729,6 +830,8 @@ bool onEvent(const SEvent &event)
             mnu->addItem(L"Print position", DEBUG_PRINT_START_POS);
             mnu->addItem(L"Adjust Lights", DEBUG_ADJUST_LIGHTS);
             mnu->addItem(L"Scripting console", DEBUG_SCRIPT_CONSOLE);
+            mnu->addItem(L"Run cutscene(s)", DEBUG_RUN_CUTSCENE);
+            mnu->addItem(L"Texture console", DEBUG_TEXTURE_CONSOLE);
 
             g_debug_menu_visible = true;
             irr_driver->showPointer();
@@ -766,14 +869,19 @@ bool onEvent(const SEvent &event)
 
 bool handleStaticAction(int key)
 {
-    if (key == KEY_F1)
+    if (key == IRR_KEY_F1)
     {
         handleContextMenuAction(DEBUG_GUI_CAM_FREE);
     }
-    else if (key == KEY_F2)
+    else if (key == IRR_KEY_F2)
     {
         handleContextMenuAction(DEBUG_GUI_CAM_NORMAL);
     }
+    else if (key == IRR_KEY_F3)
+    {
+        STKTexManager::getInstance()->reloadTexture("");
+        return true;
+    }
     // TODO: create more keyboard shortcuts
 
     return false;
