@@ -81,3 +81,16 @@ void StkTime::getDate(int *day, int *month, int *year)
     if(month) *month = now->tm_mon + 1;
     if(year)  *year  = now->tm_year + 1900;
 }   // getDate
+
+StkTime::ScopeProfiler::ScopeProfiler(const char* name)
+{
+    Log::info("ScopeProfiler", "%s {\n", name);
+    m_time = (float)getRealTime();
+    m_name = name;
+}
+
+StkTime::ScopeProfiler::~ScopeProfiler()
+{
+    float f2 = (float)getRealTime();
+    Log::info("ScopeProfiler", "} // took %f s (%s)\n", (f2 - m_time), m_name.c_str());
+}
\ No newline at end of file
