@@ -53,7 +53,10 @@ private:
     std::map<const irr::core::stringw, const irr::core::stringw> m_fribidized_strings;
     bool m_rtl;
 
+    std::map<std::string, std::string> m_localized_name;
+
     std::string m_current_language_name;
+    std::string m_current_language_name_code;
 
 public:
                        Translations();
@@ -74,7 +77,13 @@ public:
 
     const std::vector<std::string>* getLanguageList() const;
 
-    std::string        getCurrentLanguageName();
+    std::set<wchar_t>        getCurrentAllChar();
+
+    std::string              getCurrentLanguageName();
+
+    std::string              getCurrentLanguageNameCode();
+
+    const std::string&       getLocalizedName(const std::string& str) const;
 
 private:
     irr::core::stringw fribidizeLine(const irr::core::stringw &str);
